@@ -3,7 +3,7 @@
 -- DgnTriggerers and triggerables:
 --
 -- This is similar to the observable/observer design pattern: a triggerable
--- class which does something and a triggerrer which sets it off.  As an
+-- class which does something and a triggerer which sets it off.  As an
 -- example, the ChangeFlags class (clua/lm_flags.lua), rather than having
 -- three subclasses (for monster death, feature change and item pickup)
 -- needs no subclasses, but is just one class which is given triggerers
@@ -11,7 +11,7 @@
 -- can be developed and used without have to update the ChangeFlags code.
 --
 -- Unlike with the observable/observer design pattern, each triggerer is
--- associated with a signle triggerable, rather than there being one observable
+-- associated with a single triggerable, rather than there being one observable
 -- and multiple observers, since each triggerer might have a data payload which
 -- is meant to be different for each triggerable.
 --
@@ -52,6 +52,8 @@
 -- events.  However, we don't need that much flexibility yet, so it's
 -- all handled inside of the Triggerable class.
 
+require('clua/lm_mslav.lua')
+
 Triggerable = { CLASS = "Triggerable" }
 Triggerable.__index = Triggerable
 
@@ -71,7 +73,7 @@ function Triggerable:new(props)
 end
 
 function Triggerable:unmangle(x)
-  if x and type(x) == 'function' then
+  if x and util.callable(x) then
     return x(self)
   else
     return x
@@ -95,9 +97,7 @@ function Triggerable:add_listener(listener)
   if not listener.func then
     error("listener has no func")
   end
-  if type(listener.func) ~= "function" then
-    error("listener.func isn't a function")
-  end
+  listener.func = global_function(listener.func, "triggerable listener")
   table.insert(self.listeners, listener)
 end
 
@@ -155,7 +155,7 @@ end
 
 function Triggerable:remove(marker)
   if self.removed then
-    crawl.mpr("ERROR: Trigerrable already removed", "error")
+    crawl.mpr("ERROR: Triggerable already removed", "error")
     return
   end
 
@@ -178,7 +178,7 @@ end
 
 function Triggerable:activate(marker)
   if self.removed then
-    error("Can't activate, trigerrable removed")
+    error("Can't activate, triggerable removed")
   end
 
   if #self.triggerers == 0 then
@@ -411,8 +411,7 @@ end
 -- A simple class to invoke an arbitrary Lua function.  Should be split out
 -- into own file if/when it becomes more complex.
 
-TriggerableFunction       = util.subclass(Triggerable) 
-TriggerableFunction.CLASS = "TriggerableFunction"
+util.subclass(Triggerable, "TriggerableFunction")
 
 function TriggerableFunction:new(pars)
   pars = pars or { }
@@ -421,11 +420,9 @@ function TriggerableFunction:new(pars)
 
   if not pars.func then
     error("Must provide func to TriggerableFunction")
-  elseif type(pars.func) ~= "function" then
-    error("TriggerableFunction func must be function, not " .. type(pars.msg))
   end
 
-  tf.func     = pars.func
+  tf.func     = global_function(pars.func, "triggerable function")
   tf.repeated = pars.repeated
   tf.data     = pars.data or {}
   tf.props    = pars.props or {}
@@ -444,7 +441,7 @@ end
 function TriggerableFunction:write(marker, th)
   TriggerableFunction.super.write(self, marker, th)
 
-  file.marshall(th, self.func)
+  lmark.marshall_table(th, self.func)
   file.marshall_meta(th, self.repeated)
   lmark.marshall_table(th, self.data)
 end
@@ -452,30 +449,37 @@ end
 function TriggerableFunction:read(marker, th)
   TriggerableFunction.super.read(self, marker, th)
 
-  self.func     = file.unmarshall_fn(th)
+  self.func     = lmark.unmarshall_table(th)
   self.repeated = file.unmarshall_meta(th)
   self.data     = lmark.unmarshall_table(th)
 
-  setmetatable(self, TriggerableFunction) 
+  setmetatable(self, TriggerableFunction)
 
   return self
 end
 
-function function_at_spot(func, data, repeated, props)
-  local tf = TriggerableFunction:new 
-      { func = func, data = data, repeated = repeated, props = props }
-
-  tf:add_triggerer( DgnTriggerer:new { type   = "player_move" } )
-
-  return tf
+function triggerable_function_constructor(trigger_type)
+  return function (func, data, repeated, props)
+           local tf = TriggerableFunction:new {
+             func = func,
+             data = data,
+             repeated = repeated,
+             props = props
+           }
+           tf:add_triggerer( DgnTriggerer:new { type = trigger_type } )
+           return tf
+         end
 end
 
+function_at_spot = triggerable_function_constructor('player_move')
+function_in_los  = triggerable_function_constructor('player_los')
+
 --------------------------
 
 -- A simple class to give out messages.  Should be split out into own
 -- file if/when it becomes more complex.
 
-TriggerableMessage       = util.subclass(Triggerable) 
+TriggerableMessage       = util.subclass(Triggerable)
 TriggerableMessage.CLASS = "TriggerableMessage"
 
 function TriggerableMessage:new(pars)
@@ -523,13 +527,13 @@ function TriggerableMessage:read(marker, th)
   self.channel  = file.unmarshall_string(th)
   self.repeated = file.unmarshall_meta(th)
 
-  setmetatable(self, TriggerableMessage) 
+  setmetatable(self, TriggerableMessage)
 
   return self
 end
 
 function message_at_spot(msg, channel, repeated, props)
-  local tm = TriggerableMessage:new 
+  local tm = TriggerableMessage:new
       { msg = msg, channel = channel, repeated = repeated, props = props }
 
   tm:add_triggerer( DgnTriggerer:new { type   = "player_move" } )
@@ -575,9 +579,8 @@ end
 -- * player_move: Wait for the player to move to a cell.  The
 --      triggerable/marker must be placed on top of cell in question.
 --
--- * player_los: Wait for the player to come into LOS of a cell, which
---      must contain a notable feature..  The triggerable/marker must be
---      placed on top of cell in question.
+-- * player_los: Wait for the player to come into LOS of a cell. The
+--      triggerable/marker must be placed on top of cell in question.
 --
 -- * turn: Called once for each player turn that passes.
 --
@@ -589,11 +592,15 @@ end
 --      missile (stones, etc).
 --
 -- * door_opened, door_closed: Called whenever doors are opened and closed by
---      the player, or whenever they are closed by monsters (monsters do not
---      open doors).
+--      the player, or whenever they are opened by monsters (monsters do not
+--      close doors).
+--
+-- * hp_warning: Called whenever a HP warning is triggered.
+--
+-- * pressure_plate: Called when someone (player or a monster) steps on a
+--      pressure plate trap.
 
-DgnTriggerer = { CLASS = "DgnTriggerer" }
-DgnTriggerer.__index = DgnTriggerer
+util.defclass("DgnTriggerer")
 
 function DgnTriggerer:new(pars)
   if not pars then
@@ -781,6 +788,14 @@ function DgnTriggerer:door_closed(triggerable, marker, ev)
   triggerable:do_trigger(self, marker, ev)
 end
 
+function DgnTriggerer:hp_warning(triggerable, marker, ev)
+  triggerable:do_trigger(self, marker, ev)
+end
+
+function DgnTriggerer:pressure_plate(triggerable, marker, ev)
+  triggerable:do_trigger(self, marker, ev)
+end
+
 function DgnTriggerer:turn(triggerable, marker, ev)
   if not self.countdown then
     triggerable:do_trigger(self, marker, ev)
@@ -818,16 +833,12 @@ end
 function DgnTriggerer:write(marker, th)
   -- Will always be "dgn_event", so we don't need to save it.
   self.method = nil
-
   lmark.marshall_table(th, self)
 end
 
 function DgnTriggerer:read(marker, th)
   local tr = lmark.unmarshall_table(th)
-
   setmetatable(tr, DgnTriggerer)
-
   tr:setup()
-
   return tr
 end
