@@ -1,8 +1,6 @@
 ------------------------------------------------------------------------------
 -- lm_trove.lua:
--- Custom (temporarily) timed, and tolled (gold or item) map marker.
---  Basically just a combination of TimedMarker and TollStair, with added veto
---  checks against items.
+-- Custom tolled (item) map marker.
 --
 -- Current only accepts the following:
 --  any armour
@@ -12,50 +10,27 @@
 --  any scroll
 --  any potion
 --  any wand
+--  runes and the horn of Geryon
 ------------------------------------------------------------------------------
 
-require('clua/lm_tmsg.lua')
 require('clua/lm_1way.lua')
 
 TroveMarker = util.subclass(OneWayStair)
 TroveMarker.CLASS = "TroveMarker"
 
 function TroveMarker:new(props)
-  -- On top of the normal Toll stair (the 'amount' value), this also accepts
-  -- secondary_amount and toll_item.
   props = props or { }
 
   local tmarker = self.super.new(self, props)
 
-  if not props.msg then
-    error("No messaging object provided (msg = nil)")
-  end
-
-  props.high = props.high or props.low or props.turns or 1
-  props.low  = props.low or props.high or props.turns or 1
-
-  local dur = crawl.random_range(props.low, props.high, props.navg or 1)
   if fnum == dgn.feature_number('unseen') then
     error("Bad feature name: " .. feat)
   end
 
-  tmarker.dur = dur * 10
-  tmarker.msg = props.msg
-
-  if not props.amount or props.amount < 1 then
-    error("Bad toll amount: " .. props.amount)
-  end
-
-  if not props.secondary_amount or props.secondary_amount < 1 then
-    error("Bad second toll amount: " .. props.secondary_amount)
-  end
-
   if not props.toll_item or props.toll_item == nil then
     error("Need a toll item.")
   end
 
-  tmarker.seen = false
-  tmarker.no_timeout = false
   item = props.toll_item
   if item.quantity == nil then
     error("Item needs a quantity.")
@@ -81,36 +56,30 @@ function TroveMarker:new(props)
 
   tmarker.toll_item = item
 
-  props.msg = nil
-
   return tmarker
 end
 
 function TroveMarker:activate(marker, verbose)
   self.super.activate(self, marker, verbose)
-  self.msg:init(self, marker, verbose)
-  dgn.register_listener(dgn.dgn_event_type('turn'), marker)
-  dgn.register_listener(dgn.dgn_event_type("v_leave_level"),
-                        marker, marker:pos())
-  dgn.register_listener(dgn.dgn_event_type("player_los"),
-                        marker, marker:pos())
 end
 
 function TroveMarker:fdesc_long (marker)
-  local state = ""
-  if self.no_timeout then
-    state = "This portal is broken and requires " .. self:item_name() .. " to function.\n"
+  local state
+  if self.props.toll_item.base_type == "miscellaneous" then
+    state = "This portal requires the presence of " ..
+            self:item_name() .. " to function.\n"
   else
-    state = "This portal will remain here for a" .. self.props.dur .. " time.\n"
+    state = "This portal needs " ..
+            self:item_name() .. " to function.\n"
   end
   return state .. "\n" .. self.props.desc_long
 end
 
 function TroveMarker:overview_note (marker)
-  if self.no_timeout then
-    return self:item_name(false)
+  if self.props.toll_item.base_type == "miscellaneous" then
+    return "show " .. self:item_name(false)
   else
-    return "" .. self.props.secondary_amount .. " gold/" .. self.props.amount .. " gold"
+    return "give " .. self:item_name(false)
   end
 end
 
@@ -209,62 +178,6 @@ function TroveMarker:property(marker, pname)
   return self.super.property(self, marker, pname)
 end
 
-function TroveMarker:disappear(marker, affect_player, x, y)
-  dgn.remove_listener(marker)
-  self.super.disappear(self, marker, affect_player, x, y)
-end
-
-function TroveMarker:timeout(marker, verbose, affect_player)
-  local x, y = marker:pos()
-
-  if you.pos() == marker:pos() and you.taking_stairs() then
-    if verbose then
-      crawl.mpr( dgn.feature_desc_at(x, y, "The") .. " vanishes " ..
-                "just as you enter it!")
-      return
-    end
-  end
-
-  if verbose then
-    if you.see_cell(marker:pos()) then
-      crawl.mpr( util.expand_entity(self.props.entity, self.props.disappear) or
-                 dgn.feature_desc_at(x, y, "The") .. " disappears!")
-    else
-      crawl.mpr("The walls and floor vibrate strangely for a moment.")
-    end
-  end
-
-  self:disappear(marker, affect_player, x, y)
-end
-
-function TroveMarker:event(marker, ev)
-  local x, y = ev:pos()
-  if ev:type() == dgn.dgn_event_type('player_los') and self.seen ~= true then
-    self.seen = true
-  end
-
-  if self.super.event(self, marker, ev) then
-    return true
-  end
-
-  self.ticktype = self.ticktype or dgn.dgn_event_type('turn')
-
-  if ev:type() ~= self.ticktype then
-    return
-  end
-
-  if self.no_timeout then
-    return
-  end
-
-  self.dur = self.dur - ev:ticks()
-  self.msg:event(self, marker, ev)
-  if self.dur <= 0 then
-    self:timeout(marker, true, true)
-    return true
-  end
-end
-
 function TroveMarker:describe(marker)
   local desc = self:unmangle(self.props.desc_long)
   if desc then
@@ -272,16 +185,12 @@ function TroveMarker:describe(marker)
   else
     desc = ""
   end
-  return desc .. "The portal charges " .. self.props.amount .. " for entry.\n"
+  return desc .. "The portal requires " .. self.item:name() .. " for entry.\n"
 end
 
 function TroveMarker:read(marker, th)
   TroveMarker.super.read(self, marker, th)
-  self.seen = file.unmarshall_meta(th)
-  self.no_timeout = file.unmarshall_meta(th)
   self.toll_item = lmark.unmarshall_table(th)
-  self.dur  = file.unmarshall_number(th)
-  self.msg  = file.unmarshall_fn(th)(th)
 
   setmetatable(self, TroveMarker)
   return self
@@ -289,12 +198,7 @@ end
 
 function TroveMarker:write(marker, th)
   TroveMarker.super.write(self, marker, th)
-  file.marshall_meta(th, self.seen)
-  file.marshall_meta(th, self.no_timeout)
   lmark.marshall_table(th, self.toll_item)
-  file.marshall(th, self.dur)
-  file.marshall(th, self.msg.read)
-  self.msg:write(th)
 end
 
 function TroveMarker:item_name(do_grammar)
@@ -304,6 +208,22 @@ function TroveMarker:item_name(do_grammar)
     s = s .. item.quantity
   end
 
+  if item.sub_type == "rune of Zot" then
+    if do_grammar == false then
+      return item.ego_type .. " rune of Zot"
+    else
+      return crawl.grammar(item.ego_type .. " rune of Zot", "a")
+    end
+  end
+
+  if item.sub_type == "horn of Geryon" then
+    if do_grammar == false then
+      return "horn of Geryon"
+    else
+      return "the horn of Geryon"
+    end
+  end
+
   if item.artefact_name ~= false then
     if string.find(item.artefact_name, "'s") or do_grammar == false then
       return item.artefact_name
@@ -334,7 +254,8 @@ function TroveMarker:item_name(do_grammar)
   local jwith_pluses = {"ring of protection", "ring of evasion",
                         "ring of strength", "ring of intelligence",
                         "ring of dexterity", "ring of slaying"}
-  if item.base_type == "jewellery" and util.contains(jwith_pluses, item.sub_type) then
+  if item.base_type == "jewellery" and
+     util.contains(jwith_pluses, item.sub_type) then
     s = s .. " +" .. item.plus1
     if item.sub_type == "ring of slaying" and item.plus2 ~= item.plus1 then
       s = s .. ", +" .. item.plus2
@@ -348,8 +269,10 @@ function TroveMarker:item_name(do_grammar)
       s = s .. " " .. item.base_type .. " of"
     end
   elseif item.base_type == "book" then
-    books = {"Necronomicon", "tome of Destruction", "Young Poisoner's Handbook", "Monster Manual"}
-    if not string.find(item.sub_type, "manual") and not util.contains(books, item.sub_type) then
+    books = {"Necronomicon", "tome of Destruction",
+             "Young Poisoner's Handbook", "Monster Manual"}
+    if not string.find(item.sub_type, "manual")
+       and not util.contains(books, item.sub_type) then
       s = s .. " book of"
     end
   elseif item.base_type == "wand" then
@@ -386,9 +309,9 @@ function TroveMarker:check_item(marker, pname, position, dry_run)
   if position == "inventory" then
     if dry_run ~= nil then crawl.mpr("Checking inventory.") end
     iter_table = items.inventory()
-  else
-    if dry_run ~= nil then crawl.mpr("Checking " .. position.x .. "/" .. position.y) end
-    iter_table = dgn.items_at(position.x, position.y)
+  --else
+  --  if dry_run ~= nil then crawl.mpr("Checking " .. position.x .. "/" .. position.y) end
+  --  iter_table = dgn.items_at(position.x, position.y)
   end
 
   if #iter_table == 0 then
@@ -401,6 +324,12 @@ function TroveMarker:check_item(marker, pname, position, dry_run)
   for it in iter.invent_iterator:new(iter_table) do
     local iplus1, iplus2 = it.pluses()
     local this_item = true
+    -- For misc items we check plus1 and nothing else.
+    if it.base_type == "miscellaneous" then
+      if iplus1 ~= false and item.plus1 ~= false and iplus1 ~= item.plus1 then
+        this_item = false
+      end
+    else
     if dry_run ~= nil then crawl.mpr("Checking item: " .. it.name()) end
 
     if not it.identified("type properties pluses") then
@@ -471,9 +400,11 @@ function TroveMarker:check_item(marker, pname, position, dry_run)
         this_item = false
       end
     end
-
-    -- Now all we need to do is to make sure that the item is the one we're looking for
-    if this_item and item.sub_type == it.sub_type and item.base_type == it.base_type then
+    end
+    -- Now all we need to do is to make sure that the item
+    -- is the one we're looking for
+    if this_item and item.sub_type == it.sub_type
+       and item.base_type == it.base_type then
        if dry_run ~= nil then crawl.mpr("Accepting " .. it.name() .. " and added to queue.") end
       table.insert(acceptable_items, it)
     elseif this_item then
@@ -492,7 +423,8 @@ function TroveMarker:check_item(marker, pname, position, dry_run)
 
   -- If there are no pluses but multiple acceptable items, or there is only one
   -- acceptable item, take the first in the list.
-  if #acceptable_items == 1 or (item.plus1 == false and item.plus2 == false) then
+  if #acceptable_items == 1
+     or (item.plus1 == false and item.plus2 == false) then
     local it = acceptable_items[1]
     if dry_run == nil then
       return self:accept_item(it)
@@ -514,12 +446,14 @@ function TroveMarker:check_item(marker, pname, position, dry_run)
       if this_p1 < titem_p1 and (item.plus2 == false or this_p2 < titem_p2) then
         titem = it
         titem_p1, titem_p2 = titem.pluses()
-        --crawl.mpr("Picking " .. titem.name() .. " instead (lesser pluses instad)")
+        --crawl.mpr("Picking " .. titem.name() ..
+        --          " instead (lesser pluses instad)")
         --crawl.mpr("This item p1: " .. titem_p1 .. ", p2: " .. titem_p2)
       elseif this_p1 == item.plus1 and this_p2 == item.plus2 then
         titem = it
         titem_p1, titem_p2 = titem.pluses()
-        --crawl.mpr("Picking " .. titem.name() .. " instead (matches wanted pluses)")
+        --crawl.mpr("Picking " .. titem.name() ..
+        --          " instead (matches wanted pluses)")
         --crawl.mpr("This item p1: " .. titem_p1 .. ", p2: " .. titem_p2)
       end
     end
@@ -531,8 +465,13 @@ function TroveMarker:check_item(marker, pname, position, dry_run)
 end
 
 function TroveMarker:accept_item (it)
-  crawl.mpr("The portal accepts the item" .. self:plural() .. " and buzzes to life!")
-  it.dec_quantity(self.props.toll_item.quantity)
+  -- We don't take misc items away from people.
+  if it.base_type == "miscellaneous" then
+    crawl.mpr("The portal draws power from the presence of the item" .. self:plural() .. " and buzzes to life!")
+  else
+    crawl.mpr("The portal accepts the item" .. self:plural() .. " and buzzes to life!")
+    it.dec_quantity(self.props.toll_item.quantity)
+  end
   return true
 end
 
@@ -545,88 +484,49 @@ function TroveMarker:plural ()
 end
 
 function TroveMarker:check_veto(marker, pname)
-  -- Have we enough gold?
-  local gold = you.gold()
-  local needed = self.props.amount
-  local needed2 = self.props.secondary_amount
   local quantity = false
   local plus = 0
 
-  if gold < needed2 and self.no_timeout == false then
-    crawl.mpr("This portal charges a minimum of " .. needed2 .. " gold for entry; " ..
-              "you have only " .. gold .. " gold.")
-    return "veto"
-  end
-
   local _x, _y = marker:pos()
 
-  if pname == "veto_stair" then
-    -- Let's see if they want to pay the smaller amount
-    if self.no_timeout == true then
-      -- Let's check to see if there's an item at our current position
-      if crawl.yesno("This trove needs " .. self:item_name() .. " to function. Give it the item" .. self:plural() .. "?", true, "n") then
-        if self:check_item(marker, pname, dgn.point(_x, _y)) == true then
-          return
-        else
-          if self:check_item(marker, pname, "inventory") == true then
-            self:note_payed(self:item_name())
-            return
-          else
-            crawl.mpr("You don't have the item" .. self:plural() .. " to give!")
-            return "veto"
-          end
-        end
-      end
-      return "veto"
+  -- The message is slightly different for items that aren't actually taken by the trove (currently misc items).
+  if self.props.toll_item.base_type == "miscellaneous" then
+    if crawl.yesno("This trove requires the presence of " .. self:item_name() ..
+                 " to function. Show it the item" ..
+                 self:plural() .. "?", true, "n") then
+    if self:check_item(marker, pname, "inventory") == true then
+      return
     else
-      crawl.mpr("There is a foul-smelling imp just inside the portal. It says, " ..
-                "\"I can lock this portal here for " .. needed2 .."gp!\".")
-      if crawl.yesno("Pay? (you can refuse and instead pay the full fee of " .. needed .. "gp to enter now)", true, "n") then
-        self.no_timeout = true
-        you.gold(you.gold() - needed2)
-        self:note_payed(needed2 .. "gp", true)
-        crawl.mpr("The imp fiddles with the portal and frowns. It says, \"I think I broke it! You might be able to fix it with " .. self:item_name() .. ".\" before disappearing in a puff of smoke.")
-        return "veto"
-      else
-        crawl.mpr("The foul-smelling imp curses your mother!")
-      end
-
-      -- Ok, ask if the player wants to spend the $$$.
-      if crawl.yesno("You can still enter the portal, but it charges " .. needed ..
-                         "gp for entry. Pay?", true, "n") then
-        if you.gold() < needed then
-          crawl.mpr("You don't have the money to pay for it!")
-          return "veto"
-        else
-          you.gold(you.gold() - needed)
-          self:note_payed(needed .. "gp")
-        end
-      else
-        return "veto"
-      end
+      crawl.mpr("You don't have " .. self:item_name() ..
+                " with you.")
+      return "veto"
     end
   end
+  else
+  if crawl.yesno("This trove needs " .. self:item_name() ..
+                 " to function. Give it the item" ..
+                 self:plural() .. "?", true, "n") then
+    if self:check_item(marker, pname, "inventory") == true then
+      return
+    else
+      crawl.mpr("You don't have the item" .. self:plural() ..
+                " to give! Perhaps you haven't completely identified the item yet?")
+      return "veto"
+    end
+  end
+  end
+  return "veto"
 end
 
-function TroveMarker:note_payed(amount, lock)
+function TroveMarker:note_payed(name)
   local toll_desc
   if self.props.toll_desc then
-      toll_desc = self.props.toll_desc
+    toll_desc = self.props.toll_desc
   else
-      toll_desc = "at " .. crawl.article_a(self.props.desc)
+    toll_desc = "at " .. crawl.article_a(self.props.desc)
   end
 
-  local tdesc = "toll"
-
-  if lock ~= nil then
-    tdesc = "fee"
-    if self.props.lock_desc then
-      toll_desc = self.props.lock_desc
-    end
-  end
-
-
-  crawl.take_note("You paid a " .. tdesc .. " of " .. amount .. " " ..
+  crawl.take_note("You paid a toll of " .. name .. " " ..
                   toll_desc .. ".")
 end
 
