@@ -27,14 +27,16 @@ function zig()
   return dgn.persist.ziggurat
 end
 
+function callback.ziggurat_onclimb()
+  crawl.mark_milestone("zig.exit",
+                       "left a Ziggurat at level " ..
+                         zig().depth .. ".")
+  dgn.persist.ziggurat = { }
+end
+
 function cleanup_ziggurat()
   return one_way_stair {
-    onclimb = function()
-                crawl.mark_milestone("zig.exit",
-                                     "left a Ziggurat at level " ..
-                                       zig().depth .. ".")
-                dgn.persist.ziggurat = { }
-              end,
+    onclimb = "callback.ziggurat_onclimb",
     dstplace = zig().origin_level
   }
 end
@@ -52,6 +54,13 @@ function initialise_ziggurat(z, portal)
     z.portal = portal.props
   end
 
+  if not z.depth then
+    -- At this point we're still on the parent level (the level
+    -- containing the ziggurat entrance), barring wizmoded ziggurat
+    -- portals, which aren't our target demographic anyway.
+    crawl.mark_milestone("zig.enter", "entered a Ziggurat.")
+  end
+
   z.depth = 1
 
   -- Any given ziggurat will use the same builder for all its levels,
@@ -69,7 +78,7 @@ function initialise_ziggurat(z, portal)
   z.origin_level = dgn.level_name(dgn.level_id())
 end
 
-function ziggurat_initialiser(portal)
+function callback.ziggurat_initialiser(portal)
   -- First ziggurat will be initialised twice.
   initialise_ziggurat(zig(), portal)
 end
@@ -80,7 +89,7 @@ end
 
 -- Increments the depth in the ziggurat when the player takes a
 -- downstair in the ziggurat.
-function zig_depth_increment()
+function callback.zig_depth_increment()
   zig().depth = zig().depth + 1
   zig().level = { }
 end
@@ -108,7 +117,7 @@ function ziggurat_portal(e, portal_only)
       dstname_abbrev = "Zig:1",
       dstorigin = "on level 1 of a ziggurat",
       floor = "stone_arch",
-      onclimb = ziggurat_initialiser
+      onclimb = "callback.ziggurat_initialiser"
     }
   end
 
@@ -153,7 +162,7 @@ function ziggurat_build_level(e)
   -- Deeper levels can have all monsters awake.
   -- Does never happen at depths 1-4; does always happen at depths 25-27.
   local generate_awake = depth > 4 + crawl.random2(21)
-  zig().monster_hook = generate_awake and ziggurat_awaken_all
+  zig().monster_hook = generate_awake and global_function("ziggurat_awaken_all")
 
   -- Deeper levels may block controlled teleports.
   -- Does never happen at depths 1-6; does always happen at depths 25-27.
@@ -172,7 +181,7 @@ local zigstair = dgn.gridmark
 local function zig_go_deeper()
   local newdepth = zig().depth + 1
   return one_way_stair {
-    onclimb = zig_depth_increment,
+    onclimb = "callback.zig_depth_increment",
     dstname = "Ziggurat:" .. newdepth,
     dstname_abbrev = "Zig:" .. newdepth,
     dstorigin = "on level " .. newdepth .. " of a ziggurat"
@@ -319,14 +328,33 @@ end
 mset(with_props("place:Slime:$", { jelly_protect = true }),
      "place:Snake:$",
      with_props("place:Lair:$", { weight = 5 }),
+     "place:Spider:$ w:50 / ghost moth / red wasp / tarantella",
      "place:Crypt:$",
+     "place:Dwarf:$",
      "place:Abyss",
      with_props("place:Shoals:$", { weight = 5 }),
      with_props("place:Coc:$", { weight = 5 }),
      with_props("place:Geh:$", { weight = 5 }),
      with_props("place:Dis:$", { weight = 5 }),
      with_props("place:Tar:$", { weight = 5 }),
-     with_props("daeva / angel", { weight = 2 }))
+     with_props("daeva / angel", { weight = 2 }),
+     with_props("hill giant / cyclops / stone giant / fire giant / " ..
+                "frost giant / titan", { weight = 2 }),
+     with_props("fire elemental / fire drake / hell hound / efreet / " ..
+                "dragon / fire giant / orb of fire", { weight = 2 }),
+     with_props("ice beast / polar bear / freezing wraith / ice dragon / " ..
+                "frost giant / ice devil / ice fiend / place:D:$ simulacrum", { weight = 2 }),
+     with_props("insubstantial wisp / air elemental / vapour / titan / " ..
+                "storm dragon / electric golem", { weight = 2 }),
+     with_props("clay golem / earth elemental / stone golem / iron golem / " ..
+                "crystal golem / stone giant / iron dragon", { weight = 1 }),
+     with_props("swamp drake / fire drake / death drake / steam dragon / " ..
+                "swamp dragon / dragon / ice dragon / storm dragon / " ..
+                "iron dragon / shadow dragon / quicksilver dragon / " ..
+                "golden dragon / mottled dragon / wyvern / hydra", { weight = 2 }),
+     with_props("centaur / yaktaur / centaur warrior / yaktaur captain / " ..
+                "cyclops / stone giant / merfolk javelineer / " ..
+                "deep elf master archer", { weight = 2 }))
 
 -- spec_fn can be used to wrap a function that returns a monster spec.
 -- This is useful if you want to adjust monster weights in the spec
