@@ -1,8 +1,7 @@
-/*
- *  File:       dbg-scan.cc
- *  Summary:    Debugging code to scan the list of items and monsters.
- *  Written by: Linley Henzell and Jesse Jones
- */
+/**
+ * @file
+ * @brief Debugging code to scan the list of items and monsters.
+**/
 
 #include "AppHdr.h"
 
@@ -20,21 +19,21 @@
 #include "mon-util.h"
 #include "state.h"
 
-#define DEBUG_ITEM_SCAN 1
-#if DEBUG_ITEM_SCAN
-static void _dump_item( const char *name, int num, const item_def &item )
+#define DEBUG_ITEM_SCAN
+#ifdef DEBUG_ITEM_SCAN
+static void _dump_item(const char *name, int num, const item_def &item)
 {
     mpr(name, MSGCH_ERROR);
 
-    mprf("    item #%d:  base: %d; sub: %d; plus: %d; plus2: %d; special: %ld",
+    mprf("    item #%d:  base: %d; sub: %d; plus: %d; plus2: %d; special: %d",
          num, item.base_type, item.sub_type,
-         item.plus, item.plus2, item.special );
+         item.plus, item.plus2, item.special);
 
-    mprf("    quant: %d; colour: %d; ident: 0x%08lx; ident_type: %d",
+    mprf("    quant: %d; colour: %d; ident: 0x%08"PRIx64"; ident_type: %d",
          item.quantity, item.colour, item.flags,
-         get_ident_type( item ) );
+         get_ident_type(item));
 
-    mprf("    x: %d; y: %d; link: %d", item.pos.x, item.pos.y, item.link );
+    mprf("    x: %d; y: %d; link: %d", item.pos.x, item.pos.y, item.link);
 
     crawl_state.cancel_cmd_repeat();
 }
@@ -44,7 +43,7 @@ static void _dump_item( const char *name, int num, const item_def &item )
 // debug_item_scan
 //
 //---------------------------------------------------------------
-void debug_item_scan( void )
+void debug_item_scan(void)
 {
     int   i;
     char  name[256];
@@ -81,11 +80,11 @@ void debug_item_scan( void )
             }
 
             // Check for invalid (zero quantity) items that are linked in.
-            if (!mitm[obj].is_valid())
+            if (!mitm[obj].defined())
             {
                 mprf(MSGCH_ERROR, "Linked invalid item at (%d,%d)!",
                      ri->x, ri->y);
-                _dump_item( mitm[obj].name(DESC_PLAIN).c_str(), obj, mitm[obj] );
+                _dump_item(mitm[obj].name(DESC_PLAIN).c_str(), obj, mitm[obj]);
             }
 
             // Check that item knows what stack it's in.
@@ -93,8 +92,8 @@ void debug_item_scan( void )
             {
                 mprf(MSGCH_ERROR,"Item position incorrect at (%d,%d)!",
                      ri->x, ri->y);
-                _dump_item( mitm[obj].name(DESC_PLAIN).c_str(),
-                            obj, mitm[obj] );
+                _dump_item(mitm[obj].name(DESC_PLAIN).c_str(),
+                            obj, mitm[obj]);
             }
 
             // If we run into a premarked item we're in real trouble,
@@ -112,29 +111,29 @@ void debug_item_scan( void )
     // Now scan all the items on the level:
     for (i = 0; i < MAX_ITEMS; ++i)
     {
-        if (!mitm[i].is_valid())
+        if (!mitm[i].defined())
             continue;
 
-        strcpy(name, mitm[i].name(DESC_PLAIN).c_str());
+        strlcpy(name, mitm[i].name(DESC_PLAIN).c_str(), sizeof(name));
 
-        const monsters* mon = mitm[i].holding_monster();
+        const monster* mon = mitm[i].holding_monster();
 
         // Don't check (-1, -1) player items or (-2, -2) monster items
         // (except to make sure that the monster is alive).
         if (mitm[i].pos.origin())
         {
             mpr("Unlinked temporary item:", MSGCH_ERROR);
-            _dump_item( name, i, mitm[i] );
+            _dump_item(name, i, mitm[i]);
         }
         else if (mon != NULL && mon->type == MONS_NO_MONSTER)
         {
             mpr("Unlinked item held by dead monster:", MSGCH_ERROR);
-            _dump_item( name, i, mitm[i] );
+            _dump_item(name, i, mitm[i]);
         }
         else if ((mitm[i].pos.x > 0 || mitm[i].pos.y > 0) && !visited[i])
         {
             mpr("Unlinked item:", MSGCH_ERROR);
-            _dump_item( name, i, mitm[i] );
+            _dump_item(name, i, mitm[i]);
 
             if (!in_bounds(mitm[i].pos))
             {
@@ -144,7 +143,7 @@ void debug_item_scan( void )
             else
             {
                 mprf("igrd(%d,%d) = %d",
-                     mitm[i].pos.x, mitm[i].pos.y, igrd( mitm[i].pos ));
+                     mitm[i].pos.x, mitm[i].pos.y, igrd(mitm[i].pos));
             }
 
             // Let's check to see if it's an errant monster object:
@@ -172,63 +171,63 @@ void debug_item_scan( void )
         //   -- items described as buggy (typically adjectives out of range)
         //      (note: covers buggy, bugginess, buggily, whatever else)
         //
-        if (strstr( name, "questionable" ) != NULL
-            || strstr( name, "eggplant" ) != NULL
-            || strstr( name, "bola" ) != NULL
-            || strstr( name, "bugg" ) != NULL)
+        if (strstr(name, "questionable") != NULL
+            || strstr(name, "eggplant") != NULL
+            || strstr(name, "bola") != NULL
+            || strstr(name, "bugg") != NULL)
         {
             mpr("Bad item:", MSGCH_ERROR);
-            _dump_item( name, i, mitm[i] );
+            _dump_item(name, i, mitm[i]);
         }
         else if ((mitm[i].base_type == OBJ_WEAPONS
                     && (abs(mitm[i].plus) > 30
                         || abs(mitm[i].plus2) > 30
-                        || !is_artefact( mitm[i] )
+                        || !is_artefact(mitm[i])
                            && mitm[i].special >= NUM_SPECIAL_WEAPONS))
 
                  || (mitm[i].base_type == OBJ_MISSILES
                      && (abs(mitm[i].plus) > 25
-                         || !is_artefact( mitm[i] )
+                         || !is_artefact(mitm[i])
                             && mitm[i].special >= NUM_SPECIAL_MISSILES))
 
                  || (mitm[i].base_type == OBJ_ARMOUR
                      && (abs(mitm[i].plus) > 25
-                         || !is_artefact( mitm[i] )
+                         || !is_artefact(mitm[i])
                             && mitm[i].special >= NUM_SPECIAL_ARMOURS)))
         {
             mpr("Bad plus or special value:", MSGCH_ERROR);
-            _dump_item( name, i, mitm[i] );
+            _dump_item(name, i, mitm[i]);
         }
         else if (mitm[i].flags & ISFLAG_SUMMONED
                  && in_bounds(mitm[i].pos))
         {
             mpr("Summoned item on floor:", MSGCH_ERROR);
-            _dump_item( name, i, mitm[i] );
+            _dump_item(name, i, mitm[i]);
         }
     }
 
     // Quickly scan monsters for "program bug"s.
     for (i = 0; i < MAX_MONSTERS; ++i)
     {
-        const monsters& monster = menv[i];
+        const monster& mons = menv[i];
 
-        if (monster.type == MONS_NO_MONSTER)
+        if (mons.type == MONS_NO_MONSTER)
             continue;
 
-        if (monster.name(DESC_PLAIN, true).find("questionable") !=
+        if (mons.name(DESC_PLAIN, true).find("questionable") !=
             std::string::npos)
         {
             mprf(MSGCH_ERROR, "Program bug detected!");
             mprf(MSGCH_ERROR,
                  "Buggy monster detected: monster #%d; position (%d,%d)",
-                 i, monster.pos().x, monster.pos().y);
+                 i, mons.pos().x, mons.pos().y);
         }
     }
 }
 #endif
 
-#define DEBUG_MONS_SCAN 1
-#if DEBUG_MONS_SCAN
+#define DEBUG_MONS_SCAN
+#ifdef DEBUG_MONS_SCAN
 static void _announce_level_prob(bool warned)
 {
     if (!warned && Generating_Level)
@@ -252,9 +251,9 @@ static std::vector<std::string> _in_vaults(const coord_def &pos)
 {
     std::vector<std::string> out;
 
-    for (unsigned int i = 0; i < Level_Vaults.size(); ++i)
+    for (unsigned int i = 0; i < env.level_vaults.size(); ++i)
     {
-        const vault_placement &vault = Level_Vaults[i];
+        const vault_placement &vault = *env.level_vaults[i];
         if (_inside_vault(vault, pos))
             out.push_back(vault.map.name);
     }
@@ -290,7 +289,7 @@ void debug_mons_scan()
                 continue;
             }
 
-            const monsters *m = &menv[mons];
+            const monster* m = &menv[mons];
             const coord_def pos(x, y);
             if (m->pos() != pos)
             {
@@ -317,6 +316,9 @@ void debug_mons_scan()
             }
         }
 
+    ASSERT(you.type == MONS_PLAYER);
+    ASSERT(you.mid == MID_PLAYER);
+
     std::vector<int> floating_mons;
     bool             is_floating[MAX_MONSTERS];
 
@@ -324,10 +326,11 @@ void debug_mons_scan()
     {
         is_floating[i] = false;
 
-        const monsters *m = &menv[i];
+        const monster* m = &menv[i];
         if (!m->alive())
             continue;
 
+        ASSERT(m->mid > 0);
         coord_def pos = m->pos();
 
         if (!in_bounds(pos))
@@ -352,7 +355,7 @@ void debug_mons_scan()
                 if (i == j)
                     continue;
 
-                const monsters *m2 = &menv[j];
+                const monster* m2 = &menv[j];
 
                 if (m2->pos() != m->pos())
                     continue;
@@ -388,7 +391,7 @@ void debug_mons_scan()
             }
             item_def &item(mitm[idx]);
 
-            if (!item.is_valid())
+            if (!item.defined())
             {
                 _announce_level_prob(warned);
                 warned = true;
@@ -399,7 +402,7 @@ void debug_mons_scan()
                 continue;
             }
 
-            const monsters* holder = item.holding_monster();
+            const monster* holder = item.holding_monster();
 
             if (holder == NULL)
             {
@@ -409,8 +412,8 @@ void debug_mons_scan()
                                  "item (midx = %d)",
                      m->full_name(DESC_PLAIN, true).c_str(),
                      pos.x, pos.y, i);
-                _dump_item( item.name(DESC_PLAIN, false, true).c_str(),
-                            idx, item );
+                _dump_item(item.name(DESC_PLAIN, false, true).c_str(),
+                            idx, item);
                 continue;
             }
 
@@ -458,7 +461,7 @@ void debug_mons_scan()
     }
 
     // No vaults to report on?
-    if (Level_Vaults.size() == 0 && Temp_Vaults.size() == 0)
+    if (env.level_vaults.empty() && Temp_Vaults.empty())
     {
         mpr("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", MSGCH_ERROR);
         // Force the dev to notice problems. :P
@@ -471,7 +474,7 @@ void debug_mons_scan()
     for (unsigned int i = 0; i < floating_mons.size(); ++i)
     {
         const int       idx = floating_mons[i];
-        const monsters* mon = &menv[idx];
+        const monster* mon = &menv[idx];
         std::vector<std::string> vaults = _in_vaults(mon->pos());
 
         std::string str =
@@ -494,7 +497,7 @@ void debug_mons_scan()
     {
         const coord_def pos = bogus_pos[i];
         const int       idx = bogus_idx[i];
-        const monsters* mon = &menv[idx];
+        const monster* mon = &menv[idx];
 
         std::string str =
             make_stringf("Bogus mgrd (%d, %d) pointing to %s",
