@@ -1,8 +1,7 @@
-/*
- *  File:       dgn-overview.cc
- *  Summary:    Records location of stairs etc
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Records location of stairs etc
+**/
 
 #include "AppHdr.h"
 
@@ -22,15 +21,17 @@
 #include "dgnevent.h"
 #include "directn.h"
 #include "dungeon.h"
-#include "map_knowledge.h"
+#include "env.h"
 #include "feature.h"
 #include "files.h"
+#include "map_knowledge.h"
 #include "menu.h"
+#include "message.h"
 #include "religion.h"
 #include "shopping.h"
 #include "state.h"
 #include "stuff.h"
-#include "env.h"
+#include "tagstring.h"
 #include "terrain.h"
 #include "travel.h"
 
@@ -40,10 +41,7 @@ typedef std::map<level_pos, god_type> altar_map_type;
 typedef std::map<level_pos, portal_type> portal_map_type;
 typedef std::map<level_pos, std::string> portal_vault_map_type;
 typedef std::map<level_pos, std::string> portal_note_map_type;
-// NOTE: The value for colours here is char rather than unsigned char
-// because g++ needs it to be char for marshallMap in tags.cc to
-// compile properly.
-typedef std::map<level_pos, char> portal_vault_colour_map_type;
+typedef std::map<level_pos, uint8_t> portal_vault_colour_map_type;
 typedef std::map<level_id, std::string> annotation_map_type;
 
 stair_map_type stair_level;
@@ -70,22 +68,37 @@ static std::string _get_notes();
 static std::string _print_altars_for_gods(const std::vector<god_type>& gods,
                                           bool print_unseen, bool display);
 
+void overview_clear()
+{
+    stair_level.clear();
+    shops_present.clear();
+    altars_present.clear();
+    portals_present.clear();
+    portal_vaults_present.clear();
+    portal_vault_notes.clear();
+    portal_vault_colours.clear();
+    level_annotations.clear();
+    level_exclusions.clear();
+}
+
 void seen_notable_thing(dungeon_feature_type which_thing, const coord_def& pos)
 {
-    // Tell the world first.
-    dungeon_events.fire_position_event(DET_PLAYER_IN_LOS, pos);
-
     // Don't record in temporary terrain
     if (you.level_type != LEVEL_DUNGEON)
         return;
 
+    // Or mimics. This could provide mimic information leak, but is safer
+    // than storing the mimic as a branch stair, etc.
+    if (feature_mimic_at(pos))
+        return;
+
     const god_type god = feat_altar_god(which_thing);
     if (god != GOD_NO_GOD)
-        _seen_altar( god, pos );
-    else if (feat_is_branch_stairs( which_thing ))
-        _seen_staircase( which_thing, pos );
+        _seen_altar(god, pos);
+    else if (feat_is_branch_stairs(which_thing))
+        _seen_staircase(which_thing, pos);
     else
-        _seen_other_thing( which_thing, pos );
+        _seen_other_thing(which_thing, pos);
 }
 
 bool move_notable_thing(const coord_def& orig, const coord_def& dest)
@@ -163,18 +176,18 @@ inline static std::string place_desc(const level_pos &pos)
 
 inline static std::string altar_description(god_type god)
 {
-    return feature_description( altar_for_god(god) );
+    return feature_description(altar_for_god(god));
 }
 
 inline static std::string portal_description(portal_type portal)
 {
-    return feature_description( portal_to_feature(portal) );
+    return feature_description(portal_to_feature(portal));
 }
 
 bool overview_knows_portal(dungeon_feature_type portal)
 {
-    for ( portal_map_type::const_iterator pl_iter = portals_present.begin();
-          pl_iter != portals_present.end(); ++pl_iter )
+    for (portal_map_type::const_iterator pl_iter = portals_present.begin();
+          pl_iter != portals_present.end(); ++pl_iter)
     {
         if (portal_to_feature(pl_iter->second) == portal)
             return (true);
@@ -185,8 +198,8 @@ bool overview_knows_portal(dungeon_feature_type portal)
 int overview_knows_num_portals(dungeon_feature_type portal)
 {
     int num = 0;
-    for ( portal_map_type::const_iterator pl_iter = portals_present.begin();
-          pl_iter != portals_present.end(); ++pl_iter )
+    for (portal_map_type::const_iterator pl_iter = portals_present.begin();
+          pl_iter != portals_present.end(); ++pl_iter)
     {
         if (portal_to_feature(pl_iter->second) == portal)
             num++;
@@ -203,19 +216,19 @@ static std::string _portals_description_string()
     {
         last_id.depth = 10000;
         portal_map_type::const_iterator ci_portals;
-        for ( ci_portals = portals_present.begin();
+        for (ci_portals = portals_present.begin();
               ci_portals != portals_present.end();
-              ++ci_portals )
+              ++ci_portals)
         {
             // one line per region should be enough, they're all of
             // the form D:XX, except for labyrinth portals, of which
             // you would need 11 (at least) to have a problem.
-            if ( ci_portals->second == cur_portal )
+            if (ci_portals->second == cur_portal)
             {
-                if ( last_id.depth == 10000 )
+                if (last_id.depth == 10000)
                     disp += portaltype_to_string(ci_portals->second);
 
-                if ( ci_portals->first.id == last_id )
+                if (ci_portals->first.id == last_id)
                     disp += '*';
                 else
                 {
@@ -225,7 +238,7 @@ static std::string _portals_description_string()
                 last_id = ci_portals->first.id;
             }
         }
-        if ( last_id.depth != 10000 )
+        if (last_id.depth != 10000)
             disp += "\n";
     }
     return disp;
@@ -250,7 +263,7 @@ static std::string _portal_vaults_description_string()
     {
         vault_names_vec.push_back(*i);
     }
-    std::sort(vault_names_vec.begin(), vault_names_vec.end() );
+    std::sort(vault_names_vec.begin(), vault_names_vec.end());
 
     std::string disp;
     level_id    last_id;
@@ -267,8 +280,7 @@ static std::string _portal_vaults_description_string()
             {
                 if (last_id.depth == 10000)
                 {
-                    unsigned char col =
-                        (unsigned char) portal_vault_colours[ci_portals->first];
+                    uint8_t col = portal_vault_colours[ci_portals->first];
                     disp += '<';
                     disp += colour_to_str(col) + '>';
                     disp += vault_names_vec[i];
@@ -340,15 +352,28 @@ static std::string _get_seen_branches(bool display)
         disp += " (use <white>G</white> to reach them and "
                 "<white>?/B</white> for more information)";
     }
-    disp += EOL;
+    disp += "\n";
 
     level_id dungeon_lid(branches[0].id, 0);
     dungeon_lid = find_deepest_explored(dungeon_lid);
-    snprintf(buffer, sizeof(buffer),
-             dungeon_lid.depth < 10 ?
-                    "<yellow>Dungeon</yellow> <darkgrey>(%d/27)</darkgrey>            " :
-                    "<yellow>Dungeon</yellow> <darkgrey>(%d/27)</darkgrey>           ",
-             dungeon_lid.depth);
+    if (crawl_state.game_is_sprint())
+    {
+        snprintf(buffer, sizeof(buffer),
+                        "<yellow>Dungeon</yellow> <darkgrey>(1/1)</darkgrey>");
+    }
+    else if (crawl_state.game_is_zotdef())
+    {
+        snprintf(buffer, sizeof(buffer),
+                        "<yellow>Zot</yellow>     <darkgrey>(1/1)</darkgrey>");
+    }
+    else
+    {
+        snprintf(buffer, sizeof(buffer),
+                dungeon_lid.depth < 10 ?
+                        "<yellow>Dungeon</yellow> <darkgrey>(%d/27)</darkgrey>            " :
+                        "<yellow>Dungeon</yellow> <darkgrey>(%d/27)</darkgrey>           ",
+                dungeon_lid.depth);
+    }
     disp += buffer;
 
     for (int i = BRANCH_FIRST_NON_DUNGEON; i < NUM_BRANCHES; i++)
@@ -394,43 +419,30 @@ static std::string _get_unseen_branches()
     {
         const branch_type branch = branches[i].id;
 
-        if (i != BRANCH_SWAMP && i != BRANCH_SNAKE_PIT && i != BRANCH_SHOALS) {
+        if (i != BRANCH_SWAMP && i != BRANCH_SNAKE_PIT && i != BRANCH_SHOALS)
             continue;
-        }
 
-        if (stair_level.find(branch) != stair_level.end()) {
+        if (stair_level.find(branch) != stair_level.end())
             possibly_missing_lair_branches++;
-        }
-        else {
+        else
             missing_lair_branch = i;
-        }
     }
-    if (possibly_missing_lair_branches < 2) {
+    if (possibly_missing_lair_branches < 2)
         missing_lair_branch = -1;
-    }
-
-    bool found_portals[NUM_PORTALS] = { 0 };
-    /* see if we've found portals to hell, pan, the abyss */
-    for (int cur_portal = PORTAL_NONE; cur_portal < NUM_PORTALS; ++cur_portal)
-    {
-        portal_map_type::const_iterator ci_portals;
-        for ( ci_portals = portals_present.begin();
-              ci_portals != portals_present.end();
-              ++ci_portals )
-        {
-            found_portals[ci_portals->second] = true;
-        }
-    }
 
     for (int i = BRANCH_FIRST_NON_DUNGEON; i < NUM_BRANCHES; i++)
     {
         const branch_type branch = branches[i].id;
 
-        if (i == missing_lair_branch) {
+        if (i == missing_lair_branch)
             continue;
-        }
 
-        if (i == BRANCH_VESTIBULE_OF_HELL && found_portals[PORTAL_HELL]) {
+        if (i == BRANCH_VESTIBULE_OF_HELL)
+            continue;
+
+        if (i == BRANCH_FOREST || i == BRANCH_SPIDER_NEST
+            || i == BRANCH_DWARVEN_HALL)
+        {
             continue;
         }
 
@@ -439,8 +451,10 @@ static std::string _get_unseen_branches()
             const branch_type parent_branch = branches[i].parent_branch;
             level_id lid(parent_branch, 0);
             lid = find_deepest_explored(lid);
-            if (lid.depth >= branches[branch].mindepth) {
-                if (branches[branch].mindepth != branches[branch].maxdepth) {
+            if (lid.depth >= branches[branch].mindepth)
+            {
+                if (branches[branch].mindepth != branches[branch].maxdepth)
+                {
                     snprintf(buffer, sizeof buffer,
                         "<darkgrey>%6s: %s:%d-%d</darkgrey>",
                             branches[branch].abbrevname,
@@ -448,7 +462,8 @@ static std::string _get_unseen_branches()
                             branches[branch].mindepth,
                             branches[branch].maxdepth);
                 }
-                else {
+                else
+                {
                     snprintf(buffer, sizeof buffer,
                         "<darkgrey>%6s: %s:%d</darkgrey>",
                             branches[branch].abbrevname,
@@ -467,22 +482,6 @@ static std::string _get_unseen_branches()
         }
     }
 
-    /* not actual branches, have to hardcode this stuff here */
-    if (find_deepest_explored(level_id(BRANCH_MAIN_DUNGEON, 0)).depth >= 21) {
-        if (!found_portals[PORTAL_PANDEMONIUM]) {
-            disp += "<darkgrey>   Pan: D:21-27</darkgrey>";
-            num_printed_branches++;
-            disp += (num_printed_branches % 4) == 0
-                ? "\n" : "     ";
-        }
-        if (!found_portals[PORTAL_ABYSS]) {
-            disp += "<darkgrey> Abyss: D:21-27</darkgrey>";
-            num_printed_branches++;
-            disp += (num_printed_branches % 4) == 0
-                ? "\n" : "     ";
-        }
-    }
-
     if (num_printed_branches % 4 != 0)
         disp += "\n";
     return disp;
@@ -496,6 +495,10 @@ static std::string _get_branches(bool display)
 // iterate through every god and display their altar's discovery state by color
 static std::string _get_altars(bool display)
 {
+    // Just wastes space for demigods.
+    if (you.species == SP_DEMIGOD)
+        return "";
+
     std::string disp;
 
     disp += "\n<green>Altars:</green>";
@@ -504,7 +507,7 @@ static std::string _get_altars(bool display)
         disp += " (use <white>Ctrl-F \"altar\"</white> to reach them and "
                 "<white>?/G</white> for information about gods)";
     }
-    disp += EOL;
+    disp += "\n";
     disp += _print_altars_for_gods(temple_god_list(), true, display);
     disp += _print_altars_for_gods(nontemple_god_list(), false, display);
 
@@ -514,7 +517,7 @@ static std::string _get_altars(bool display)
 // Loops through gods, printing their altar status by color.
 static std::string _print_altars_for_gods(const std::vector<god_type>& gods,
                                           bool print_unseen, bool display)
-{   
+{
     std::string disp;
     char buffer[100];
     int num_printed = 0;
@@ -537,9 +540,9 @@ static std::string _print_altars_for_gods(const std::vector<god_type>& gods,
         }
 
         // If dumping, only laundry list the seen gods
-        if(!display)
+        if (!display)
         {
-            if(has_altar_been_seen)
+            if (has_altar_been_seen)
                 disp += god_name(god, false) + "\n";
             continue;
         }
@@ -559,13 +562,23 @@ static std::string _print_altars_for_gods(const std::vector<god_type>& gods,
         num_printed++;
 
         if (num_printed % 5 == 0)
-            disp += EOL;
+            disp += "\n";
         else
-            disp += std::string(17 - god_name(god, false).length(), ' ');
+            // manually aligning the god columns: five whitespaces between columns
+            switch (num_printed % 5)
+            {
+            case 1: disp += std::string(14 - strwidth(god_name(god, false)), ' ');
+                    break;
+            case 2: disp += std::string(18 - strwidth(god_name(god, false)), ' ');
+                    break;
+            case 3: disp += std::string(13 - strwidth(god_name(god, false)), ' ');
+                    break;
+            case 4: disp += std::string(16 - strwidth(god_name(god, false)), ' ');
+            }
     }
 
     if (num_printed > 0 && num_printed % 5 != 0)
-        disp += EOL;
+        disp += "\n";
     return disp;
 }
 
@@ -580,7 +593,7 @@ static std::string _get_shops(bool display)
         disp +="\n<green>Shops:</green>";
         if (display)
             disp += " (use <white>Ctrl-F \"shop\"</white> to reach them - yellow denotes antique shop)";
-        disp += EOL;
+        disp += "\n";
     }
     last_id.depth = 10000;
     std::map<level_pos, shop_type>::const_iterator ci_shops;
@@ -610,7 +623,7 @@ static std::string _get_shops(bool display)
 
             const std::string loc = ci_shops->first.id.describe(false, true);
             disp += loc;
-            column_count += loc.length();
+            column_count += strwidth(loc);
 
             disp += ": ";
             disp += "</brown>";
@@ -635,9 +648,7 @@ static std::string _get_portals()
     std::string disp;
 
     if (!portals_present.empty() || !portal_vaults_present.empty())
-    {
         disp += "\n<green>Portals:</green>\n";
-    }
     disp += _portals_description_string();
     disp += _portal_vaults_description_string();
 
@@ -661,7 +672,7 @@ static std::string _get_notes()
         {
             const level_id li(branch.id, depth);
 
-            if (get_level_annotation(li).length() > 0)
+            if (!get_level_annotation(li).empty())
             {
                 notes_exist  = true;
                 has_notes[i] = true;
@@ -685,7 +696,7 @@ static std::string _get_notes()
             {
                 const level_id li(branch.id, depth);
 
-                if (get_level_annotation(li).length() > 0)
+                if (!get_level_annotation(li).empty())
                 {
                     sprintf(depth_str, "%d", depth);
 
@@ -694,7 +705,10 @@ static std::string _get_notes()
                     disp += "</yellow>:";
                     disp += depth_str;
                     disp += " ";
-                    disp += get_level_annotation(li);
+                    if (level_annotation_has("!", li))
+                        disp += get_coloured_level_annotation(LIGHTRED, li);
+                    else
+                        disp += get_coloured_level_annotation(LIGHTMAGENTA, li);
                     disp += "\n";
                 }
             }
@@ -773,8 +787,8 @@ void display_overview()
     redraw_screen();
 }
 
-static void _seen_staircase( dungeon_feature_type which_staircase,
-                             const coord_def& pos )
+static void _seen_staircase(dungeon_feature_type which_staircase,
+                             const coord_def& pos)
 {
     // which_staircase holds the grid value of the stair, must be converted
     // Only handles stairs, not gates or arches
@@ -791,11 +805,11 @@ static void _seen_staircase( dungeon_feature_type which_staircase,
             break;
         }
     }
-    ASSERT( i != NUM_BRANCHES );
+    ASSERT(i != NUM_BRANCHES);
 }
 
 // If player has seen an altar; record it.
-static void _seen_altar( god_type god, const coord_def& pos )
+static void _seen_altar(god_type god, const coord_def& pos)
 {
     // Can't record in Abyss or Pan.
     if (you.level_type != LEVEL_DUNGEON)
@@ -813,7 +827,7 @@ void unnotice_altar()
         altars_present.erase(curpos);
 }
 
-portal_type feature_to_portal( unsigned char feat )
+portal_type feature_to_portal(dungeon_feature_type feat)
 {
     switch (feat)
     {
@@ -826,14 +840,25 @@ portal_type feature_to_portal( unsigned char feat )
 }
 
 // If player has seen any other thing; record it.
-void _seen_other_thing( dungeon_feature_type which_thing, const coord_def& pos )
+static void _seen_other_thing(dungeon_feature_type which_thing,
+                              const coord_def& pos)
 {
     level_pos where(level_id::current(), pos);
 
+    monster* mimic = monster_at(pos);
+
     switch (which_thing)
     {
     case DNGN_ENTER_SHOP:
-        shops_present[where] = static_cast<shop_type>(get_shop(pos)->type);
+        if (mimic && mons_is_feat_mimic(mimic->type)
+            && mimic->props.exists("shop_type"))
+        {
+            shops_present[where] = static_cast<shop_type>(mimic->props[
+                                       "shop_type"].get_short());
+        }
+        else
+            shops_present[where] = static_cast<shop_type>(get_shop(pos)->type);
+
         break;
 
     case DNGN_ENTER_PORTAL_VAULT:
@@ -841,6 +866,13 @@ void _seen_other_thing( dungeon_feature_type which_thing, const coord_def& pos )
         std::string portal_name;
 
         portal_name = env.markers.property_at(pos, MAT_ANY, "overview");
+
+        if (mimic && mons_is_feat_mimic(mimic->type)
+            && mimic->props.exists("portal_desc"))
+        {
+            portal_name = (mimic->props["portal_desc"].get_string());
+        }
+
         if (portal_name.empty())
             portal_name = env.markers.property_at(pos, MAT_ANY, "dstname");
         if (portal_name.empty())
@@ -851,12 +883,18 @@ void _seen_other_thing( dungeon_feature_type which_thing, const coord_def& pos )
         portal_name = replace_all(portal_name, "_", " ");
         portal_vaults_present[where] = uppercase_first(portal_name);
 
-        unsigned char col;
+        uint8_t col;
         if (env.grid_colours(pos) != BLACK)
             col = env.grid_colours(pos);
         else
             col = get_feature_def(which_thing).colour;
-        portal_vault_colours[where] = (char) element_colour(col, true);
+        portal_vault_colours[where] = element_colour(col, true);
+
+        if (mimic && mons_is_feat_mimic(mimic->type)
+            && mimic->props.exists("portal_desc"))
+        {
+            portal_vault_colours[where] = mimic->colour;
+        }
 
         portal_vault_notes[where] =
             env.markers.property_at(pos, MAT_ANY, "overview_note");
@@ -931,6 +969,31 @@ std::string get_level_annotation(level_id li, bool skip_excl)
     return (i->second + ", " + j->second);
 }
 
+std::string get_coloured_level_annotation(int col, level_id li, bool skip_excl)
+{
+    annotation_map_type::const_iterator i = level_annotations.find(li);
+
+    if (skip_excl)
+    {
+        if (i == level_annotations.end())
+            return "";
+
+        return (colour_string(i->second, col));
+    }
+
+    annotation_map_type::const_iterator j = level_exclusions.find(li);
+
+    if (i == level_annotations.end() && j == level_exclusions.end())
+        return "";
+
+    if (i == level_annotations.end())
+        return (j->second);
+    if (j == level_exclusions.end())
+        return (colour_string(i->second, col));
+
+    return (colour_string(i->second, col) + ", " + j->second);
+}
+
 bool level_annotation_has(std::string find, level_id li)
 {
     std::string str = get_level_annotation(li);
@@ -967,23 +1030,25 @@ void annotate_level()
 
     if (!get_level_annotation(li).empty())
     {
-        mpr("Current level annotation is:", MSGCH_PROMPT);
-        mpr(get_level_annotation(li, true).c_str() );
+        mpr("Current level annotation: " +
+            colour_string(get_level_annotation(li, true), LIGHTGREY),
+            MSGCH_PROMPT);
     }
 
-    mpr("Set level annotation to what (using ! forces prompt)? ",
-        MSGCH_PROMPT);
+    const std::string prompt = "New level annotation "
+                               "(include '!' for warning): ";
 
     char buf[77];
-    if (cancelable_get_line( buf, sizeof(buf) ))
+    if (msgwin_get_line_autohist(prompt, buf, sizeof(buf)))
         return;
 
     if (buf[0] == 0)
     {
-        if (get_level_annotation(li, true).length() > 0)
+        if (!get_level_annotation(li, true).empty())
         {
-            if (!yesno("Really clear the annotation?", false, 'n'))
+            if (!yesno("Really clear the annotation?", true, 'n'))
                 return;
+            mpr("Cleared.");
         }
         else
         {
