@@ -1,8 +1,7 @@
-/*
- *  File:       externs.h
- *  Summary:    Fixed size 2D vector class that asserts if you do something bad.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Fixed size 2D vector class that asserts if you do something bad.
+**/
 
 #ifndef EXTERNS_H
 #define EXTERNS_H
@@ -17,6 +16,9 @@
 #include <deque>
 
 #include <time.h>
+#include <stdint.h>
+#define __STDC_FORMAT_MACROS
+#include <inttypes.h>
 
 #include "defines.h"
 #include "enum.h"
@@ -26,16 +28,22 @@
 #include "store.h"
 
 #ifdef USE_TILE
+
+#include "tiledef_defines.h"
+
 struct tile_flavour
 {
-    // The floor tile to use.
+    unsigned short floor_idx;
+    unsigned short wall_idx;
+    unsigned short feat_idx;
+
     unsigned short floor;
-    // The wall tile to use.
     unsigned short wall;
-    // Used as a random value or for special cases e.g. (bazaars, gates).
-    unsigned short special;
     // Used (primarily) by the vault 'TILE' overlay.
     unsigned short feat;
+
+    // Used as a random value or for special cases e.g. (bazaars, gates).
+    unsigned short special;
 };
 
 // A glorified unsigned int that assists with ref-counting the mcache.
@@ -43,11 +51,11 @@ class tile_fg_store
 {
 public:
     tile_fg_store() : m_tile(0) {}
-    tile_fg_store(unsigned int tile) : m_tile(tile) {}
-    operator unsigned int() { return m_tile; }
-    unsigned int operator=(unsigned int tile);
+    tile_fg_store(tileidx_t tile) : m_tile(tile) {}
+    operator tileidx_t() { return m_tile; }
+    tileidx_t operator=(tileidx_t tile);
 protected:
-    unsigned int m_tile;
+    tileidx_t m_tile;
 };
 
 
@@ -61,9 +69,9 @@ protected:
 
 #define BURDEN_TO_AUM 0.1f           // scale factor for converting burden to aum
 
-extern char info[INFO_SIZE];         // defined in acr.cc {dlb}
+extern char info[INFO_SIZE];         // defined in main.cc {dlb}
 
-const int kNameLen = 30;
+#define kNameLen        30
 #ifdef SHORT_FILE_NAMES
     const int kFileNameLen = 6;
 #else
@@ -78,6 +86,7 @@ const int kPathLen = 256;
 #define NO_BERSERK_PENALTY    -1
 
 typedef FixedArray<dungeon_feature_type, GXM, GYM> feature_grid;
+typedef FixedArray<unsigned short, GXM, GYM> map_mask;
 
 struct item_def;
 struct coord_def;
@@ -96,12 +105,14 @@ template <typename Z> inline Z sgn(Z x)
     return (x < 0 ? -1 : (x > 0 ? 1 : 0));
 }
 
+inline int dist_range(int x) { return x*x + 1; };
+
 struct coord_def
 {
     int         x;
     int         y;
 
-    explicit coord_def( int x_in = 0, int y_in = 0 ) : x(x_in), y(y_in) { }
+    explicit coord_def(int x_in = 0, int y_in = 0) : x(x_in), y(y_in) { }
 
     void set(int xi, int yi)
     {
@@ -250,6 +261,8 @@ struct coord_def
     }
 };
 
+const coord_def INVALID_COORD(-1, -1);
+
 typedef bool (*coord_predicate)(const coord_def &c);
 
 struct run_check_dir
@@ -258,14 +271,21 @@ struct run_check_dir
     coord_def delta;
 };
 
+typedef uint32_t mid_t;
+#define MID_PLAYER      ((mid_t)0xffffffff)
+// the numbers are meaningless, there's just plenty of space for gods, env,
+// and whatever else we want to have, while keeping all monster ids smaller.
+#define MID_ANON_FRIEND ((mid_t)0xffff0000)
+
 struct cloud_struct
 {
     coord_def     pos;
     cloud_type    type;
     int           decay;
-    unsigned char spread_rate;
+    uint8_t       spread_rate;
     kill_category whose;
     killer_type   killer;
+    mid_t         source;
     int           colour;
     std::string   name;
     std::string   tile;
@@ -276,22 +296,36 @@ struct cloud_struct
     {
     }
 
+    bool defined() const { return type != CLOUD_NONE; }
+
     void set_whose(kill_category _whose);
     void set_killer(killer_type _killer);
 
+    std::string cloud_name(const std::string &default_name = "",
+                           bool terse = false) const;
+    void announce_actor_engulfed(const actor *engulfee,
+                                 bool beneficial = false) const;
+
     static kill_category killer_to_whose(killer_type killer);
     static killer_type   whose_to_killer(kill_category whose);
-
 };
 
 struct shop_struct
 {
     coord_def           pos;
-    unsigned char       greed;
+    uint8_t             greed;
     shop_type           type;
-    unsigned char       level;
+    uint8_t             level;
+    std::string         shop_name;
+    std::string         shop_type_name;
+    std::string         shop_suffix_name;
 
-    FixedVector<unsigned char, 3> keeper_name;
+    FixedVector<uint8_t, 3> keeper_name;
+
+    shop_struct () : pos(), greed(0), type(SHOP_UNASSIGNED), level(0),
+                     shop_name(""), shop_type_name(""), shop_suffix_name("") { }
+
+    bool defined() const { return type != SHOP_UNASSIGNED; }
 };
 
 
@@ -302,6 +336,8 @@ struct delay_queue_item
     int         parm1;
     int         parm2;
     bool        started;
+    int         trits[6];
+    size_t      len;
 };
 
 
@@ -379,19 +415,19 @@ public:
         return (*this);
     }
 
-    bool operator == ( const level_id &id ) const
+    bool operator == (const level_id &id) const
     {
         return (level_type == id.level_type
                 && (level_type != LEVEL_DUNGEON
                     || (branch == id.branch && depth == id.depth)));
     }
 
-    bool operator != ( const level_id &id ) const
+    bool operator != (const level_id &id) const
     {
         return !operator == (id);
     }
 
-    bool operator <( const level_id &id ) const
+    bool operator <(const level_id &id) const
     {
         if (level_type != id.level_type)
             return (level_type < id.level_type);
@@ -402,12 +438,12 @@ public:
         return (branch < id.branch) || (branch==id.branch && depth < id.depth);
     }
 
-    bool operator == ( const branch_type _branch ) const
+    bool operator == (const branch_type _branch) const
     {
         return (branch == _branch && level_type == LEVEL_DUNGEON);
     }
 
-    bool operator != ( const branch_type _branch  ) const
+    bool operator != (const branch_type _branch) const
     {
         return !operator == (_branch);
     }
@@ -441,17 +477,17 @@ struct level_pos
     // Returns the level_pos of where the player is standing.
     static level_pos current();
 
-    bool operator == ( const level_pos &lp ) const
+    bool operator == (const level_pos &lp) const
     {
         return id == lp.id && pos == lp.pos;
     }
 
-    bool operator != ( const level_pos &lp ) const
+    bool operator != (const level_pos &lp) const
     {
         return id != lp.id || pos != lp.pos;
     }
 
-    bool operator <  ( const level_pos &lp ) const
+    bool operator <  (const level_pos &lp) const
     {
         return (id < lp.id) || (id == lp.id && pos < lp.pos);
     }
@@ -461,7 +497,7 @@ struct level_pos
         return id.depth > -1 && pos.x != -1 && pos.y != -1;
     }
 
-    bool is_on( const level_id _id)
+    bool is_on(const level_id _id)
     {
         return id == _id;
     }
@@ -476,18 +512,25 @@ struct level_pos
     void load(reader&);
 };
 
-class monsters;
+class monster;
+
+// We are not 64 bits clean here yet since many places still pass (or store!)
+// it as 32 bits or, worse, longs.  I considered setting this as uint32_t,
+// however, since free bits are exhausted, it's very likely we'll have to
+// extend this in the future, so this should be easier than undoing the change.
+typedef uint32_t iflags_t;
 
 struct item_def
 {
-    object_class_type  base_type;  // basic class (ie OBJ_WEAPON)
-    unsigned char  sub_type;       // type within that class (ie WPN_DAGGER)
+    object_class_type base_type:8; // basic class (ie OBJ_WEAPON)
+    uint8_t        sub_type;       // type within that class (ie WPN_DAGGER)
     short          plus;           // +to hit, charges, corpse mon id
     short          plus2;          // +to dam, sub-sub type for boots/helms
-    long           special;        // special stuff
-    unsigned char  colour;         // item colour
-    unsigned long  flags;          // item status flags
+    int            special;        // special stuff
+    uint8_t        colour;         // item colour
+    uint8_t        rnd;            // random number, used for tile choice
     short          quantity;       // number of items
+    iflags_t       flags;          // item status flags
 
     coord_def pos;     // for inventory items == (-1, -1)
     short  link;       // link to next item;  for inventory items = slot
@@ -502,7 +545,7 @@ struct item_def
 
 public:
     item_def() : base_type(OBJ_UNASSIGNED), sub_type(0), plus(0), plus2(0),
-                 special(0L), colour(0), flags(0L), quantity(0),
+                 special(0L), colour(0), rnd(0), quantity(0), flags(0L),
                  pos(), link(NON_ITEM), slot(0), orig_place(0),
                  orig_monnum(0), inscription()
     {
@@ -512,7 +555,7 @@ public:
                      bool terse = false, bool ident = false,
                      bool with_inscription = true,
                      bool quantity_in_words = false,
-                     unsigned long ignore_flags = 0x0) const;
+                     iflags_t ignore_flags = 0x0) const;
     bool has_spells() const;
     bool cursed() const;
     int book_number() const;
@@ -535,11 +578,12 @@ public:
     void set_holding_monster(int midx);
 
     // Returns monster holding this item.  NULL if none.
-    monsters* holding_monster() const;
+    monster* holding_monster() const;
 
     // Returns true if a monster is holding this item.
     bool held_by_monster() const;
 
+    bool defined() const;
     bool is_valid() const;
 
     // Returns true if this item should be preserved as far as possible.
@@ -550,10 +594,13 @@ public:
 
 private:
     std::string name_aux(description_level_type desc,
-                         bool terse, bool ident,
-                         unsigned long ignore_flags) const;
+                         bool terse, bool ident, bool with_inscription,
+                         iflags_t ignore_flags) const;
 };
 
+typedef item_def item_info;
+item_info get_item_info(const item_def& info);
+
 class runrest
 {
 public:
@@ -579,6 +626,8 @@ public:
     bool is_explore() const;
     bool is_any_travel() const;
 
+    std::string runmode_name() const;
+
     // Clears run state.
     void clear();
 
@@ -594,50 +643,7 @@ public:
 
 private:
     void set_run_check(int index, int compass_dir);
-    bool run_grids_changed() const;
-};
-
-class PlaceInfo
-{
-public:
-    int level_type;     // enum level_area_type
-    int branch;         // enum branch_type if LEVEL_DUNGEON; otherwise -1
-
-    unsigned long num_visits;
-    unsigned long levels_seen;
-
-    unsigned long mon_kill_exp;
-    unsigned long mon_kill_exp_avail;
-    unsigned long mon_kill_num[KC_NCATEGORIES];
-
-    long turns_total;
-    long turns_explore;
-    long turns_travel;
-    long turns_interlevel;
-    long turns_resting;
-    long turns_other;
-
-    long elapsed_total;
-    long elapsed_explore;
-    long elapsed_travel;
-    long elapsed_interlevel;
-    long elapsed_resting;
-    long elapsed_other;
-
-public:
-    PlaceInfo();
-
-    bool is_global() const;
-    void make_global();
-
-    void assert_validity() const;
-
-    const std::string short_name() const;
-
-    const PlaceInfo &operator += (const PlaceInfo &other);
-    const PlaceInfo &operator -= (const PlaceInfo &other);
-    PlaceInfo operator + (const PlaceInfo &other) const;
-    PlaceInfo operator - (const PlaceInfo &other) const;
+    bool run_should_stop() const;
 };
 
 typedef std::vector<delay_queue_item> delay_queue_type;
@@ -661,7 +667,10 @@ public:
     map_markers &operator = (const map_markers &);
     ~map_markers();
 
+    bool need_activate() const { return have_inactive_markers; }
+    void clear_need_activate();
     void activate_all(bool verbose = true);
+    void activate_markers_at(coord_def p);
     void add(map_marker *marker);
     void remove(map_marker *marker);
     void remove_markers_at(const coord_def &c, map_marker_type type = MAT_ANY);
@@ -678,7 +687,7 @@ public:
     void clear();
 
     void write(writer &) const;
-    void read(reader &, int minorVersion);
+    void read(reader &);
 
 private:
     typedef std::multimap<coord_def, map_marker *> dgn_marker_map;
@@ -686,9 +695,11 @@ private:
 
     void init_from(const map_markers &);
     void unlink_marker(const map_marker *);
+    void check_empty();
 
 private:
     dgn_marker_map markers;
+    bool have_inactive_markers;
 };
 
 struct message_filter
@@ -696,14 +707,14 @@ struct message_filter
     int             channel;        // Use -1 to match any channel.
     text_pattern    pattern;        // Empty pattern matches any message
 
-    message_filter( int ch, const std::string &s )
+    message_filter(int ch, const std::string &s)
         : channel(ch), pattern(s)
     {
     }
 
-    message_filter( const std::string &s ) : channel(-1), pattern(s) { }
+    message_filter(const std::string &s) : channel(-1), pattern(s) { }
 
-    bool is_filtered( int ch, const std::string &s ) const {
+    bool is_filtered(int ch, const std::string &s) const {
         bool channel_match = ch == channel || channel == -1;
         if (!channel_match || pattern.empty())
             return channel_match;
@@ -771,7 +782,7 @@ private:
 struct mon_display
 {
     monster_type type;
-    unsigned     glyph;
+    wchar_t      glyph;
     unsigned     colour;
     monster_type detected; // What a monster of type "type" is detected as.
 
@@ -781,4 +792,12 @@ struct mon_display
        : type(m), glyph(gly), colour(col), detected(d) { }
 };
 
+struct final_effect
+{
+    final_effect_flavour flavour;
+    short att, def;
+    coord_def pos;
+    int x;
+};
+
 #endif // EXTERNS_H
