@@ -1,8 +1,7 @@
-/*
- *  File:       kills.cc
- *  Summary:    Player kill tracking
- *  Written by: Darshan Shaligram
- */
+/**
+ * @file
+ * @brief Player kill tracking
+**/
 
 #include "AppHdr.h"
 
@@ -20,6 +19,7 @@
 #include "kills.h"
 #include "clua.h"
 #include "options.h"
+#include "viewchar.h"
 
 #define KILLS_MAJOR_VERSION 4
 #define KILLS_MINOR_VERSION 1
@@ -79,8 +79,8 @@ void KillMaster::save(writer& outf) const
 
 void KillMaster::load(reader& inf)
 {
-    unsigned char major = unmarshallByte(inf),
-                  minor = unmarshallByte(inf);
+    int major = unmarshallByte(inf),
+        minor = unmarshallByte(inf);
     if (major != KILLS_MAJOR_VERSION
         || (minor != KILLS_MINOR_VERSION && minor > 0))
     {
@@ -95,7 +95,7 @@ void KillMaster::load(reader& inf)
     }
 }
 
-void KillMaster::record_kill(const monsters *mon, int killer, bool ispet)
+void KillMaster::record_kill(const monster* mon, int killer, bool ispet)
 {
     const kill_category kc =
         YOU_KILL(killer)? KC_YOU :
@@ -104,16 +104,16 @@ void KillMaster::record_kill(const monsters *mon, int killer, bool ispet)
     categorized_kills[kc].record_kill(mon);
 }
 
-long KillMaster::total_kills() const
+int KillMaster::total_kills() const
 {
-    long grandtotal = 0L;
+    int grandtotal = 0;
     for (int i = KC_YOU; i < KC_NCATEGORIES; ++i)
     {
         if (categorized_kills[i].empty())
             continue;
 
         std::vector<kill_exp> kills;
-        long count = categorized_kills[i].get_kills(kills);
+        int count = categorized_kills[i].get_kills(kills);
         grandtotal += count;
     }
     return (grandtotal);
@@ -128,13 +128,13 @@ std::string KillMaster::kill_info() const
 
     bool needseparator = false;
     int categories = 0;
-    long grandtotal = 0L;
+    int grandtotal = 0;
 
     Kills catkills[KC_NCATEGORIES];
     for (int i = 0; i < KC_NCATEGORIES; ++i)
     {
         int targ = Options.kill_map[i];
-        catkills[targ].merge( categorized_kills[i] );
+        catkills[targ].merge(categorized_kills[i]);
     }
 
     for (int i = KC_YOU; i < KC_NCATEGORIES; ++i)
@@ -144,15 +144,15 @@ std::string KillMaster::kill_info() const
 
         categories++;
         std::vector<kill_exp> kills;
-        long count = catkills[i].get_kills(kills);
+        int count = catkills[i].get_kills(kills);
         grandtotal += count;
 
-        add_kill_info( killtext,
+        add_kill_info(killtext,
                        kills,
                        count,
                        i == KC_YOU? NULL :
                                     category_name((kill_category) i),
-                       needseparator );
+                       needseparator);
         needseparator = true;
     }
 
@@ -161,7 +161,7 @@ std::string KillMaster::kill_info() const
     {
         char buf[200];
         snprintf(buf, sizeof buf,
-                "Grand Total: %ld creatures vanquished",
+                "Grand Total: %d creatures vanquished",
                 grandtotal);
         grandt = buf;
     }
@@ -186,7 +186,7 @@ std::string KillMaster::kill_info() const
 
 void KillMaster::add_kill_info(std::string &killtext,
                                std::vector<kill_exp> &kills,
-                               long count,
+                               int count,
                                const char *category,
                                bool separator) const
 {
@@ -211,7 +211,7 @@ void KillMaster::add_kill_info(std::string &killtext,
 #endif
     {
 #ifdef CLUA_BINDINGS
-        if (clua.error.length())
+        if (!clua.error.empty())
         {
             killtext += "Lua error:\n";
             killtext += clua.error + "\n\n";
@@ -234,21 +234,21 @@ void KillMaster::add_kill_info(std::string &killtext,
         {
             char numbuf[100];
             snprintf(numbuf, sizeof numbuf,
-                    "%ld creature%s vanquished." "\n", count,
+                    "%d creature%s vanquished." "\n", count,
                     count > 1? "s" : "");
             killtext += numbuf;
         }
     }
 }
 
-long KillMaster::num_kills(const monsters *mon, kill_category cat) const
+int KillMaster::num_kills(const monster* mon, kill_category cat) const
 {
     return categorized_kills[cat].num_kills(mon);
 }
 
-long KillMaster::num_kills(const monsters *mon) const
+int KillMaster::num_kills(const monster* mon) const
 {
-    long total = 0;
+    int total = 0;
     for (int cat = 0; cat < KC_NCATEGORIES; cat++)
         total += categorized_kills[cat].num_kills(mon);
 
@@ -264,7 +264,7 @@ bool Kills::empty() const
 
 void Kills::merge(const Kills &k)
 {
-    ghosts.insert( ghosts.end(), k.ghosts.begin(), k.ghosts.end() );
+    ghosts.insert(ghosts.end(), k.ghosts.begin(), k.ghosts.end());
 
     // Regular kills are messier to merge.
     for (kill_map::const_iterator i = k.kills.begin();
@@ -278,7 +278,7 @@ void Kills::merge(const Kills &k)
     }
 }
 
-void Kills::record_kill(const struct monsters *mon)
+void Kills::record_kill(const monster* mon)
 {
     // Handle player ghosts separately, but don't handle summoned
     // ghosts at all. {due}
@@ -299,22 +299,22 @@ void Kills::record_kill(const struct monsters *mon)
         k = kill_def(mon);
 }
 
-long Kills::get_kills(std::vector<kill_exp> &all_kills) const
+int Kills::get_kills(std::vector<kill_exp> &all_kills) const
 {
-    long count = 0;
+    int count = 0;
     kill_map::const_iterator iter = kills.begin();
     for (; iter != kills.end(); ++iter)
     {
         const kill_monster_desc &md = iter->first;
         const kill_def &k = iter->second;
-        all_kills.push_back( kill_exp(k, md) );
+        all_kills.push_back(kill_exp(k, md));
         count += k.kills;
     }
 
     ghost_vec::const_iterator gi = ghosts.begin();
     for (; gi != ghosts.end(); ++gi)
     {
-        all_kills.push_back( kill_exp(*gi) );
+        all_kills.push_back(kill_exp(*gi));
     }
     count += ghosts.size();
 
@@ -325,9 +325,9 @@ long Kills::get_kills(std::vector<kill_exp> &all_kills) const
 void Kills::save(writer& outf) const
 {
     // How many kill records do we have?
-    marshallLong(outf, kills.size());
+    marshallInt(outf, kills.size());
 
-    for ( kill_map::const_iterator iter = kills.begin();
+    for (kill_map::const_iterator iter = kills.begin();
           iter != kills.end(); ++iter)
     {
         iter->first.save(outf);
@@ -346,9 +346,9 @@ void Kills::save(writer& outf) const
 void Kills::load(reader& inf)
 {
     // How many kill records?
-    long kill_count = unmarshallLong(inf);
+    int kill_count = unmarshallInt(inf);
     kills.clear();
-    for (long i = 0; i < kill_count; ++i)
+    for (int i = 0; i < kill_count; ++i)
     {
         kill_monster_desc md;
         md.load(inf);
@@ -365,7 +365,7 @@ void Kills::load(reader& inf)
     }
 }
 
-void Kills::record_ghost_kill(const struct monsters *mon)
+void Kills::record_ghost_kill(const monster* mon)
 {
     // We should never get to this point, but just in case... {due}
     if (mon->is_summoned())
@@ -374,7 +374,7 @@ void Kills::record_ghost_kill(const struct monsters *mon)
     ghosts.push_back(ghostk);
 }
 
-int Kills::num_kills(const monsters *mon) const
+int Kills::num_kills(const monster* mon) const
 {
     kill_monster_desc desc(mon);
     kill_map::const_iterator iter = kills.find(desc);
@@ -390,33 +390,12 @@ int Kills::num_kills(const monsters *mon) const
     return total;
 }
 
-kill_def::kill_def(const struct monsters *mon) : kills(0), exp(0)
+kill_def::kill_def(const monster* mon) : kills(0), exp(0)
 {
     exp = exper_value(mon);
     add_kill(mon, get_packed_place());
 }
 
-std::string apostrophise(const std::string &name)
-{
-    if (name.empty())
-        return (name);
-
-    if (name == "it" || name == "It")
-        return (name + "s");
-
-    const char lastc = name[name.length() - 1];
-    return (name + (lastc == 's' ? "'" : "'s"));
-}
-
-std::string apostrophise_fixup(const std::string &msg)
-{
-    if (msg.empty())
-        return (msg);
-
-    // XXX: This is rather hackish.
-    return (replace_all(msg, "s's", "s'"));
-}
-
 // For monster names ending with these suffixes, we pluralise directly without
 // attempting to use the "of" rule. For instance:
 //
@@ -484,7 +463,7 @@ void kill_def::merge(const kill_def &k, bool uniq)
     }
 }
 
-void kill_def::add_kill(const struct monsters *mon, unsigned short place)
+void kill_def::add_kill(const monster* mon, unsigned short place)
 {
     kills++;
     // They're only unique if they aren't summoned.
@@ -503,7 +482,7 @@ void kill_def::add_place(unsigned short place, bool force)
 std::string kill_def::base_name(const kill_monster_desc &md) const
 {
     std::string name;
-    if (md.monnum == MONS_PANDEMONIUM_DEMON)
+    if (md.monnum == MONS_PANDEMONIUM_DEMON || md.monnum == MONS_HELL_DEMON)
         name = "demon lord";
     else
         name = mons_type_name(md.monnum, DESC_PLAIN);
@@ -581,8 +560,8 @@ std::string kill_def::append_places(const kill_monster_desc &md,
     if (Options.dump_kill_places == KDO_NO_PLACES) return name;
 
     int nplaces = places.size();
-    if ( nplaces == 1 || mons_is_unique(md.monnum)
-            || Options.dump_kill_places == KDO_ALL_PLACES )
+    if (nplaces == 1 || mons_is_unique(md.monnum)
+            || Options.dump_kill_places == KDO_ALL_PLACES)
     {
         std::string augmented = name;
         augmented += " (";
@@ -623,7 +602,7 @@ void kill_def::load(reader& inf)
         places.push_back((unsigned short) unmarshallShort(inf));
 }
 
-kill_ghost::kill_ghost(const monsters *mon)
+kill_ghost::kill_ghost(const monster* mon)
 {
     exp = exper_value(mon);
     place = get_packed_place();
@@ -632,7 +611,10 @@ kill_ghost::kill_ghost(const monsters *mon)
     // Check whether this is really a ghost, since we also have to handle
     // the Pandemonic demons.
     if (mon->type == MONS_PLAYER_GHOST && !mon->is_summoned())
-        ghost_name = "The ghost of " + get_ghost_description(*mon, true);
+    {
+        monster_info mi(mon);
+        ghost_name = "The ghost of " + get_ghost_description(mi, true);
+    }
 }
 
 std::string kill_ghost::info() const
@@ -656,7 +638,7 @@ void kill_ghost::load(reader& inf)
     place = (unsigned short) unmarshallShort(inf);
 }
 
-kill_monster_desc::kill_monster_desc(const monsters *mon)
+kill_monster_desc::kill_monster_desc(const monster* mon)
 {
     monnum = mon->type;
     modifier = M_NORMAL;
@@ -712,7 +694,7 @@ void kill_monster_desc::load(reader& inf)
             return 0; \
         } \
           \
-        kill_exp *ke = static_cast<kill_exp*>( lua_touserdata(ls, 1) ); \
+        kill_exp *ke = static_cast<kill_exp*>(lua_touserdata(ls, 1)); \
         if (ke) \
         { \
             lua_push##type(ls, ke->field); \
@@ -744,7 +726,7 @@ static int kill_lualc_modifier(lua_State *ls)
         return 0;
     }
 
-    kill_exp *ke = static_cast<kill_exp*>( lua_touserdata(ls, 1) );
+    kill_exp *ke = static_cast<kill_exp*>(lua_touserdata(ls, 1));
     if (ke)
     {
         const char *modifier;
@@ -783,7 +765,7 @@ static int kill_lualc_places(lua_State *ls)
         return 0;
     }
 
-    kill_exp *ke = static_cast<kill_exp*>( lua_touserdata(ls, 1) );
+    kill_exp *ke = static_cast<kill_exp*>(lua_touserdata(ls, 1));
     if (ke)
     {
         lua_newtable(ls);
@@ -819,7 +801,7 @@ static int kill_lualc_holiness(lua_State *ls)
         return 0;
     }
 
-    kill_exp *ke = static_cast<kill_exp*>( lua_touserdata(ls, 1) );
+    kill_exp *ke = static_cast<kill_exp*>(lua_touserdata(ls, 1));
     if (ke)
     {
         const char *verdict = "strange";
@@ -854,11 +836,11 @@ static int kill_lualc_symbol(lua_State *ls)
         return 0;
     }
 
-    kill_exp *ke = static_cast<kill_exp*>( lua_touserdata(ls, 1) );
+    kill_exp *ke = static_cast<kill_exp*>(lua_touserdata(ls, 1));
     if (ke)
     {
-        unsigned char ch = ke->monnum != -1?
-                    mons_char(ke->monnum) :
+        wchar_t ch = ke->monnum != -1?
+                     mons_char(ke->monnum) :
               is_ghost(ke)? 'p' : '&';
 
         if (ke->monnum == MONS_PROGRAM_BUG)
@@ -868,20 +850,21 @@ static int kill_lualc_symbol(lua_State *ls)
         {
         case kill_monster_desc::M_ZOMBIE:
         case kill_monster_desc::M_SKELETON:
+            ch = mons_char(mons_zombie_size(ke->monnum) == Z_SMALL ?
+                           MONS_ZOMBIE_SMALL : MONS_ZOMBIE_LARGE);
+            break;
         case kill_monster_desc::M_SIMULACRUM:
-            ch = mons_zombie_size(ke->monnum) == Z_SMALL ? 'z' : 'Z';
+            ch = mons_char(mons_zombie_size(ke->monnum) == Z_SMALL ?
+                           MONS_SIMULACRUM_SMALL : MONS_SIMULACRUM_LARGE);
             break;
         case kill_monster_desc::M_SPECTRE:
-            ch = 'W';
+            ch = mons_char(MONS_SPECTRAL_THING);
             break;
         default:
             break;
         }
 
-        char s[2];
-        s[0] = (char) ch;
-        s[1] = 0;
-        lua_pushstring(ls, s);
+        lua_pushstring(ls, stringize_glyph(ch).c_str());
         return 1;
     }
     return 0;
@@ -899,7 +882,7 @@ static int kill_lualc_rawwrite(lua_State *ls)
         return 0;
     }
 
-    std::string *skill = static_cast<std::string *>( lua_touserdata(ls, -1) );
+    std::string *skill = static_cast<std::string *>(lua_touserdata(ls, -1));
     // Pop the userdata off the stack.
     lua_settop(ls, -2);
 
@@ -917,7 +900,7 @@ static int kill_lualc_write(lua_State *ls)
         return 0;
     }
 
-    kill_exp *ke = static_cast<kill_exp*>( lua_touserdata(ls, 1) );
+    kill_exp *ke = static_cast<kill_exp*>(lua_touserdata(ls, 1));
     if (ke)
     {
         lua_pushstring(ls, "cr_skill");
@@ -930,7 +913,7 @@ static int kill_lualc_write(lua_State *ls)
         }
 
         std::string *skill = static_cast<std::string *>(
-                                    lua_touserdata(ls, -1) );
+                                    lua_touserdata(ls, -1));
         // Pop the userdata off the stack.
         lua_settop(ls, -2);
 
@@ -948,7 +931,7 @@ static int kill_lualc_summary(lua_State *ls)
         return 0;
     }
 
-    unsigned long count = 0;
+    unsigned int count = 0;
     for (int i = 1; ; ++i)
     {
         lua_rawgeti(ls, 1, i);
@@ -964,7 +947,7 @@ static int kill_lualc_summary(lua_State *ls)
             return 0;
         }
 
-        kill_exp *ke = static_cast<kill_exp*>( lua_touserdata(ls, -1) );
+        kill_exp *ke = static_cast<kill_exp*>(lua_touserdata(ls, -1));
         lua_settop(ls, -2);
         if (ke)
             count += ke->nkills;
@@ -972,7 +955,7 @@ static int kill_lualc_summary(lua_State *ls)
     char buf[120];
     *buf = 0;
     if (count)
-        snprintf(buf, sizeof buf, "%lu creature%s vanquished.",
+        snprintf(buf, sizeof buf, "%u creature%s vanquished.",
                 count, count > 1? "s" : "");
     lua_pushstring(ls, buf);
     return 1;
