@@ -9,6 +9,7 @@
 #include "cloud.h"
 #include "colour.h"
 #include "coord.h"
+#include "coordit.h"
 #include "directn.h"
 #include "dungeon.h"
 #include "dgn-shoals.h"
@@ -23,7 +24,8 @@
 #include "spl-util.h"
 #include "state.h"
 #ifdef USE_TILE
-#include "tiledef-dngn.h"
+ #include "tiledef-dngn.h"
+ #include "tileview.h"
 #endif
 #include "view.h"
 
@@ -44,17 +46,13 @@ void dgn_reset_default_depth()
 
 std::string dgn_set_default_depth(const std::string &s)
 {
-    std::vector<std::string> frags = split_string(",", s);
-    for (int i = 0, size = frags.size(); i < size; ++i)
+    try
     {
-        try
-        {
-            lc_default_depths.push_back( level_range::parse(frags[i]) );
-        }
-        catch (const std::string &error)
-        {
-            return (error);
-        }
+        lc_default_depths = depth_ranges::parse_depth_ranges(s);
+    }
+    catch (const std::string &error)
+    {
+        return (error);
     }
     return ("");
 }
@@ -64,31 +62,22 @@ static void dgn_add_depths(depth_ranges &drs, lua_State *ls, int s, int e)
     for (int i = s; i <= e; ++i)
     {
         const char *depth = luaL_checkstring(ls, i);
-        std::vector<std::string> frags = split_string(",", depth);
-        for (int j = 0, size = frags.size(); j < size; ++j)
+        try
         {
-            try
-            {
-                drs.push_back( level_range::parse(frags[j]) );
-            }
-            catch (const std::string &error)
-            {
-                luaL_error(ls, error.c_str());
-            }
+            drs.add_depths(depth_ranges::parse_depth_ranges(depth));
+        }
+        catch (const std::string &error)
+        {
+            luaL_error(ls, error.c_str());
         }
     }
 }
 
-static std::string dgn_depth_list_string(const depth_ranges &drs)
-{
-    return (comma_separated_line(drs.begin(), drs.end(), ", ", ", "));
-}
-
 static int dgn_depth_proc(lua_State *ls, depth_ranges &dr, int s)
 {
     if (lua_gettop(ls) < s)
     {
-        PLUARET(string, dgn_depth_list_string(dr).c_str());
+        PLUARET(string, dr.describe().c_str());
     }
 
     if (lua_isnil(ls, s))
@@ -135,6 +124,19 @@ static int dgn_place(lua_State *ls)
     PLUARET(string, map->place.describe().c_str());
 }
 
+static int dgn_desc(lua_State *ls)
+{
+    MAP(ls, 1, map);
+    if (lua_gettop(ls) > 1)
+    {
+        if (lua_isnil(ls, 2))
+            map->description.clear();
+        else
+            map->description = trimmed_string(luaL_checkstring(ls, 2));
+    }
+    PLUARET(string, map->description.c_str());
+}
+
 static int dgn_tags(lua_State *ls)
 {
     MAP(ls, 1, map);
@@ -151,6 +153,12 @@ static int dgn_tags(lua_State *ls)
     PLUARET(string, map->tags.c_str());
 }
 
+static int dgn_has_tag(lua_State *ls)
+{
+    MAP(ls, 1, map);
+    PLUARET(boolean, map->has_tag(luaL_checkstring(ls, 2)));
+}
+
 static int dgn_tags_remove(lua_State *ls)
 {
     MAP(ls, 1, map);
@@ -159,11 +167,7 @@ static int dgn_tags_remove(lua_State *ls)
     for (int i = 2; i <= top; ++i)
     {
         const std::string axee = luaL_checkstring(ls, i);
-        const std::string::size_type pos = map->tags.find(axee);
-        if (pos != std::string::npos)
-            map->tags =
-            map->tags.substr(0, pos)
-            + map->tags.substr(pos + axee.length());
+        while (strip_tag(map->tags, axee));
     }
     PLUARET(string, map->tags.c_str());
 }
@@ -192,7 +196,8 @@ static int dgn_change_level_flags(lua_State *ls)
 {
     map_flags flags;
 
-    try {
+    try
+    {
         flags = map_flags::parse(level_flag_names,
                                  luaL_checkstring(ls, 1));
     }
@@ -214,13 +219,14 @@ static int dgn_change_level_flags(lua_State *ls)
 }
 
 static const std::string branch_flag_names[] =
-{"no_tele_control", "not_mappable", "no_magic_map", ""};
+{"no_tele_control", "not_mappable", "no_magic_map", "has_orb", "islanded", ""};
 
 static int dgn_bflags(lua_State *ls)
 {
     MAP(ls, 1, map);
 
-    try {
+    try
+    {
         map->branch_flags = map_flags::parse(branch_flag_names,
                                              luaL_checkstring(ls, 2));
     }
@@ -236,7 +242,8 @@ static int dgn_change_branch_flags(lua_State *ls)
 {
     map_flags flags;
 
-    try {
+    try
+    {
         flags = map_flags::parse(branch_flag_names,
                                  luaL_checkstring(ls, 1));
     }
@@ -257,30 +264,74 @@ static int dgn_change_branch_flags(lua_State *ls)
     return (1);
 }
 
+static void _chance_magnitude_check(lua_State *ls, int which_par, int chance)
+{
+    if (chance < 0 || chance > CHANCE_ROLL)
+        luaL_argerror(ls, which_par,
+                      make_stringf("Chance must be in the range [0,%d]",
+                                   CHANCE_ROLL).c_str());
+}
+
 static int dgn_chance(lua_State *ls)
 {
     MAP(ls, 1, map);
-    if (!lua_isnil(ls, 2) && !lua_isnil(ls, 3))
+    if (lua_isnumber(ls, 2))
     {
-        const int chance_priority = luaL_checkint(ls, 2);
-        const int chance = luaL_checkint(ls, 3);
-        if (chance < 0 || chance > CHANCE_ROLL)
-            luaL_argerror(ls, 2,
-                          make_stringf("Chance must be in the range [0,%d]",
-                                       CHANCE_ROLL).c_str());
-
-        map->chance_priority = chance_priority;
-        map->chance = chance;
+        const bool has_priority = lua_isnumber(ls, 3);
+        const int chance_priority =
+            has_priority? luaL_checkint(ls, 2) : DEFAULT_CHANCE_PRIORITY;
+        const int chance_par = 2 + has_priority;
+        const int chance = luaL_checkint(ls, chance_par);
+        _chance_magnitude_check(ls, chance_par, chance);
+        map->_chance.set_default(map_chance(chance_priority, chance));
     }
-    PLUARET(number, map->chance);
+    return 0;
 }
 
+static int dgn_depth_chance(lua_State *ls)
+{
+    MAP(ls, 1, map);
+    const std::string depth(luaL_checkstring(ls, 2));
+    const bool has_priority = lua_gettop(ls) == 4;
+    const int chance_priority =
+        has_priority? luaL_checkint(ls, 3) : DEFAULT_CHANCE_PRIORITY;
+    const int chance_par = 3 + has_priority;
+    const int chance = luaL_checkint(ls, chance_par);
+    _chance_magnitude_check(ls, chance_par, chance);
+    try
+    {
+        map->_chance.add_range(depth, map_chance(chance_priority, chance));
+    }
+    catch (const std::string &error)
+    {
+        luaL_error(ls, error.c_str());
+    }
+    return (0);
+}
+
+#define WEIGHT(ls, n, weight) \
+    const int weight = luaL_checkint(ls, n); \
+    if (weight < 0)                          \
+        luaL_error(ls, "Bad weight: %d (must be >= 0)", weight);
+
 static int dgn_weight(lua_State *ls)
 {
     MAP(ls, 1, map);
     if (!lua_isnil(ls, 2))
-        map->weight = luaL_checkint(ls, 2);
-    PLUARET(number, map->weight);
+    {
+        WEIGHT(ls, 2, weight);
+        map->_weight.set_default(weight);
+    }
+    return 0;
+}
+
+static int dgn_depth_weight(lua_State *ls)
+{
+    MAP(ls, 1, map);
+    const std::string depth(luaL_checkstring(ls, 2));
+    WEIGHT(ls, 3, weight);
+    map->_weight.add_range(depth, weight);
+    return 0;
 }
 
 static int dgn_orient(lua_State *ls)
@@ -737,7 +788,7 @@ static int dgn_lfloorcol(lua_State *ls)
 
             return (0);
         }
-        map->floor_colour = (unsigned char) colour;
+        map->floor_colour = colour;
     }
     PLUARET(string, colour_to_str(map->floor_colour).c_str());
 }
@@ -771,7 +822,7 @@ static int dgn_lrockcol(lua_State *ls)
             return (0);
         }
 
-        map->rock_colour = (unsigned char) colour;
+        map->rock_colour = colour;
     }
     PLUARET(string, colour_to_str(map->rock_colour).c_str());
 }
@@ -814,10 +865,10 @@ static int dgn_change_floor_colour(lua_State *ls)
     const int colour = _lua_colour(ls, 1, BLACK);
     const bool update_now = _lua_boolean(ls, 2, false);
 
-    env.floor_colour = (unsigned char) colour;
+    env.floor_colour = colour;
 
     if (crawl_state.need_save && update_now)
-        viewwindow(false);
+        viewwindow();
     return (0);
 }
 
@@ -826,10 +877,10 @@ static int dgn_change_rock_colour(lua_State *ls)
     const int colour = _lua_colour(ls, 1, BLACK);
     const bool update_now = _lua_boolean(ls, 2, false);
 
-    env.rock_colour = (unsigned char) colour;
+    env.rock_colour = colour;
 
     if (crawl_state.need_save && update_now)
-        viewwindow(false);
+        viewwindow();
     return (0);
 }
 
@@ -909,10 +960,10 @@ static int dgn_terrain_changed(lua_State *ls)
     lua_isboolean(ls, 5)? lua_toboolean(ls, 5) : true;
     const bool preserve_items =
     lua_isboolean(ls, 6)? lua_toboolean(ls, 6) : true;
-    dungeon_terrain_changed( coord_def( luaL_checkint(ls, 1),
-                                       luaL_checkint(ls, 2) ),
+    dungeon_terrain_changed(coord_def(luaL_checkint(ls, 1),
+                                       luaL_checkint(ls, 2)),
                             type, affect_player,
-                            preserve_features, preserve_items );
+                            preserve_features, preserve_items);
     return (0);
 }
 
@@ -967,6 +1018,24 @@ static int dgn_fprop_at (lua_State *ls)
     return (1);
 }
 
+static int dgn_cloud_at (lua_State *ls)
+{
+    COORDS(c, 1, 2);
+
+    if (!in_bounds(c))
+        return (0);
+
+    int cloudno = env.cgrid(c);
+
+    if (cloudno == EMPTY_CLOUD)
+        lua_pushstring(ls, "none");
+    else
+        lua_pushstring(ls, cloud_name_at_index(cloudno).c_str());
+
+    return (1);
+}
+
+
 static int lua_dgn_set_lt_callback(lua_State *ls)
 {
     const char *level_type = luaL_checkstring(ls, 1);
@@ -984,12 +1053,16 @@ static int lua_dgn_set_lt_callback(lua_State *ls)
     return (0);
 }
 
-bool _valid_border_feat (dungeon_feature_type feat)
+static bool _valid_border_feat (dungeon_feature_type feat)
 {
     return ((feat <= DNGN_MAXWALL && feat >= DNGN_MINWALL)
-            || (feat == DNGN_TREES || feat == DNGN_OPEN_SEA
-               || feat == DNGN_LAVA || feat == DNGN_DEEP_WATER
-               || feat == DNGN_SHALLOW_WATER || feat == DNGN_FLOOR));
+            || (feat == DNGN_TREE
+               || feat == DNGN_SWAMP_TREE
+               || feat == DNGN_OPEN_SEA
+               || feat == DNGN_LAVA
+               || feat == DNGN_DEEP_WATER
+               || feat == DNGN_SHALLOW_WATER
+               || feat == DNGN_FLOOR));
 }
 
 // XXX: Currently, this is hacked so that map_def->border_fill_type is marsalled
@@ -1023,28 +1096,24 @@ static int dgn_fixup_stairs(lua_State *ls)
     dungeon_feature_by_name(luaL_checkstring(ls, 2));
 
     if (up_feat == DNGN_UNSEEN && down_feat == DNGN_UNSEEN)
-        return(0);
+        return 0;
 
-    for (int y = 0; y < GYM; ++y)
+    for (rectangle_iterator ri(0); ri; ++ri)
     {
-        for (int x = 0; x < GXM; ++x)
+        const dungeon_feature_type feat = grd(*ri);
+        if (feat_is_stone_stair(feat) || feat_is_escape_hatch(feat))
         {
-            const dungeon_feature_type feat = grd[x][y];
-            if (feat_is_stone_stair(feat) || feat_is_escape_hatch(feat))
+            dungeon_feature_type new_feat = DNGN_UNSEEN;
+
+            if (feat_stair_direction(feat) == CMD_GO_DOWNSTAIRS)
+                new_feat = down_feat;
+            else
+                new_feat = up_feat;
+
+            if (new_feat != DNGN_UNSEEN)
             {
-                dungeon_feature_type new_feat = DNGN_UNSEEN;
-
-                if (feat_stair_direction(feat) == CMD_GO_DOWNSTAIRS)
-                    new_feat = down_feat;
-                else
-                    new_feat = up_feat;
-
-                if (new_feat != DNGN_UNSEEN)
-                {
-                    grd[x][y] = new_feat;
-                    env.markers.add(new map_feature_marker(coord_def(x, y),
-                                                           new_feat));
-                }
+                grd(*ri) = new_feat;
+                env.markers.add(new map_feature_marker(*ri, new_feat));
             }
         }
     }
@@ -1081,32 +1150,28 @@ static int dgn_floor_halo(lua_State *ls)
     if (!error.empty())
     {
         luaL_argerror(ls, 2, error.c_str());
-        return(0);
+        return 0;
     }
 
-    for (int y = 0; y < GYM; ++y)
-        for (int x = 0; x < GXM; ++x)
+    for (rectangle_iterator ri(0); ri; ++ri)
+    {
+        if (grd(*ri) == target)
         {
-            const dungeon_feature_type feat = grd[x][y];
-            if (feat == target)
+            for (adjacent_iterator ai(*ri, false); ai; ++ai)
             {
+                if (!map_bounds(*ai))
+                    continue;
 
-                for (int i = -1; i <= 1; i++)
-                    for (int j = -1; j <= 1; j++)
-                    {
-                        if (!map_bounds(x+i, y+j))
-                            continue;
+                const dungeon_feature_type feat2 = grd(*ai);
 
-                        const dungeon_feature_type feat2 = grd[x+i][y+j];
-
-                        if (feat2 == DNGN_FLOOR
-                            || feat2 == DNGN_UNDISCOVERED_TRAP)
-                        {
-                            env.grid_colours[x+i][y+j] = colour;
-                        }
-                    }
+                if (feat2 == DNGN_FLOOR
+                    || feat2 == DNGN_UNDISCOVERED_TRAP)
+                {
+                    env.grid_colours(*ai) = colour;
+                }
             }
         }
+    }
 
 #ifdef USE_TILE
     unsigned int tile = get_tile_idx(ls, 3);
@@ -1205,7 +1270,7 @@ static cloud_type dgn_cloud_name_to_type(std::string name)
         return (CLOUD_DEBUGGING);
 
     for (int i = CLOUD_NONE; i < CLOUD_RANDOM; i++)
-        if (cloud_name(static_cast<cloud_type>(i)) == name)
+        if (cloud_type_name(static_cast<cloud_type>(i)) == name)
             return static_cast<cloud_type>(i);
 
     return (CLOUD_NONE);
@@ -1233,17 +1298,15 @@ static int lua_cloud_pow_max;
 static int lua_cloud_pow_rolls;
 
 static int make_a_lua_cloud(coord_def where, int garbage, int spread_rate,
-                            cloud_type ctype, kill_category whose,
-                            killer_type killer, int colour, std::string name,
-                            std::string tile)
+                            cloud_type ctype, const actor *agent, int colour,
+                            std::string name, std::string tile)
 {
-    UNUSED( garbage );
+    UNUSED(garbage);
 
     const int pow = random_range(lua_cloud_pow_min,
                                  lua_cloud_pow_max,
                                  lua_cloud_pow_rolls);
-    place_cloud( ctype, where, pow, whose, killer, spread_rate, colour, name,
-                 tile );
+    place_cloud(ctype, where, pow, agent, spread_rate, colour, name, tile);
     return 1;
 }
 
@@ -1314,7 +1377,7 @@ static int dgn_apply_area_cloud(lua_State *ls)
         return (0);
     }
 
-    if (kc == KC_NCATEGORIES)
+    if (kc == KC_NCATEGORIES || kc != KC_OTHER)
     {
         std::string error = "Invalid kill category '";
         error += kname;
@@ -1331,8 +1394,7 @@ static int dgn_apply_area_cloud(lua_State *ls)
     }
 
     apply_area_cloud(make_a_lua_cloud, coord_def(x, y), 0, size,
-                     ctype, kc, cloud_struct::whose_to_killer(kc),
-                     spread_rate, colour, name, tile);
+                     ctype, 0, spread_rate, colour, name, tile);
 
     return (0);
 }
@@ -1342,9 +1404,7 @@ static int dgn_delete_cloud(lua_State *ls)
     COORDS(c, 1, 2);
 
     if (in_bounds(c) && env.cgrid(c) != EMPTY_CLOUD)
-    {
         delete_cloud(env.cgrid(c));
-    }
 
     return (0);
 }
@@ -1382,7 +1442,7 @@ static int dgn_place_cloud(lua_State *ls)
         return (0);
     }
 
-    if (kc == KC_NCATEGORIES)
+    if (kc == KC_NCATEGORIES || kc != KC_OTHER)
     {
         std::string error = "Invalid kill category '";
         error += kname;
@@ -1398,7 +1458,7 @@ static int dgn_place_cloud(lua_State *ls)
         return (0);
     }
 
-    place_cloud(ctype, coord_def(x, y), cl_range, kc, spread_rate, colour, name, tile);
+    place_cloud(ctype, coord_def(x, y), cl_range, 0, spread_rate, colour, name, tile);
 
     return (0);
 }
@@ -1427,7 +1487,13 @@ static int dgn_register_feature_marker(lua_State *ls)
 {
     COORDS(c, 1, 2);
     FEAT(feat, 3);
-    env.markers.add( new map_feature_marker(c, feat) );
+    env.markers.add(new map_feature_marker(c, feat));
+    return (0);
+}
+
+static int _dgn_map_register_flag(lua_State *ls)
+{
+    map_register_flag(luaL_checkstring(ls, 1));
     return (0);
 }
 
@@ -1526,8 +1592,8 @@ LUAFN(dgn_with_map_anchors)
         {
             if (lua_isnumber(ls, i) && lua_isnumber(ls, i + 1))
                 map_anchor_points.push_back(
-                                            coord_def( lua_tointeger(ls, i),
-                                                      lua_tointeger(ls, i + 1) ) );
+                                            coord_def(lua_tointeger(ls, i),
+                                                      lua_tointeger(ls, i + 1)));
         }
 
         ASSERT(lua_isfunction(ls, -1));
@@ -1592,10 +1658,16 @@ LUAFN(_dgn_place_map)
         COORDS(c, 4, 5);
         where = c;
     }
-    if (dgn_place_map(map, clobber, no_exits, where) && !Level_Vaults.empty())
-        lua_pushlightuserdata(ls, &Level_Vaults[Level_Vaults.size() - 1]);
+    if (dgn_place_map(map, clobber, no_exits, where)
+        && !env.level_vaults.empty())
+    {
+        lua_pushlightuserdata(ls,
+                              env.level_vaults[env.level_vaults.size() - 1]);
+    }
     else
+    {
         lua_pushnil(ls);
+    }
     return (1);
 }
 
@@ -1603,7 +1675,7 @@ LUAFN(_dgn_in_vault)
 {
     GETCOORD(c, 1, 2, map_bounds);
     const int mask = lua_isnone(ls, 3) ? MMT_VAULT : lua_tointeger(ls, 3);
-    lua_pushboolean(ls, dgn_Map_Mask(c) & mask);
+    lua_pushboolean(ls, env.level_map_mask(c) & mask);
     return (1);
 }
 
@@ -1612,14 +1684,30 @@ LUAFN(_dgn_map_parameters)
     return clua_stringtable(ls, map_parameters);
 }
 
-int dgn_push_vault_placement(lua_State *ls, const vault_placement &vp)
+int dgn_push_vault_placement(lua_State *ls, const vault_placement *vp)
 {
-    return dlua_push_object_type(ls, VAULT_PLACEMENT_METATABLE, vp);
+    return dlua_push_object_type(ls, VAULT_PLACEMENT_METATABLE, *vp);
 }
 
 LUAFN(_dgn_maps_used_here)
 {
-    return clua_gentable(ls, Level_Vaults, dgn_push_vault_placement);
+    return clua_gentable(ls, env.level_vaults, dgn_push_vault_placement);
+}
+
+LUAFN(_dgn_vault_at)
+{
+    GETCOORD(c, 1, 2, map_bounds);
+    vault_placement *place = dgn_vault_at(c);
+    if (place)
+    {
+        dgn_push_vault_placement(ls, place);
+    }
+    else
+    {
+        lua_pushnil(ls);
+    }
+
+    return 1;
 }
 
 LUAFN(_dgn_find_marker_position_by_prop)
@@ -1685,24 +1773,6 @@ LUAFN(_dgn_marker_at_pos)
     return (1);
 }
 
-extern spec_room lua_special_room_spec;
-extern int       lua_special_room_level;
-
-LUAFN(dgn_get_special_room_info)
-{
-    if (!lua_special_room_spec.created || !in_bounds(lua_special_room_spec.tl)
-        || lua_special_room_level == -1)
-    {
-        return (0);
-    }
-
-    lua_pushnumber(ls,  lua_special_room_level);
-    dlua_push_coordinates(ls, lua_special_room_spec.tl);
-    dlua_push_coordinates(ls, lua_special_room_spec.br);
-
-    return (5);
-}
-
 LUAFN(dgn_is_validating)
 {
     MAP(ls, 1, map);
@@ -1745,8 +1815,7 @@ LUAFN(_dgn_reuse_map)
     if (!lua_isuserdata(ls, 1))
         luaL_argerror(ls, 1, "Expected vault_placement");
 
-    vault_placement &vp(
-                        *static_cast<vault_placement*>(lua_touserdata(ls, 1)));
+    vault_placement &vp(*static_cast<vault_placement*>(lua_touserdata(ls, 1)));
 
     COORDS(place, 2, 3);
 
@@ -1806,10 +1875,12 @@ LUAFN(dgn_fill_grd_area)
 
 LUAFN(dgn_apply_tide)
 {
-    shoals_apply_tides(0, true);
+    shoals_apply_tides(0, true, true);
     return (0);
 }
 
+LUAWRAP(dgn_mark_game_won, crawl_state.mark_last_game_won())
+
 const struct luaL_reg dgn_dlib[] =
 {
 { "reset_level", _dgn_reset_level },
@@ -1818,12 +1889,16 @@ const struct luaL_reg dgn_dlib[] =
 { "name", dgn_name },
 { "depth", dgn_depth },
 { "place", dgn_place },
+{ "desc", dgn_desc },
 { "tags",  dgn_tags },
+{ "has_tag", dgn_has_tag },
 { "tags_remove", dgn_tags_remove },
 { "lflags", dgn_lflags },
 { "bflags", dgn_bflags },
 { "chance", dgn_chance },
+{ "depth_chance", dgn_depth_chance },
 { "weight", dgn_weight },
+{ "depth_weight", dgn_depth_weight },
 { "welcome", dgn_welcome },
 { "orient", dgn_orient },
 { "shuffle", dgn_shuffle },
@@ -1849,6 +1924,7 @@ const struct luaL_reg dgn_dlib[] =
 
 { "colour_at", dgn_colour_at },
 { "fprop_at", dgn_fprop_at },
+{ "cloud_at", dgn_cloud_at },
 
 { "terrain_changed", dgn_terrain_changed },
 { "fprop_changed", dgn_fprop_changed },
@@ -1881,6 +1957,7 @@ const struct luaL_reg dgn_dlib[] =
 
 { "is_passable", _dgn_is_passable },
 
+{ "map_register_flag", _dgn_map_register_flag },
 { "register_feature_marker", dgn_register_feature_marker },
 { "register_lua_marker", dgn_register_lua_marker },
 
@@ -1899,6 +1976,7 @@ const struct luaL_reg dgn_dlib[] =
 { "map_parameters", _dgn_map_parameters },
 
 { "maps_used_here", _dgn_maps_used_here },
+{ "vault_at", _dgn_vault_at },
 
 { "find_marker_position_by_prop", _dgn_find_marker_position_by_prop },
 { "find_marker_positions_by_prop", _dgn_find_marker_positions_by_prop },
@@ -1906,19 +1984,20 @@ const struct luaL_reg dgn_dlib[] =
 
 { "marker_at_pos", _dgn_marker_at_pos },
 
-{ "get_special_room_info", dgn_get_special_room_info },
 { "is_validating", dgn_is_validating },
 
 { "fill_grd_area", dgn_fill_grd_area },
 
 { "apply_tide", dgn_apply_tide },
 
+{ "mark_game_won", dgn_mark_game_won },
+
 { NULL, NULL }
 };
 
 #define VP(name) \
     vault_placement &name =                                             \
-        **clua_get_userdata<vault_placement*>(                          \
+        **clua_get_userdata<vault_placement*>(                       \
             ls, VAULT_PLACEMENT_METATABLE)
 
 LUAFN(_vp_pos)
@@ -1960,18 +2039,6 @@ LUAFN(_vp_level_number)
     PLUARET(number, vp.level_number)
 }
 
-LUAFN(_vp_num_runes)
-{
-    VP(vp);
-    PLUARET(number, vp.num_runes)
-}
-
-LUAFN(_vp_rune_subst)
-{
-    VP(vp);
-    PLUARET(number, vp.rune_subst)
-}
-
 static const luaL_reg dgn_vaultplacement_ops[] =
 {
     { "pos", _vp_pos },
@@ -1980,8 +2047,6 @@ static const luaL_reg dgn_vaultplacement_ops[] =
     { "map", _vp_map },
     { "exits", _vp_exits },
     { "level_number", _vp_level_number },
-    { "num_runes", _vp_num_runes },
-    { "rune_subst", _vp_rune_subst },
     { NULL, NULL }
 };
 
@@ -1990,7 +2055,7 @@ static void _dgn_register_metatables(lua_State *ls)
     clua_register_metatable(ls,
                             VAULT_PLACEMENT_METATABLE,
                             dgn_vaultplacement_ops,
-                            lua_object_gc<vault_placement*>);
+                            lua_object_gc<vault_placement>);
 }
 
 void dluaopen_dgn(lua_State *ls)
