@@ -1,16 +1,18 @@
-/*
- * File:       l_dgnbld.cc
- * Summary:    Building routines (library "dgn").
- */
+/**
+ * @file
+ * @brief Building routines (library "dgn").
+**/
 
 #include "AppHdr.h"
 
 #include <cmath>
 
+#include "dungeon.h"
+#include "dgn-shoals.h"
+#include "dgn-swamp.h"
 #include "cluautil.h"
 #include "coord.h"
 #include "coordit.h"
-#include "dungeon.h"
 #include "l_libs.h"
 #include "mapdef.h"
 #include "random.h"
@@ -156,7 +158,7 @@ static int _fill_area (lua_State *ls, map_lines &lines, int x1, int y1, int x2,
 }
 
 // Specifically only deals with horizontal lines.
-std::vector<coord_def> _box_side (int x1, int y1, int x2, int y2, int side)
+static std::vector<coord_def> _box_side (int x1, int y1, int x2, int y2, int side)
 {
     std::vector<coord_def> line;
 
@@ -168,7 +170,7 @@ std::vector<coord_def> _box_side (int x1, int y1, int x2, int y2, int side)
     case 1: start_x = x2; start_y = y1; stop_x = x2; stop_y = y2; break;
     case 2: start_x = x1; start_y = y2; stop_x = x2; stop_y = y2; break;
     case 3: start_x = x1; start_y = y1; stop_x = x1; stop_y = y2; break;
-    default: ASSERT(!"invalid _box_side"); return (line);
+    default: die("invalid _box_side");
     }
 
     x = start_x; y = start_y;
@@ -873,27 +875,11 @@ LUAFN(dgn_spotty_map)
                && strchr(replace, lines(x, y-2))
                && strchr(replace, lines(x, y+2)));
 
-        if (strchr(replace, lines(x, y)))
-            lines(x, y) = fill;
-        if (strchr(replace, lines(x, y-1)))
-            lines(x, y-1) = fill;
-        if (strchr(replace, lines(x, y+1)))
-            lines(x, y+1) = fill;
-        if (strchr(replace, lines(x-1, y)))
-            lines(x-1, y) = fill;
-        if (strchr(replace, lines(x+1, y)))
-            lines(x+1, y) = fill;
-
-        if (boxy)
+        for (radius_iterator ai(coord_def(x, y), 1, boxy ? C_SQUARE : C_POINTY,
+                                NULL, false); ai; ++ai)
         {
-            if (strchr(replace, lines(x-1, y-1)))
-                lines(x-1, y-1) = fill;
-            if (strchr(replace, lines(x+1, y+1)))
-                lines(x+1, y+1) = fill;
-            if (strchr(replace, lines(x-1, y+1)))
-                lines(x-1, y+1) = fill;
-            if (strchr(replace, lines(x+1, y-1)))
-                lines(x+1, y-1) = fill;
+            if (strchr(replace, lines(*ai)))
+                lines(*ai) = fill;
         }
     }
 
@@ -906,6 +892,45 @@ static int dgn_width(lua_State *ls)
     PLUARET(number, lines.width());
 }
 
+LUAFN(dgn_layout_type)
+{
+    env.level_layout_types.insert(luaL_checkstring(ls, 2));
+    return 0;
+}
+
+/* Wrappers for C++ layouts, to facilitate choosing of layouts by weight and
+ * depth */
+
+LUAFN(dgn_layout_basic)
+{
+    builder_basic(you.absdepth0);
+    return 0;
+}
+
+LUAFN(dgn_layout_bigger_room)
+{
+    bigger_room();
+    return 0;
+}
+
+LUAFN(dgn_layout_plan_4)
+{
+    plan_4(NUM_FEATURES);
+    return 0;
+}
+
+LUAFN(dgn_layout_shoals)
+{
+    dgn_build_shoals_level(you.absdepth0);
+    return 0;
+}
+
+LUAFN(dgn_layout_swamp)
+{
+    dgn_build_swamp_level(you.absdepth0);
+    return 0;
+}
+
 const struct luaL_reg dgn_build_dlib[] =
 {
     { "count_feature_in_box", &dgn_count_feature_in_box },
@@ -934,6 +959,13 @@ const struct luaL_reg dgn_build_dlib[] =
     { "smear_map", &dgn_smear_map },
     { "spotty_map", &dgn_spotty_map },
     { "width", dgn_width },
+    { "layout_type", &dgn_layout_type },
+
+    { "layout_basic", &dgn_layout_basic },
+    { "layout_bigger_room", &dgn_layout_bigger_room },
+    { "layout_plan_4", &dgn_layout_plan_4 },
+    { "layout_shoals", &dgn_layout_shoals },
+    { "layout_swamp", &dgn_layout_swamp },
 
     { NULL, NULL }
 };
