@@ -1,7 +1,7 @@
-/*
- *  File:       libutil.cc
- *  Summary:    Functions that may be missing from some systems
- */
+/**
+ * @file
+ * @brief Functions that may be missing from some systems
+**/
 
 #include "AppHdr.h"
 
@@ -12,6 +12,7 @@
 #include "macro.h"
 #include "message.h"
 #include "stuff.h"
+#include "unicode.h"
 #include "viewgeom.h"
 
 #include <sstream>
@@ -102,12 +103,12 @@ bool shell_safe(const char *file)
     return (match < 0 || !file[match]);
 }
 
-void play_sound( const char *file )
+void play_sound(const char *file)
 {
 #if defined(WINMM_PLAY_SOUNDS)
     // Check whether file exists, is readable, etc.?
     if (file && *file)
-        sndPlaySound(file, SND_ASYNC | SND_NODEFAULT);
+        sndPlaySoundW(utf8_to_16(file).c_str(), SND_ASYNC | SND_NODEFAULT);
 
 #elif defined(SOUND_PLAY_COMMAND)
     char command[255];
@@ -116,7 +117,7 @@ void play_sound( const char *file )
         && shell_safe(file))
     {
         snprintf(command, sizeof command, SOUND_PLAY_COMMAND, file);
-        system(command);
+        system(OUTS(command));
     }
 #endif
 }
@@ -128,7 +129,7 @@ std::string strip_filename_unsafe_chars(const std::string &s)
 
 std::string vmake_stringf(const char* s, va_list args)
 {
-    char buf1[400];
+    char buf1[8000];
     va_list orig_args;
     va_copy(orig_args, args);
     size_t len = vsnprintf(buf1, sizeof buf1, s, orig_args);
@@ -187,6 +188,24 @@ std::string &escape_path_spaces(std::string &s)
     return s;
 }
 
+void wait_for_keypress()
+{
+    // Double getchm() was necessary if first call returned zero; this
+    // should theoretically be needed only for DOS.
+    getchm() || getchm();
+}
+
+bool key_is_escape(int key)
+{
+    switch (key)
+    {
+    CASE_ESCAPE
+        return (true);
+    default:
+        return (false);
+    }
+}
+
 std::string &uppercase(std::string &s)
 {
     for (unsigned i = 0, sz = s.size(); i < sz; ++i)
@@ -254,6 +273,17 @@ extern "C" int stricmp(const char *str1, const char *str2)
 }
 #endif
 
+bool strip_suffix(std::string &s, const std::string &suffix)
+{
+    if (ends_with(s, suffix))
+    {
+        s.erase(s.length() - suffix.length(), suffix.length());
+        trim_string(s);
+        return (true);
+    }
+    return false;
+}
+
 // Returns true if s contains tag 'tag', and strips out tag from s.
 bool strip_tag(std::string &s, const std::string &tag, bool skip_padding)
 {
@@ -296,6 +326,22 @@ bool strip_tag(std::string &s, const std::string &tag, bool skip_padding)
     return (false);
 }
 
+std::vector<std::string> strip_multiple_tag_prefix (std::string &s, const std::string &tagprefix)
+{
+    std::vector<std::string> results;
+
+    while (true)
+    {
+        std::string this_result = strip_tag_prefix(s, tagprefix);
+        if (this_result.empty())
+            break;
+
+        results.push_back(this_result);
+    }
+
+    return results;
+}
+
 std::string strip_tag_prefix(std::string &s, const std::string &tagprefix)
 {
     std::string::size_type pos = s.find(tagprefix);
@@ -405,6 +451,10 @@ std::string pluralise(const std::string &name,
         // Vortex; vortexes is legal, but the classic plural is cooler.
         return name.substr(0, name.length() - 2) + "ices";
     }
+    else if (ends_with(name, "mosquito"))
+    {
+        return name + "es";
+    }
     else if (ends_with(name, "cyclops"))
     {
         return name.substr(0, name.length() - 1) + "es";
@@ -447,10 +497,12 @@ std::string pluralise(const std::string &name,
         // mage -> magi
         return name.substr(0, name.length() - 1) + "i";
     }
-    else if (ends_with(name, "sheep") || ends_with(name, "manes")
-              || ends_with(name, "fish") || ends_with(name, "folk"))
+    else if (ends_with(name, "sheep") || ends_with(name, "fish")
+             || ends_with(name, "folk") || ends_with(name, "spawn")
+             || ends_with(name, "kenku") || ends_with(name, "shedu")
+             // "shedu" is male, "lammasu" is female of the same creature
+             || ends_with(name, "lammasu") || ends_with(name, "lamassu"))
     {
-        // Maybe we should generalise 'manes' to ends_with("es")?
         return name;
     }
     else if (ends_with(name, "ch") || ends_with(name, "sh")
@@ -460,9 +512,10 @@ std::string pluralise(const std::string &name,
         // ending with sh (except fish, which are caught in the previous check).
         return name + "es";
     }
-    else if (ends_with(name, "simulacrum"))
+    else if (ends_with(name, "simulacrum") || ends_with(name, "eidolon"))
     {
-        // simulacrum -> simulacra
+        // simulacrum -> simulacra (correct Latin pluralisation)
+        // also eidolon -> eidola (correct Greek pluralisation)
         return name.substr(0, name.length() - 2) + "a";
     }
     else if (ends_with(name, "efreet"))
@@ -472,10 +525,40 @@ std::string pluralise(const std::string &name,
     }
     else if (name == "foot")
         return "feet";
+    else if (name == "ophan" || name == "cherub" || name == "seraph")
+    {
+        // unlike "angel" which is fully assimilated and "cherub" which may be
+        // pluralized both ways, "ophan" always uses hebrew pluralization
+        return name + "im";
+    }
 
     return name + "s";
 }
 
+std::string apostrophise(const std::string &name)
+{
+    if (name.empty())
+        return (name);
+
+    if (name == "you" || name == "You")
+        return (name + "r");
+
+    if (name == "it" || name == "It")
+        return (name + "s");
+
+    const char lastc = name[name.length() - 1];
+    return (name + (lastc == 's' ? "'" : "'s"));
+}
+
+std::string apostrophise_fixup(const std::string &msg)
+{
+    if (msg.empty())
+        return (msg);
+
+    // XXX: This is rather hackish.
+    return (replace_all(msg, "s's", "s'"));
+}
+
 static std::string pow_in_words(int pow)
 {
     switch (pow)
@@ -554,7 +637,7 @@ std::string replace_all(std::string s,
 
     while ((found = s.find(find, start)) != std::string::npos)
     {
-        s.replace( found, find.length(), repl );
+        s.replace(found, find.length(), repl);
         start = found + repl.length();
     }
 
@@ -572,7 +655,7 @@ std::string replace_all_of(std::string s,
 
     while ((found = s.find_first_of(tofind, start)) != std::string::npos)
     {
-        s.replace( found, 1, replacement );
+        s.replace(found, 1, replacement);
         start = found + replacement.length();
     }
 
@@ -593,28 +676,28 @@ int count_occurrences(const std::string &text, const std::string &s)
     return (nfound);
 }
 
-std::string trimmed_string( std::string s )
+std::string trimmed_string(std::string s)
 {
     trim_string(s);
     return (s);
 }
 
 // also used with macros
-std::string &trim_string( std::string &str )
+std::string &trim_string(std::string &str)
 {
-    str.erase( 0, str.find_first_not_of( " \t\n\r" ) );
-    str.erase( str.find_last_not_of( " \t\n\r" ) + 1 );
+    str.erase(0, str.find_first_not_of(" \t\n\r"));
+    str.erase(str.find_last_not_of(" \t\n\r") + 1);
 
     return (str);
 }
 
-std::string &trim_string_right( std::string &str )
+std::string &trim_string_right(std::string &str)
 {
-    str.erase( str.find_last_not_of( " \t\n\r" ) + 1 );
+    str.erase(str.find_last_not_of(" \t\n\r") + 1);
     return (str);
 }
 
-static void add_segment( std::vector<std::string> &segs,
+static void add_segment(std::vector<std::string> &segs,
                          std::string s,
                          bool trim,
                          bool accept_empty)
@@ -626,7 +709,7 @@ static void add_segment( std::vector<std::string> &segs,
         segs.push_back(s);
 }
 
-std::vector<std::string> split_string( const std::string &sep,
+std::vector<std::string> split_string(const std::string &sep,
                                        std::string s,
                                        bool trim_segments,
                                        bool accept_empty_segments,
@@ -653,10 +736,65 @@ std::vector<std::string> split_string( const std::string &sep,
     return segments;
 }
 
+// The provided string is consumed!
+std::string wordwrap_line(std::string &s, int width)
+{
+    const char *cp0 = s.c_str();
+    const char *cp = cp0, *space = 0;
+    ucs_t c;
+
+    while (int clen = utf8towc(&c, cp))
+    {
+        int cw = wcwidth(c);
+        if (c == ' ')
+            space = cp;
+        else if (c == '\n')
+        {
+            space = cp;
+            break;
+        }
+        if (cw > width)
+            break;
+        if (cw >= 0)
+            width -= cw;
+        cp += clen;
+    }
+
+    if (!c)
+    {
+        // everything fits
+        std::string ret = s;
+        s.clear();
+        return ret;
+    }
+
+    if (space)
+        cp = space;
+    const std::string ret = s.substr(0, cp - cp0);
+
+    // eat all trailing spaces and up to one newline
+    while (*cp == ' ')
+        cp++;
+    if (*cp == '\n')
+        cp++;
+    s.erase(0, cp - cp0);
+
+    return ret;
+}
+
 // The old school way of doing short delays via low level I/O sync.
 // Good for systems like old versions of Solaris that don't have usleep.
 #ifdef NEED_USLEEP
 
+# ifdef TARGET_OS_WINDOWS
+void usleep(unsigned long time)
+{
+    ASSERT(time > 0);
+    ASSERT(!(time % 1000));
+    Sleep(time/1000);
+}
+# else
+
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/unistd.h>
@@ -670,6 +808,7 @@ void usleep(unsigned long time)
 
     select(0, NULL, NULL, NULL, &timer);
 }
+# endif
 #endif
 
 #ifndef USE_TILE
@@ -705,8 +844,8 @@ void cgotoxy(int x, int y, GotoRegion region)
     const coord_def tl = cgettopleft(region);
     const coord_def sz = cgetsize(region);
 
-    ASSERT(x >= 1 && x <= sz.x);
-    ASSERT(y >= 1 && y <= sz.y);
+    ASSERT_SAVE(x >= 1 && x <= sz.x);
+    ASSERT_SAVE(y >= 1 && y <= sz.y);
 
     gotoxy_sys(tl.x + x - 1, tl.y + y - 1);
 }
@@ -744,3 +883,87 @@ void cscroll(int n, GotoRegion region)
 
 
 mouse_mode mouse_control::ms_current_mode = MOUSE_MODE_NORMAL;
+
+size_t strlcpy(char *dst, const char *src, size_t n)
+{
+    if (!n)
+        return strlen(src);
+
+    const char *s = src;
+
+    while (--n > 0)
+        if (!(*dst++ = *s++))
+            break;
+
+    if (!n)
+    {
+        *dst++ = 0;
+        while (*s++)
+            ;
+    }
+
+    return s - src - 1;
+}
+
+#ifdef TARGET_OS_WINDOWS
+// FIXME: This function should detect if aero is running, but the DwmIsCompositionEnabled
+// function isn't included in msys, so I don't know how to do that. Instead, I just check
+// if we are running vista or higher. -rla
+bool _is_aero()
+{
+    OSVERSIONINFOEX osvi;
+    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
+    if (GetVersionEx((OSVERSIONINFO *) &osvi))
+        return (osvi.dwMajorVersion >= 6);
+    else
+        return false;
+}
+
+taskbar_pos get_taskbar_pos()
+{
+    RECT rect;
+    HWND taskbar = FindWindow("Shell_traywnd", NULL);
+    if (taskbar && GetWindowRect(taskbar, &rect))
+    {
+        if (rect.right - rect.left > rect.bottom - rect.top)
+        {
+            if (rect.top > 0)
+                return TASKBAR_BOTTOM;
+            else
+                return TASKBAR_TOP;
+        }
+        else
+        {
+            if (rect.left > 0)
+                return TASKBAR_RIGHT;
+            else
+                return TASKBAR_LEFT;
+        }
+    }
+    return TASKBAR_NO;
+}
+
+int get_taskbar_size()
+{
+    RECT rect;
+    int size;
+    taskbar_pos tpos = get_taskbar_pos();
+    HWND taskbar = FindWindow("Shell_traywnd", NULL);
+
+    if (taskbar && GetWindowRect(taskbar, &rect))
+    {
+        if (tpos & TASKBAR_H)
+                size = rect.bottom - rect.top;
+        else if (tpos & TASKBAR_V)
+                size = rect.right - rect.left;
+        else
+            return 0;
+
+        if (_is_aero())
+            size += 3; // Taskbar behave strangely when aero is active.
+
+        return size;
+    }
+    return 0;
+}
+#endif
