@@ -1,19 +1,17 @@
-/*
- *  File:       macro.cc
- *  Summary:    Crude macro-capability
- *  Written by: Juho Snellman <jsnell@lyseo.edu.ouka.fi>
- */
+/**
+ * @file
+ * @brief Crude macro-capability
+**/
 
 /*
  * The macro-implementation works like this:
- *   - For generic game code, #define getch() getchm().
  *   - getchm() works by reading characters from an internal
  *     buffer. If none are available, new characters are read into
  *     the buffer with _getch_mul().
  *   - _getch_mul() reads at least one character, but will read more
  *     if available (determined using kbhit(), which should be defined
  *     in the platform specific libraries).
- *   - Before adding the characters read into the buffer, any macros
+ *   - Before adding the characters read into the buffer, any keybindings
  *     in the sequence are replaced (see macro_add_buf_long for the
  *     details).
  *
@@ -22,11 +20,9 @@
 
 #include "AppHdr.h"
 
-#define MACRO_CC
 #include "macro.h"
+#include "state.h"
 
-#include <iostream>
-#include <fstream>
 #include <string>
 #include <sstream>
 #include <map>
@@ -39,11 +35,14 @@
 
 #include "cio.h"
 #include "externs.h"
+#include "files.h"
 #include "libutil.h"
 #include "options.h"
 #include "message.h"
 #include "state.h"
 #include "stuff.h"
+#include "syscalls.h"
+#include "unicode.h"
 
 // for trim_string:
 #include "initfile.h"
@@ -65,6 +64,7 @@ static macromap *all_maps[] =
 };
 
 static keybuf Buffer;
+static keybuf SendKeysBuffer;
 
 #define USERFUNCBASE -10000
 static std::vector<std::string> userfunctions;
@@ -103,7 +103,7 @@ typedef std::map<int, int> cmd_to_key_map;
 static key_to_cmd_map _keys_to_cmds[KMC_CONTEXT_COUNT];
 static cmd_to_key_map _cmds_to_keys[KMC_CONTEXT_COUNT];
 
-inline int userfunc_index(int key)
+static inline int userfunc_index(int key)
 {
     int index = (key <= USERFUNCBASE? USERFUNCBASE - key : -1);
     return (index < 0 || index >= (int) userfunctions.size()? -1 : index);
@@ -214,7 +214,11 @@ static std::string get_macro_file()
 
 #if defined(DGL_MACRO_ABSOLUTE_PATH)
     return (dir.empty()? "macro.txt" : dir);
-#elif defined(DGL_NAMED_MACRO_FILE)
+#endif
+
+    check_mkdir("Macro directory", &dir, true);
+
+#if defined(DGL_NAMED_MACRO_FILE)
     return (dir + strip_filename_unsafe_chars(you.your_name) + "-macro.txt");
 #else
     return (dir + "macro.txt");
@@ -234,13 +238,13 @@ static void buf2keyseq(const char *buff, keyseq &k)
     {
         int index = userfunc_getindex(buff + 3);
         if (index != -1)
-            k.push_back( USERFUNCBASE - index );
+            k.push_back(USERFUNCBASE - index);
     }
     else
     {
-        const int len = strlen( buff );
+        const int len = strlen(buff);
         for (int i = 0; i < len; i++)
-            k.push_back( buff[i] );
+            k.push_back(buff[i]);
     }
 }
 
@@ -275,11 +279,10 @@ static int read_key_code(std::string s)
  *   \{x40} produces 64 (hexadecimal code)
  *   \{!more} or \{!m} disables -more- prompt until the end of the macro.
  */
-static keyseq parse_keyseq( std::string s )
+static keyseq parse_keyseq(std::string s)
 {
     int state = 0;
     keyseq v;
-    int num;
 
     if (s.find("===") == 0)
     {
@@ -308,10 +311,7 @@ static keyseq parse_keyseq( std::string s )
                 v.push_back(c);
             }
             else if (c == '{')
-            {
                 state = 2;
-                num = 0;
-            }
             // XXX Error handling
             break;
 
@@ -347,7 +347,7 @@ static keyseq parse_keyseq( std::string s )
  * Serialises a key sequence into a string of the format described
  * above.
  */
-static std::string vtostr( const keyseq &seq )
+static std::string vtostr(const keyseq &seq)
 {
     std::ostringstream s;
 
@@ -372,7 +372,7 @@ static std::string vtostr( const keyseq &seq )
             else
             {
                 char buff[20];
-                snprintf( buff, sizeof(buff), "\\{%d}", *i );
+                snprintf(buff, sizeof(buff), "\\{%d}", *i);
                 s << buff;
             }
         }
@@ -388,32 +388,53 @@ static std::string vtostr( const keyseq &seq )
 /*
  * Add a macro (suprise, suprise).
  */
-static void macro_add( macromap &mapref, keyseq key, keyseq action )
+static void macro_add(macromap &mapref, keyseq key, keyseq action)
 {
     mapref[key] = action;
 }
 
-static void macro_add( macromap &mapref, keyseq key, const char *buff )
-{
-    keyseq  act;
-    buf2keyseq(buff, act);
-    if (!act.empty())
-        macro_add( mapref, key, act );
-}
-
 /*
  * Remove a macro.
  */
-static void macro_del( macromap &mapref, keyseq key )
+static bool macro_del(macromap &mapref, keyseq key)
 {
-    mapref.erase( key );
+    return mapref.erase(key) != 0;
+}
+
+static void _register_expanded_keys(int num, bool reverse);
+
+// Safely add macro-expanded keystrokes to the end of Crawl's keyboard
+// buffer. Macro-expanded keystrokes will be held in a separate
+// keybuffer until the primary keybuffer is empty, at which point
+// they'll be added to the primary keybuffer.
+void macro_sendkeys_end_add_expanded(int key)
+{
+    SendKeysBuffer.push_back(key);
+}
+
+// Effectively the same as macro_sendkeys_end_add_expanded, but adding to
+// the front of the keybuffer.
+void macro_sendkeys_start_add_expanded(int key)
+{
+    macro_buf_add(key, true, true);
+}
+
+static void _macro_inject_sent_keys()
+{
+    if (Buffer.empty() && !SendKeysBuffer.empty())
+    {
+        // Transfer keys from SendKeysBuffer to the main Buffer as
+        // expanded keystrokes and empty SendKeysBuffer.
+        macro_buf_add(SendKeysBuffer, false, true);
+        SendKeysBuffer.clear();
+    }
 }
 
 /*
  * Adds keypresses from a sequence into the internal keybuffer. Ignores
  * macros.
  */
-void macro_buf_add( const keyseq &actions, bool reverse)
+void macro_buf_add(const keyseq &actions, bool reverse, bool expanded)
 {
     keyseq act;
     bool need_more_reset = false;
@@ -431,19 +452,24 @@ void macro_buf_add( const keyseq &actions, bool reverse)
     if (need_more_reset)
         act.push_back(KEY_MACRO_ENABLE_MORE);
 
-    Buffer.insert( reverse? Buffer.begin() : Buffer.end(),
-                   act.begin(), act.end() );
+    Buffer.insert(reverse? Buffer.begin() : Buffer.end(),
+                   act.begin(), act.end());
+
+    if (expanded)
+        _register_expanded_keys(act.size(), reverse);
 }
 
 /*
  * Adds a single keypress into the internal keybuffer.
  */
-void macro_buf_add( int key, bool reverse )
+void macro_buf_add(int key, bool reverse, bool expanded)
 {
     if (reverse)
-        Buffer.push_front( key );
+        Buffer.push_front(key);
     else
-        Buffer.push_back( key );
+        Buffer.push_back(key);
+    if (expanded)
+        _register_expanded_keys(1, reverse);
 }
 
 /*
@@ -456,15 +482,15 @@ void macro_buf_add_cmd(command_type cmd, bool reverse)
     // There should be plenty of room between the synthetic keys
     // (KEY_MACRO_MORE_PROTECT == -10) and USERFUNCBASE (-10000) for
     // command_type to fit (currently 1000 through 2069).
-    macro_buf_add( -((int) cmd), reverse);
+    macro_buf_add(-((int) cmd), reverse, true);
 }
 
 /*
  * Adds keypresses from a sequence into the internal keybuffer. Does some
- * O(N^2) analysis to the sequence to replace macros.
+ * O(N^2) analysis to the sequence to apply keymaps.
  */
-static void macro_buf_add_long( keyseq actions,
-                                macromap &keymap = Keymaps[KMC_DEFAULT] )
+static void macro_buf_add_long(keyseq actions,
+                               macromap &keymap = Keymaps[KMC_DEFAULT])
 {
     keyseq tmp;
 
@@ -488,7 +514,7 @@ static void macro_buf_add_long( keyseq actions,
             // macro into the buffer.
             if (subst != keymap.end() && !subst->second.empty())
             {
-                macro_buf_add(subst->second);
+                macro_buf_add(subst->second, false, false);
                 break;
             }
 
@@ -501,7 +527,7 @@ static void macro_buf_add_long( keyseq actions,
         {
             // Didn't find a macro. Add the first keypress of the sequence
             // into the buffer, remove it from the sequence, and try again.
-            macro_buf_add( actions.front() );
+            macro_buf_add(actions.front(), false, false);
             actions.pop_front();
         }
         else
@@ -514,8 +540,36 @@ static void macro_buf_add_long( keyseq actions,
     }
 }
 
+// Number of keys from start of buffer that have already gone through
+// macro expansion.
+static int expanded_keys_left = 0;
+static void _register_expanded_keys(int num, bool reverse)
+{
+    expanded_keys_left += num;
+    if (!reverse)
+    {
+        // We added at the end of the buffer, so the whole buffer had
+        // better be full of expanded keys.
+        ASSERT((int)Buffer.size() == expanded_keys_left);
+        expanded_keys_left = Buffer.size();
+    }
+}
+
 static int macro_keys_left = -1;
 
+void macro_clear_buffers()
+{
+    crawl_state.cancel_cmd_repeat();
+    crawl_state.cancel_cmd_again();
+
+    Buffer.clear();
+    SendKeysBuffer.clear();
+    expanded_keys_left = 0;
+    macro_keys_left = -1;
+
+    crawl_state.show_more_prompt = true;
+}
+
 bool is_processing_macro()
 {
     return (macro_keys_left >= 0);
@@ -525,9 +579,12 @@ bool is_processing_macro()
  * Command macros are only applied from the immediate front of the
  * buffer, and only when the game is expecting a command.
  */
-static void macro_buf_apply_command_macro( void )
+static void macro_buf_apply_command_macro()
 {
-    keyseq  tmp = Buffer;
+    if (macro_keys_left > 0 || expanded_keys_left > 0)
+        return;
+
+    keyseq tmp = Buffer;
 
     // find the longest match from the start of the buffer and replace it
     while (tmp.size() > 0)
@@ -540,17 +597,11 @@ static void macro_buf_apply_command_macro( void )
 
             // Found macro, remove match from front:
             for (unsigned int i = 0; i < tmp.size(); i++)
-            {
                 Buffer.pop_front();
-                if (macro_keys_left >= 0)
-                    macro_keys_left--;
-            }
 
-            if (macro_keys_left == -1)
-                macro_keys_left = 0;
-            macro_keys_left += result.size();
+            macro_keys_left = result.size();
 
-            macro_buf_add(result, true);
+            macro_buf_add(result, true, true);
 
             break;
         }
@@ -563,8 +614,13 @@ static void macro_buf_apply_command_macro( void )
  * Removes the earliest keypress from the keybuffer, and returns its
  * value. If buffer was empty, returns -1;
  */
-static int macro_buf_get( void )
+int macro_buf_get()
 {
+    ASSERT(macro_keys_left <= (int)Buffer.size()
+           && expanded_keys_left <= (int)Buffer.size());
+
+    _macro_inject_sent_keys();
+
     if (Buffer.size() == 0)
     {
         // If we're trying to fetch a new keystroke, then the processing
@@ -580,6 +636,8 @@ static int macro_buf_get( void )
 
     if (macro_keys_left >= 0)
         macro_keys_left--;
+    if (expanded_keys_left > 0)
+        expanded_keys_left--;
 
     for (int i = 0, size_i = recorders.size(); i < size_i; i++)
         recorders[i]->add_key(key);
@@ -587,7 +645,7 @@ static int macro_buf_get( void )
     return (key);
 }
 
-static void write_map(std::ofstream &f, const macromap &mp, const char *key)
+static void write_map(FILE *f, const macromap &mp, const char *key)
 {
     for (macromap::const_iterator i = mp.begin(); i != mp.end(); i++)
     {
@@ -595,8 +653,8 @@ static void write_map(std::ofstream &f, const macromap &mp, const char *key)
         // macro struct for all used keyboard commands.
         if (i->second.size())
         {
-            f << key  << vtostr((*i).first) << std::endl
-              << "A:" << vtostr((*i).second) << std::endl << std::endl;
+            fprintf(f, "%s%s\nA:%s\n\n", OUTS(key),
+                OUTS(vtostr((*i).first)), OUTS(vtostr((*i).second)));
         }
     }
 }
@@ -606,36 +664,41 @@ static void write_map(std::ofstream &f, const macromap &mp, const char *key)
  */
 void macro_save()
 {
-    std::ofstream f;
+    FILE *f;
     const std::string macrofile = get_macro_file();
-    f.open(macrofile.c_str());
+    f = fopen_u(macrofile.c_str(), "w");
     if (!f)
     {
         mprf(MSGCH_ERROR, "Couldn't open %s for writing!", macrofile.c_str());
         return;
     }
 
-    f << "# WARNING: This file is entirely auto-generated." << std::endl
-      << std::endl << "# Key Mappings:" << std::endl;
+    fprintf(f, "# %s %s macro file\n"
+               "# WARNING: This file is entirely auto-generated.\n"
+               "\n"
+               "# Key Mappings:\n",
+            OUTS(CRAWL), // ok, localizing the game name is not likely
+            OUTS(Version::Long())); // nor the version string
     for (int mc = KMC_DEFAULT; mc < KMC_CONTEXT_COUNT; ++mc)
     {
-        char keybuf[30] = "K:";
+        char buf[30] = "K:";
         if (mc)
-            snprintf(keybuf, sizeof keybuf, "K%d:", mc);
-        write_map(f, Keymaps[mc], keybuf);
+            snprintf(buf, sizeof buf, "K%d:", mc);
+        write_map(f, Keymaps[mc], buf);
     }
 
-    f << "# Command Macros:" << std::endl;
+    fprintf(f, "# Command Macros:\n");
     write_map(f, Macros, "M:");
 
-    f.close();
+    crawl_state.unsaved_macros = false;
+    fclose(f);
 }
 
 /*
  * Reads as many keypresses as are available (waiting for at least one),
  * and returns them as a single keyseq.
  */
-static keyseq _getch_mul( int (*rgetch)() = NULL )
+static keyseq _getch_mul(int (*rgetch)() = NULL)
 {
     keyseq keys;
     int    a;
@@ -652,12 +715,12 @@ static keyseq _getch_mul( int (*rgetch)() = NULL )
     if (!rgetch)
         rgetch = m_getch;
 
-    keys.push_back( a = rgetch() );
+    keys.push_back(a = rgetch());
 
     // The a == 0 test is legacy code that I don't dare to remove. I
     // have a vague recollection of it being a kludge for conio support.
     while (kbhit() || a == 0)
-        keys.push_back( a = rgetch() );
+        keys.push_back(a = rgetch());
 
     return (keys);
 }
@@ -666,9 +729,9 @@ static keyseq _getch_mul( int (*rgetch)() = NULL )
  * Replacement for getch(). Returns keys from the key buffer if available.
  * If not, adds some content to the buffer, and returns some of it.
  */
-int getchm( int (*rgetch)() )
+int getchm(int (*rgetch)())
 {
-    return getchm( KMC_DEFAULT, rgetch );
+    return getchm(KMC_DEFAULT, rgetch);
 }
 
 int getchm(KeymapContext mc, int (*rgetch)())
@@ -682,7 +745,7 @@ int getchm(KeymapContext mc, int (*rgetch)())
     // Read some keys...
     keyseq keys = _getch_mul(rgetch);
     if (mc == KMC_NONE)
-        macro_buf_add(keys);
+        macro_buf_add(keys, false, false);
     else
         macro_buf_add_long(keys, Keymaps[mc]);
 
@@ -693,19 +756,21 @@ int getchm(KeymapContext mc, int (*rgetch)())
  * Replacement for getch(). Returns keys from the key buffer if available.
  * If not, adds some content to the buffer, and returns some of it.
  */
-int getch_with_command_macros( void )
+int getch_with_command_macros()
 {
+    _macro_inject_sent_keys();
+
     if (Buffer.size() == 0)
     {
         // Read some keys...
         keyseq keys = _getch_mul();
         // ... and add them into the buffer (apply keymaps)
-        macro_buf_add_long( keys );
-
-        // Apply longest matching macro at front of buffer:
-        macro_buf_apply_command_macro();
+        macro_buf_add_long(keys);
     }
 
+    // Apply longest matching macro at front of buffer:
+    macro_buf_apply_command_macro();
+
     return (macro_buf_get());
 }
 
@@ -713,7 +778,7 @@ int getch_with_command_macros( void )
  * Flush the buffer.  Later we'll probably want to give the player options
  * as to when this happens (ex. always before command input, casting failed).
  */
-void flush_input_buffer( int reason )
+void flush_input_buffer(int reason)
 {
     ASSERT(reason != FLUSH_KEY_REPLAY_CANCEL
            || crawl_state.is_replaying_keys() || crawl_state.cmd_repeat_start);
@@ -734,7 +799,8 @@ void flush_input_buffer( int reason )
 
     if (Options.flush_input[ reason ] || reason == FLUSH_ABORT_MACRO
         || reason == FLUSH_KEY_REPLAY_CANCEL
-        || reason == FLUSH_REPLAY_SETUP_FAILURE)
+        || reason == FLUSH_REPLAY_SETUP_FAILURE
+        || reason == FLUSH_REPEAT_SETUP_DONE)
     {
         while (!Buffer.empty())
         {
@@ -744,49 +810,120 @@ void flush_input_buffer( int reason )
                 crawl_state.show_more_prompt = true;
         }
         macro_keys_left = -1;
+        expanded_keys_left = 0;
     }
 }
 
-void macro_add_query( void )
+static std::string _macro_prompt_string(const std::string &macro_type)
+{
+    return make_stringf("Input %s action: ", macro_type.c_str());
+}
+
+static void _macro_prompt(const std::string &macro_type)
+{
+    msgwin_prompt(_macro_prompt_string(macro_type));
+}
+
+static void _input_action_raw(const std::string &macro_type, keyseq* action)
+{
+    _macro_prompt(macro_type);
+    const int x = wherex();
+    const int y = wherey();
+    bool done = false;
+
+    while (!done)
+    {
+        cgotoxy(x, y);
+        cprintf("%s", vtostr(*action).c_str());
+
+        int input = m_getch();
+
+        switch (input)
+        {
+        CASE_ESCAPE
+            done = true;
+            *action = keyseq();
+            break;
+
+        case '\n':
+        case '\r':
+            done = true;
+            break;
+
+        default:
+            action->push_back(input);
+            break;
+        }
+    }
+
+    msgwin_reply(vtostr(*action));
+}
+
+static void _input_action_text(const std::string &macro_type, keyseq* action)
+{
+    char buff[1024];
+    msgwin_get_line_autohist(_macro_prompt_string(macro_type),
+                             buff, sizeof(buff));
+    *action = parse_keyseq(buff);
+}
+
+static std::string _macro_type_name(bool keymap, KeymapContext keymc)
+{
+    return make_stringf("%s%s",
+                        keymap ? (keymc == KMC_DEFAULT    ? "default " :
+                                  keymc == KMC_LEVELMAP   ? "level-map " :
+                                  keymc == KMC_TARGETING  ? "targeting " :
+                                  keymc == KMC_CONFIRM    ? "confirm " :
+                                  keymc == KMC_MENU       ? "menu "
+                                  : "buggy") : "",
+                        (keymap ? "keymap" : "macro"));
+}
+
+void macro_add_query(void)
 {
     int input;
     bool keymap = false;
+    bool raw = false;
     KeymapContext keymc = KMC_DEFAULT;
 
     mesclr();
-    mpr("(m)acro, keymap "
+    mpr("(m)acro, (M)acro raw, keymap "
         "[(k) default, (x) level-map, (t)argeting, (c)onfirm, m(e)nu], "
         "(s)ave? ",
         MSGCH_PROMPT);
     input = m_getch();
-    input = tolower( input );
-    if (input == 'k')
+    int low = tolower(input);
+
+    if (low == 'k')
     {
         keymap = true;
         keymc  = KMC_DEFAULT;
     }
-    else if (input == 'x')
+    else if (low == 'x')
     {
         keymap = true;
         keymc  = KMC_LEVELMAP;
     }
-    else if (input == 't')
+    else if (low == 't')
     {
         keymap = true;
         keymc  = KMC_TARGETING;
     }
-    else if (input == 'c')
+    else if (low == 'c')
     {
         keymap = true;
         keymc  = KMC_CONFIRM;
     }
-    else if (input == 'e')
+    else if (low == 'e')
     {
         keymap = true;
         keymc  = KMC_MENU;
     }
-    else if (input == 'm')
+    else if (low == 'm')
+    {
         keymap = false;
+        raw = (input == 'M');
+    }
     else if (input == 's')
     {
         mpr("Saving macros.");
@@ -795,23 +932,16 @@ void macro_add_query( void )
     }
     else
     {
-        mpr( "Aborting." );
+        mpr("Aborting.");
         return;
     }
 
     // reference to the appropriate mapping
     macromap &mapref = (keymap ? Keymaps[keymc] : Macros);
-
-    std::string prompt = make_stringf("Input %s%s trigger key: ",
-         keymap ? (keymc == KMC_DEFAULT    ? "default " :
-                   keymc == KMC_LEVELMAP   ? "level-map " :
-                   keymc == KMC_TARGETING  ? "targeting " :
-                   keymc == KMC_CONFIRM    ? "confirm " :
-                   keymc == KMC_MENU       ? "menu "
-                                           : "buggy") : "",
-         (keymap ? "keymap" : "macro"));
-
-    msgwin_prompt(prompt);
+    const std::string macro_type = _macro_type_name(keymap, keymc);
+    const std::string trigger_prompt = make_stringf("Input %s trigger key: ",
+                                                    macro_type.c_str());
+    msgwin_prompt(trigger_prompt);
 
     keyseq key;
     mouse_control mc(MOUSE_MODE_MACRO);
@@ -819,35 +949,58 @@ void macro_add_query( void )
 
     msgwin_reply(vtostr(key));
 
-    if (mapref[key].size() > 0)
+    if (mapref.find(key) != mapref.end() && !mapref[key].empty())
     {
-        mprf(MSGCH_WARN, "Current Action: %s", vtostr(mapref[key]).c_str());
+        std::string action = vtostr(mapref[key]);
+        action = replace_all(action, "<", "<<");
+        mprf(MSGCH_WARN, "Current Action: %s", action.c_str());
         mpr("Do you wish to (r)edefine, (c)lear, or (a)bort? ", MSGCH_PROMPT);
 
         input = m_getch();
 
-        input = tolower( input );
-        if (input == 'a' || input == ESCAPE)
+        input = tolower(input);
+        if (input == 'a' || key_is_escape(input))
         {
-            mpr( "Aborting." );
+            canned_msg(MSG_OK);
             return;
         }
         else if (input == 'c')
         {
-            mpr( "Cleared." );
-            macro_del( mapref, key );
+            mprf("Cleared %s '%s' => '%s'.",
+                 macro_type.c_str(),
+                 vtostr(key).c_str(),
+                 vtostr(mapref[key]).c_str());
+            macro_del(mapref, key);
+            crawl_state.unsaved_macros = true;
             return;
         }
     }
 
-    char    buff[4096];
-    msgwin_get_line_autohist("Input Macro Action: ", buff, sizeof(buff));
-
-    if (Options.macro_meta_entry)
-        macro_add( mapref, key, parse_keyseq(buff) );
+    keyseq action;
+    if (raw)
+        _input_action_raw(macro_type, &action);
     else
-        macro_add( mapref, key, buff );
+        _input_action_text(macro_type, &action);
 
+    if (action.empty())
+    {
+        const bool deleted_macro = macro_del(mapref, key);
+        if (deleted_macro)
+            mprf("Deleted %s for '%s'.",
+                 macro_type.c_str(),
+                 vtostr(key).c_str());
+        else
+            canned_msg(MSG_OK);
+    }
+    else
+    {
+        macro_add(mapref, key, action);
+        mprf("Created %s '%s' => '%s'.",
+             macro_type.c_str(),
+             vtostr(key).c_str(), vtostr(action).c_str());
+    }
+
+    crawl_state.unsaved_macros = true;
     redraw_screen();
 }
 
@@ -856,16 +1009,18 @@ void macro_add_query( void )
  */
 static void _read_macros_from(const char* filename)
 {
+    if (!file_exists(filename))
+        return;
+
     std::string s;
-    std::ifstream f;
+    FileLineInput f(filename);
     keyseq key, action;
     bool keymap = false;
     KeymapContext keymc = KMC_DEFAULT;
 
-    f.open( filename );
-
-    while (f >> s)
+    while (!f.eof())
     {
+        s = f.get_line();
         trim_string(s);  // remove white space from ends
 
         if (s[0] == '#')
@@ -878,7 +1033,7 @@ static void _read_macros_from(const char* filename)
         }
         else if (s.length() >= 3 && s[0] == 'K' && s[2] == ':')
         {
-            keymc  = KeymapContext( KMC_DEFAULT + s[1] - '0' );
+            keymc  = KeymapContext(KMC_DEFAULT + s[1] - '0');
             if (keymc >= KMC_DEFAULT && keymc < KMC_CONTEXT_COUNT)
             {
                 key    = parse_keyseq(s.substr(3));
@@ -893,7 +1048,7 @@ static void _read_macros_from(const char* filename)
         else if (s.substr(0, 2) == "A:")
         {
             action = parse_keyseq(s.substr(2));
-            macro_add( (keymap ? Keymaps[keymc] : Macros), key, action );
+            macro_add((keymap ? Keymaps[keymc] : Macros), key, action);
         }
     }
 }
@@ -927,7 +1082,6 @@ bool is_synthetic_key(int key)
     case KEY_MACRO_ENABLE_MORE:
     case KEY_MACRO_DISABLE_MORE:
     case KEY_MACRO_MORE_PROTECT:
-    case KEY_REPEAT_KEYS:
 #ifdef USE_TILE
     case CK_MOUSE_CMD:
 #endif
@@ -937,11 +1091,10 @@ bool is_synthetic_key(int key)
     }
 }
 
-key_recorder::key_recorder(key_recorder_callback cb, void* cb_data)
-    : paused(false), call_back(cb), call_back_data(cb_data)
+key_recorder::key_recorder()
+    : paused(false)
 {
     keys.clear();
-    macro_trigger_keys.clear();
 }
 
 void key_recorder::add_key(int key, bool reverse)
@@ -949,12 +1102,6 @@ void key_recorder::add_key(int key, bool reverse)
     if (paused)
         return;
 
-    if (call_back)
-    {
-        // Don't record key if true
-        if ((*call_back)(this, key, reverse))
-            return;
-    }
 
     if (reverse)
         keys.push_front(key);
@@ -962,36 +1109,9 @@ void key_recorder::add_key(int key, bool reverse)
         keys.push_back(key);
 }
 
-void key_recorder::remove_trigger_keys(int num_keys)
-{
-    ASSERT(num_keys >= 1);
-
-    if (paused)
-        return;
-
-    for (int i = 0; i < num_keys; i++)
-    {
-        ASSERT(keys.size() >= 1);
-
-        int key = keys[keys.size() - 1];
-
-        if (call_back)
-        {
-            // Key wasn't recorded in the first place, so no need to remove
-            // it
-            if ((*call_back)(this, key, true))
-                continue;
-        }
-
-        macro_trigger_keys.push_front(key);
-        keys.pop_back();
-    }
-}
-
 void key_recorder::clear()
 {
     keys.clear();
-    macro_trigger_keys.clear();
 }
 
 pause_all_key_recorders::pause_all_key_recorders()
@@ -1027,18 +1147,6 @@ void remove_key_recorder(key_recorder* recorder)
             recorders.erase(i);
             return;
         }
-
-    end(1, true, "remove_key_recorder(): recorder not found\n");
-}
-
-// Add macro trigger keys to beginning of the buffer, then expand
-// them.
-void insert_macro_into_buff(const keyseq& keys)
-{
-    for (int i = (int) keys.size() - 1; i >= 0; i--)
-        macro_buf_add(keys[i], true);
-
-    macro_buf_apply_command_macro();
 }
 
 int get_macro_buf_size()
@@ -1255,7 +1363,7 @@ static std::string _special_keys_to_string(int key)
     {
     case CK_ENTER:  cmd += "Enter"; break;
     case CK_BKSP:   cmd += "Backspace"; break;
-    case CK_ESCAPE: cmd += "Esc"; break;
+    CASE_ESCAPE     cmd += "Esc"; break;
     case CK_DELETE: cmd += "Del"; break;
     case CK_UP:     cmd += "Up"; break;
     case CK_DOWN:   cmd += "Down"; break;
@@ -1272,7 +1380,7 @@ static std::string _special_keys_to_string(int key)
     return (cmd);
 }
 
-std::string command_to_string(command_type cmd)
+std::string command_to_string(command_type cmd, bool tutorial)
 {
     const int key = command_to_key(cmd);
 
@@ -1281,7 +1389,12 @@ std::string command_to_string(command_type cmd)
         return (desc);
 
     if (key >= 32 && key < 256)
-        snprintf(info, INFO_SIZE, "%c", (char) key);
+    {
+        if (tutorial && key >= 'A' && key <= 'Z')
+            snprintf(info, INFO_SIZE, "uppercase %c", (char) key);
+        else
+            snprintf(info, INFO_SIZE, "%c", (char) key);
+    }
     else if (key > 1000 && key <= 1009)
     {
         const int numpad = (key - 1000);
