@@ -1,12 +1,12 @@
-/*
- * File:       makeitem.h
- * Summary:    Item creation routines.
- * Created by: haranp on Sat Apr 21 11:31:42 2007 UTC
- */
+/**
+ * @file
+ * @brief Item creation routines.
+**/
 
 #ifndef MAKEITEM_H
 #define MAKEITEM_H
 
+#include "decks.h"
 #include "itemprop-enum.h"
 
 enum item_make_species_type
@@ -16,12 +16,16 @@ enum item_make_species_type
     MAKE_ITEM_ORCISH      = 3,
 
     MAKE_ITEM_NO_RACE     = 100,
-    MAKE_ITEM_RANDOM_RACE = 250
+    MAKE_ITEM_RANDOM_RACE = 250,
 };
 
-int items( int allow_uniques, object_class_type force_class, int force_type,
+int create_item_named(std::string name, coord_def pos,
+                      std::string *error);
+
+int items(int allow_uniques, object_class_type force_class, int force_type,
            bool dont_place, int item_level, int item_race,
-           unsigned mapmask = 0, int force_ego = 0, int agent = -1 );
+           uint32_t mapmask = 0, int force_ego = 0, int agent = -1,
+           bool mundane = false);
 
 void item_colour(item_def &item);
 void init_rod_mp(item_def &item, int ncharges = -1, int item_level = -1);
@@ -32,14 +36,16 @@ armour_type get_random_body_armour_type(int level);
 armour_type get_random_armour_type(int item_level);
 void item_set_appearance(item_def &item);
 
-bool is_weapon_brand_ok(int type, int brand);
-bool is_armour_brand_ok(int type, int brand);
-bool is_missile_brand_ok(int type, int brand);
+bool is_weapon_brand_ok(int type, int brand, bool strict);
+bool is_armour_brand_ok(int type, int brand, bool strict);
+bool is_missile_brand_ok(int type, int brand, bool strict);
 
 bool got_curare_roll(const int item_level);
 void reroll_brand(item_def &item, int item_level);
 
-#if DEBUG_DIAGNOSTICS || DEBUG_TESTS
+deck_rarity_type random_deck_rarity();
+
+#if defined(DEBUG_DIAGNOSTICS) || defined(DEBUG_TESTS)
 void makeitem_tests();
 #endif
 #endif
