@@ -1,62 +1,147 @@
-/*
- *  File:       mon-behv.h
- *  Summary:    Monster behaviour functions.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Monster behaviour functions.
+**/
 
 #include "AppHdr.h"
 #include "mon-behv.h"
 
 #include "externs.h"
 
+#include "areas.h"
 #include "coord.h"
 #include "coordit.h"
+#include "database.h"
+#include "dungeon.h"
 #include "env.h"
 #include "fprop.h"
 #include "exclude.h"
+#include "items.h"
+#include "mon-act.h"
 #include "mon-death.h"
 #include "mon-iter.h"
 #include "mon-movetarget.h"
 #include "mon-pathfind.h"
+#include "mon-speak.h"
 #include "mon-stuff.h"
 #include "mon-util.h"
+#include "ouch.h"
 #include "random.h"
 #include "state.h"
 #include "terrain.h"
 #include "traps.h"
-#include "tutorial.h"
+#include "hints.h"
 #include "view.h"
 #include "shout.h"
 
-static void _set_nearest_monster_foe(monsters *monster);
+static void _set_nearest_monster_foe(monster* mons);
 
-static void _guess_invis_foe_pos(monsters *mon, bool strict = true)
+static void _guess_invis_foe_pos(monster* mon)
 {
     const actor* foe          = mon->get_foe();
     const int    guess_radius = mons_sense_invis(mon) ? 3 : 2;
 
     std::vector<coord_def> possibilities;
 
-    for (radius_iterator ri(mon->pos(), guess_radius, C_ROUND); ri; ++ri)
+    // NOTE: This depends on ignoring clouds, so that cells hidden by
+    // opaque clouds are included as a possibility for the foe's location.
+    los_def los(mon->pos(), opc_fullyopaque, circle_def(guess_radius, C_ROUND));
+    los.update();
+    for (radius_iterator ri(&los); ri; ++ri)
     {
-        // NOTE: This depends on mon_see_cell() ignoring clouds,
-        // so that cells hidden by opaque clouds are included
-        // as a possibility for the foe's location.
-        if (!strict || foe->is_habitable(*ri) && mon->mon_see_cell(*ri))
+        if (foe->is_habitable(*ri))
             possibilities.push_back(*ri);
     }
 
-    // If being strict (monster must see possible cell, foe must be
-    // able to live there) gives no possibilites, then find *some*
-    // cell near the foe.
-    if (strict && possibilities.empty())
-    {
-        _guess_invis_foe_pos(mon, false);
-        return;
-    }
-
     if (!possibilities.empty())
         mon->target = possibilities[random2(possibilities.size())];
+    else
+        mon->target = dgn_random_point_from(mon->pos(), guess_radius);
+}
+
+static void _mon_check_foe_invalid(monster* mon)
+{
+    if (mon->foe != MHITNOT && mon->foe != MHITYOU)
+    {
+        if (actor *foe = mon->get_foe())
+        {
+            const monster* foe_mons = foe->as_monster();
+            if (foe_mons->alive()
+                && (mon->friendly() != foe_mons->friendly()
+                    || mon->neutral() != foe_mons->neutral()))
+            {
+                return;
+            }
+        }
+
+        mon->foe = MHITNOT;
+    }
+}
+
+static bool _mon_tries_regain_los(monster* mon)
+{
+    // Only intelligent monsters with ranged attack will try to regain LOS.
+    if (mons_intel(mon) < I_NORMAL
+        || !mons_has_ranged_spell(mon, true) && !mons_has_ranged_attack(mon))
+    {
+        return false;
+    }
+
+    // Any special case should go here.
+    if (mons_class_flag(mon->type, M_FIGHTER)
+        && !(mon->type == MONS_CENTAUR_WARRIOR)
+        && !(mon->type == MONS_YAKTAUR_CAPTAIN))
+    {
+        return false;
+    }
+
+    // Randomize it a bit to make it less predictable.
+    return (mons_intel(mon) == I_NORMAL && !one_chance_in(10)
+            || mons_intel(mon) == I_HIGH && !one_chance_in(20));
+}
+
+// Monster tries to get into a firing position. Among the cells which have
+// a line of fire to the target, we choose the closest one to regain LOS as
+// fast as possible. If several cells are eligible, we choose the one closest
+// to ideal_range (too far = easier to escape, too close = easier to ambush).
+static void _set_firing_pos(monster* mon, coord_def target)
+{
+    const int ideal_range = LOS_RADIUS / 2;
+    const int current_distance = mon->pos().distance_from(target);
+    const los_type los = mons_has_los_ability(mon->type) ? LOS_DEFAULT
+                                                         : LOS_NO_TRANS;
+
+    // We don't consider getting farther away unless already very close.
+    const int max_range = std::max(ideal_range, current_distance);
+
+    int best_distance = INT_MAX;
+    int best_distance_to_ideal_range = INT_MAX;
+    coord_def best_pos(0, 0);
+    for (radius_iterator ri(mon->get_los(), true); ri; ++ri)
+    {
+        const coord_def p(*ri);
+        const int range = p.distance_from(target);
+
+        if (!in_bounds(p) || range > max_range
+            || !cell_see_cell(p, target, los)
+            || !mon_can_move_to_pos(mon, p - mon->pos()))
+        {
+            continue;
+        }
+
+        const int distance = p.distance_from(mon->pos());
+
+        if (distance < best_distance
+            || distance == best_distance
+               && std::abs(range - ideal_range) < best_distance_to_ideal_range)
+        {
+            best_pos = p;
+            best_distance = distance;
+            best_distance_to_ideal_range = std::abs(range - ideal_range);
+        }
+    }
+
+    mon->firing_pos = best_pos;
 }
 
 //---------------------------------------------------------------
@@ -72,7 +157,7 @@ static void _guess_invis_foe_pos(monsters *mon, bool strict = true)
 // the monster has a spell or special/nearby ability which isn't
 // affected by the wall.
 //---------------------------------------------------------------
-void handle_behaviour(monsters *mon)
+void handle_behaviour(monster* mon)
 {
     // Test spawners should always be BEH_SEEK against a foe, since
     // their only purpose is to spew out monsters for testing
@@ -96,9 +181,8 @@ void handle_behaviour(monsters *mon)
     bool isNeutral  = mon->neutral();
     bool wontAttack = mon->wont_attack();
 
-    // Whether the player is in LOS of the monster and can see
-    // or has guessed the player's location.
-    bool proxPlayer = mons_near(mon) && !crawl_state.arena;
+    // Whether the player position is in LOS of the monster.
+    bool proxPlayer = !crawl_state.game_is_arena() && mon->see_cell(you.pos());
 
 #ifdef WIZARD
     // If stealth is greater than actually possible (wizmode level)
@@ -117,6 +201,27 @@ void handle_behaviour(monsters *mon)
     static std::vector<level_exit> e;
     static int                     e_index = -1;
 
+    // Zotdef rotting
+    if (crawl_state.game_is_zotdef())
+    {
+        if (!isFriendly && !isNeutral && orb_position() == mon->pos()
+            && mon->speed)
+        {
+            const int loss = div_rand_round(10, mon->speed);
+            if (loss)
+            {
+                mpr("Your flesh rots away as the Orb of Zot is desecrated.",
+                    MSGCH_DANGER);
+                rot_hp(loss);
+                ouch(1, NON_MONSTER, KILLED_BY_ROTTING);
+            }
+        }
+    }
+
+    //mprf("AI debug: mon %d behv=%d foe=%d pos=%d %d target=%d %d",
+    //     mon->mindex(), mon->behaviour, mon->foe, mon->pos().x,
+    //     mon->pos().y, mon->target.x, mon->target.y);
+
     // Check for confusion -- early out.
     if (mon->has_ench(ENCH_CONFUSION))
     {
@@ -132,7 +237,7 @@ void handle_behaviour(monsters *mon)
     }
 
     // Make sure monsters are not targeting the player in arena mode.
-    ASSERT(!(crawl_state.arena && mon->foe == MHITYOU));
+    ASSERT(!(crawl_state.game_is_arena() && mon->foe == MHITYOU));
 
     if (mons_wall_shielded(mon) && cell_is_solid(mon->pos()))
     {
@@ -141,22 +246,15 @@ void handle_behaviour(monsters *mon)
             || isScared)
         {
             mon->behaviour = BEH_FLEE;
-
         }
     }
 
     const dungeon_feature_type can_move =
-        (mons_amphibious(mon)) ? DNGN_DEEP_WATER : DNGN_SHALLOW_WATER;
+        (mons_habitat(mon) == HT_AMPHIBIOUS) ? DNGN_DEEP_WATER
+                                             : DNGN_SHALLOW_WATER;
 
     // Validate current target exists.
-    if (mon->foe != MHITNOT && mon->foe != MHITYOU)
-    {
-        const monsters& foe_monster = menv[mon->foe];
-        if (!foe_monster.alive())
-            mon->foe = MHITNOT;
-        if (foe_monster.friendly() == isFriendly)
-            mon->foe = MHITNOT;
-    }
+    _mon_check_foe_invalid(mon);
 
     // Change proxPlayer depending on invisibility and standing
     // in shallow water.
@@ -179,21 +277,59 @@ void handle_behaviour(monsters *mon)
         {
             proxPlayer = true;
         }
+
+        // Ash penance makes monsters very likely to target you through
+        // invisibility, depending on their intelligence.
+        if (you.penance[GOD_ASHENZARI] && x_chance_in_y(intel, 6))
+            proxPlayer = true;
+    }
+
+    // Zotdef: immobile allies forget targets that are out of sight
+    if (crawl_state.game_is_zotdef())
+    {
+        if (isFriendly && mons_is_stationary(mon)
+            && (mon->foe != MHITNOT && mon->foe != MHITYOU)
+            && !mon->can_see(&menv[mon->foe]))
+        {
+            mon->foe = MHITYOU;
+            //mprf("%s resetting target (cantSee)",
+            //     mon->name(DESC_CAP_THE,true).c_str());
+        }
     }
 
     // Set friendly target, if they don't already have one.
     // Berserking allies ignore your commands!
     if (isFriendly
-        && you.pet_target != MHITNOT
         && (mon->foe == MHITNOT || mon->foe == MHITYOU)
         && !mon->berserk()
         && mon->type != MONS_GIANT_SPORE)
     {
-        mon->foe = you.pet_target;
+        if  (!crawl_state.game_is_zotdef())
+        {
+            if (you.pet_target != MHITNOT)
+                mon->foe = you.pet_target;
+        }
+        else    // Zotdef only
+        {
+            // Attack pet target if nearby
+            if (you.pet_target != MHITNOT && proxPlayer)
+            {
+                //mprf("%s setting target (player target)",
+                //     mon->name(DESC_CAP_THE,true).c_str());
+                mon->foe = you.pet_target;
+            }
+            else
+            {
+               // Zotdef - this is all new, for out-of-sight friendlies to do
+               // something useful.  If no current target, get the closest one.
+                _set_nearest_monster_foe(mon);
+            }
+        }
     }
 
     // Instead, berserkers attack nearest monsters.
-    if ((mon->berserk() || mon->type == MONS_GIANT_SPORE)
+    if (mon->behaviour != BEH_SLEEP
+        && (mon->berserk() || mon->type == MONS_GIANT_SPORE)
         && (mon->foe == MHITNOT || isFriendly && mon->foe == MHITYOU))
     {
         // Intelligent monsters prefer to attack the player,
@@ -213,6 +349,8 @@ void handle_behaviour(monsters *mon)
         && (proxPlayer || one_chance_in(3)))
     {
         _set_nearest_monster_foe(mon);
+        if (mon->foe == MHITNOT && crawl_state.game_is_zotdef())
+            mon->foe = MHITYOU;
     }
 
     // Monsters do not attack themselves. {dlb}
@@ -236,23 +374,18 @@ void handle_behaviour(monsters *mon)
 
     // Unfriendly monsters fighting other monsters will usually
     // target the player, if they're healthy.
+    // Zotdef: 2/3 chance of retargeting changed to 1/4
     if (!isFriendly && !isNeutral
         && mon->foe != MHITYOU && mon->foe != MHITNOT
         && proxPlayer && !mon->berserk() && isHealthy
-        && !one_chance_in(3))
+        && (crawl_state.game_is_zotdef() ? one_chance_in(4)
+                                         : !one_chance_in(3)))
     {
         mon->foe = MHITYOU;
     }
 
     // Validate current target again.
-    if (mon->foe != MHITNOT && mon->foe != MHITYOU)
-    {
-        const monsters& foe_monster = menv[mon->foe];
-        if (!foe_monster.alive())
-            mon->foe = MHITNOT;
-        if (foe_monster.friendly() == isFriendly)
-            mon->foe = MHITNOT;
-    }
+    _mon_check_foe_invalid(mon);
 
     while (changed)
     {
@@ -261,7 +394,7 @@ void handle_behaviour(monsters *mon)
 
         if (mon->foe == MHITYOU)
         {
-            // monsters::get_foe returns NULL for friendly monsters with
+            // monster::get_foe returns NULL for friendly monsters with
             // foe == MHITYOU, so make afoe point to the player here.
             // -cao
             afoe = &you;
@@ -272,6 +405,14 @@ void handle_behaviour(monsters *mon)
         if (afoe)
             foepos = afoe->pos();
 
+        if (crawl_state.game_is_zotdef() && mon->foe == MHITYOU)
+        {
+            foepos = PLAYER_POS;
+            proxFoe = true;
+        }
+
+        if (mon->pos() == mon->firing_pos)
+            mon->firing_pos.reset();
 
         // Track changes to state; attitude never changes here.
         beh_type new_beh       = mon->behaviour;
@@ -291,7 +432,9 @@ void handle_behaviour(monsters *mon)
             // No foe?  Then wander or seek the player.
             if (mon->foe == MHITNOT)
             {
-                if (crawl_state.arena || !proxPlayer || isNeutral || patrolling
+                if (crawl_state.game_is_arena()
+                    || !proxPlayer && !isFriendly
+                    || isNeutral || patrolling
                     || mon->type == MONS_GIANT_SPORE)
                 {
                     new_beh = BEH_WANDER;
@@ -299,13 +442,17 @@ void handle_behaviour(monsters *mon)
                 else
                 {
                     new_foe = MHITYOU;
-                    mon->target = you.pos();
+                    mon->target = PLAYER_POS;
                 }
                 break;
             }
 
             // Foe gone out of LOS?
-            if (!proxFoe)
+            if (!proxFoe
+                && !(mon->friendly()
+                     && mon->foe == MHITYOU
+                     && mon->is_travelling()
+                     && mon->travel_target == MTRAV_PLAYER))
             {
                 // Maybe the foe is just invisible.
                 if (mon->target.origin() && afoe && mon->near_foe())
@@ -329,20 +476,9 @@ void handle_behaviour(monsters *mon)
                 if (mon->travel_target == MTRAV_SIREN)
                     mon->travel_target = MTRAV_NONE;
 
-                if (mon->foe == MHITYOU && mon->is_travelling()
-                    && mon->travel_target == MTRAV_PLAYER)
+                if (isFriendly && mon->foe != MHITYOU)
                 {
-                    // We've got a target, so we'll continue on our way.
-#ifdef DEBUG_PATHFIND
-                    mpr("Player out of LoS... start wandering.");
-#endif
-                    new_beh = BEH_WANDER;
-                    break;
-                }
-
-                if (isFriendly)
-                {
-                    if (patrolling || crawl_state.arena)
+                    if (patrolling || crawl_state.game_is_arena())
                     {
                         new_foe = MHITNOT;
                         new_beh = BEH_WANDER;
@@ -365,14 +501,23 @@ void handle_behaviour(monsters *mon)
                     // but only for a few moves (smell and
                     // intuition only go so far).
 
-                    if (mon->pos() == mon->target)
-                    {
+                    if (mon->pos() == mon->target
+                        && (!isFriendly || !crawl_state.game_is_zotdef()))
+                    {   // hostiles only in Zotdef
                         if (mon->foe == MHITYOU)
                         {
-                            if (one_chance_in(you.skills[SK_STEALTH]/3))
-                                mon->target = you.pos();
+                            if (crawl_state.game_is_zotdef())
+                                mon->target = PLAYER_POS;  // infallible tracking in zotdef
                             else
-                                mon->foe_memory = 0;
+                            {
+                                if (one_chance_in(you.skill(SK_STEALTH) / 3)
+                                    || you.penance[GOD_ASHENZARI] && coinflip())
+                                {
+                                    mon->target = you.pos();
+                                }
+                                else
+                                    mon->foe_memory = 0;
+                            }
                         }
                         else
                         {
@@ -382,43 +527,64 @@ void handle_behaviour(monsters *mon)
                                 mon->foe_memory = 0;
                         }
                     }
-
-                    // Either keep chasing, or start wandering.
-                    if (mon->foe_memory < 2)
-                    {
-                        mon->foe_memory = 0;
-                        new_beh = BEH_WANDER;
-                    }
-                    break;
                 }
 
-                ASSERT(mon->foe_memory == 0);
-                // Hack: smarter monsters will tend to pursue the player longer.
-                switch (mons_intel(mon))
+
+                if (mon->foe_memory <= 0
+                    && !(mon->friendly() && mon->foe == MHITYOU))
                 {
-                case I_HIGH:
-                    mon->foe_memory = 100 + random2(200);
-                    break;
-                case I_NORMAL:
-                    mon->foe_memory = 50 + random2(100);
-                    break;
-                case I_ANIMAL:
-                case I_INSECT:
-                    mon->foe_memory = 25 + random2(75);
-                    break;
-                case I_PLANT:
-                    mon->foe_memory = 10 + random2(50);
-                    break;
+                    new_beh = BEH_WANDER;
                 }
-                break;  // switch/case BEH_SEEK
+                // If the player walk out of the LOS of a monster with a ranged
+                // attack, we assume it sees in which direction the player went
+                // and it tries to find a line of fire instead of following the
+                // player.
+                else if (grid_distance(mon->target, you.pos()) == 1
+                         && _mon_tries_regain_los(mon))
+                {
+                    _set_firing_pos(mon, you.pos());
+                }
+
+                if (!isFriendly)
+                    break;
+            }
+
+            ASSERT((proxFoe || isFriendly) && mon->foe != MHITNOT);
+
+            // Monster can see foe: set memory in case it loses sight.
+            // Hack: smarter monsters will tend to pursue the player longer.
+            switch (mons_intel(mon))
+            {
+            case I_HIGH:
+                mon->foe_memory = 100 + random2(200);
+                break;
+            case I_NORMAL:
+                mon->foe_memory = 50 + random2(100);
+                break;
+            case I_ANIMAL:
+            case I_INSECT:
+                mon->foe_memory = 25 + random2(75);
+                break;
+            case I_PLANT:
+                mon->foe_memory = 10 + random2(50);
+                break;
             }
 
-            ASSERT(proxFoe && mon->foe != MHITNOT);
             // Monster can see foe: continue 'tracking'
             // by updating target x,y.
             if (mon->foe == MHITYOU)
             {
                 // The foe is the player.
+
+                // If monster is currently getting into firing position and
+                // see the player and can attack him, clear firing_pos.
+                if (!mon->firing_pos.zero()
+                    && (mons_has_los_ability(mon->type)
+                        || mon->see_cell_no_trans(mon->target)))
+                {
+                    mon->firing_pos.reset();
+                }
+
                 if (mon->type == MONS_SIREN
                     && you.beheld_by(mon)
                     && find_siren_water_target(mon))
@@ -426,7 +592,7 @@ void handle_behaviour(monsters *mon)
                     break;
                 }
 
-                if (try_pathfind(mon, can_move))
+                if (mon->firing_pos.zero() && try_pathfind(mon, can_move))
                     break;
 
                 // Whew. If we arrived here, path finding didn't yield anything
@@ -434,7 +600,8 @@ void handle_behaviour(monsters *mon)
                 // the traditional way.
 
                 // Sometimes, your friends will wander a bit.
-                if (isFriendly && one_chance_in(8))
+                if (isFriendly && one_chance_in(8)
+                    && mon->foe == MHITYOU && proxFoe)
                 {
                     set_random_target(mon);
                     mon->foe = MHITNOT;
@@ -442,7 +609,7 @@ void handle_behaviour(monsters *mon)
                 }
                 else
                 {
-                    mon->target = you.pos();
+                    mon->target = PLAYER_POS;
                 }
             }
             else
@@ -460,25 +627,6 @@ void handle_behaviour(monsters *mon)
             {
                 new_beh = BEH_FLEE;
 
-                // This is here instead of in the BEH_FLEE section of the switch
-                // for handle_behaviour, as it only needs to happen once per
-                // fleeing.
-                if (mon->type == MONS_KRAKEN)
-                {
-                    int tcount = 0;
-                    int headnum = mon->mindex();
-                    for (monster_iterator mi; mi; ++mi)
-                        if (mi->type == MONS_KRAKEN_TENTACLE
-                            && (int)mi->number == headnum)
-                        {
-                            monster_die(*mi, KILL_MISC, NON_MONSTER, true);
-                            tcount++;
-                        }
-
-                    if (tcount > 0)
-                        mpr("The kraken's tentacles slip beneath the water.",
-                            MSGCH_WARN);
-                }
             }
             break;
 
@@ -603,7 +751,7 @@ void handle_behaviour(monsters *mon)
             if (!proxFoe)
             {
                 if ((isFriendly || proxPlayer) && !isNeutral && !patrolling
-                    && !crawl_state.arena)
+                    && !crawl_state.game_is_arena())
                 {
                     new_foe = MHITYOU;
                 }
@@ -640,7 +788,7 @@ void handle_behaviour(monsters *mon)
     }
 }
 
-static bool _mons_check_foe(monsters *mon, const coord_def& p,
+static bool _mons_check_foe(monster* mon, const coord_def& p,
                             bool friendly, bool neutral)
 {
     if (!in_bounds(p))
@@ -654,7 +802,7 @@ static bool _mons_check_foe(monsters *mon, const coord_def& p,
         return (false);
     }
 
-    if (monsters *foe = monster_at(p))
+    if (monster* foe = monster_at(p))
     {
         if (foe != mon
             && mon->can_see(foe)
@@ -662,7 +810,8 @@ static bool _mons_check_foe(monsters *mon, const coord_def& p,
             && (friendly || !is_sanctuary(p))
             && (foe->friendly() != friendly
                 || neutral && !foe->neutral())
-            && !mons_is_firewood(foe))
+            && (crawl_state.game_is_zotdef() || !mons_is_firewood(foe)))
+                // Zotdef allies take out firewood
         {
             return (true);
         }
@@ -671,8 +820,12 @@ static bool _mons_check_foe(monsters *mon, const coord_def& p,
 }
 
 // Choose random nearest monster as a foe.
-void _set_nearest_monster_foe(monsters *mon)
+static void _set_nearest_monster_foe(monster* mon)
 {
+    // These don't look for foes.
+    if (mon->good_neutral() || mon->strict_neutral())
+        return;
+
     const bool friendly = mon->friendly();
     const bool neutral  = mon->neutral();
 
@@ -706,14 +859,14 @@ void _set_nearest_monster_foe(monsters *mon)
 // 2. Call handle_behaviour to re-evaluate AI state and target x, y
 //
 //-----------------------------------------------------------------
-void behaviour_event(monsters *mon, mon_event_type event, int src,
+void behaviour_event(monster* mon, mon_event_type event, int src,
                      coord_def src_pos, bool allow_shout)
 {
     if (!mon->alive())
         return;
 
     ASSERT(src >= 0 && src <= MHITYOU);
-    ASSERT(!crawl_state.arena || src != MHITYOU);
+    ASSERT(!crawl_state.game_is_arena() || src != MHITYOU);
     ASSERT(in_bounds(src_pos) || src_pos.origin());
     if (mons_is_projectile(mon->type))
         return; // projectiles have no AI
@@ -726,6 +879,7 @@ void behaviour_event(monsters *mon, mon_event_type event, int src,
     bool setTarget        = false;
     bool breakCharm       = false;
     bool was_sleeping     = mon->asleep();
+    std::string msg;
 
     if (src == MHITYOU)
         sourceWontAttack = true;
@@ -775,7 +929,8 @@ void behaviour_event(monsters *mon, mon_event_type event, int src,
         // the head, of course, always triggers this code.
         if (event == ME_WHACK
             || ((wontAttack != sourceWontAttack || isSmart)
-                && !mons_is_fleeing(mon) && !mons_is_panicking(mon)))
+                && (!mons_is_fleeing(mon) && !mons_class_flag(mon->type, M_FLEEING))
+                && !mons_is_panicking(mon)))
         {
             // Monster types that you can't gain experience from cannot
             // fight back, so don't bother having them do so.  If you
@@ -831,13 +986,25 @@ void behaviour_event(monsters *mon, mon_event_type event, int src,
         if (mon->friendly() && mon->type == MONS_GIANT_SPORE)
             break;
 
+        // [ds] Neutral monsters don't react to your presence.
+        // XXX: Neutral monsters are a tangled mess of arbitrary logic.
+        // It's not even clear any more what behaviours are intended for
+        // neutral monsters and what are merely accidents of the code.
+        if (mon->neutral())
+        {
+            if (mon->asleep())
+                mon->behaviour = BEH_WANDER;
+            break;
+        }
+
         if (mon->asleep() && mons_near(mon))
             remove_auto_exclude(mon, true);
 
         // Will alert monster to <src> and turn them
         // against them, unless they have a current foe.
         // It won't turn friends hostile either.
-        if (!mons_is_fleeing(mon) && !mons_is_panicking(mon)
+        if ((!mons_is_fleeing(mon) || mons_class_flag(mon->type, M_FLEEING))
+            && !mons_is_panicking(mon)
             && !mons_is_cornered(mon))
         {
             mon->behaviour = BEH_SEEK;
@@ -860,8 +1027,8 @@ void behaviour_event(monsters *mon, mon_event_type event, int src,
                 && !you.see_cell(mon->pos()))
             {
                 const dungeon_feature_type can_move =
-                    (mons_amphibious(mon)) ? DNGN_DEEP_WATER
-                                           : DNGN_SHALLOW_WATER;
+                    (mons_habitat(mon) == HT_AMPHIBIOUS) ? DNGN_DEEP_WATER
+                                                         : DNGN_SHALLOW_WATER;
 
                 try_pathfind(mon, can_move);
             }
@@ -885,6 +1052,8 @@ void behaviour_event(monsters *mon, mon_event_type event, int src,
             break;
         }
 
+        msg = getSpeakString(mon->name(DESC_PLAIN) + " flee");
+
         // Assume monsters know where to run from, even if player is
         // invisible.
         mon->behaviour = BEH_FLEE;
@@ -905,27 +1074,11 @@ void behaviour_event(monsters *mon, mon_event_type event, int src,
             else
                 setTarget = true;
         }
-        else if (mon->friendly() && !crawl_state.arena)
+        else if (mon->friendly() && !crawl_state.game_is_arena())
             mon->foe = MHITYOU;
 
         if (you.see_cell(mon->pos()))
-            learned_something_new(TUT_FLEEING_MONSTER);
-
-        if (mon->type == MONS_KRAKEN)
-        {
-            int tcount = 0;
-            int headnum = mon->mindex();
-            for (monster_iterator mi; mi; ++mi)
-                if (mi->type == MONS_KRAKEN_TENTACLE
-                    && (int)mi->number == headnum)
-                {
-                    monster_die(*mi, KILL_MISC, NON_MONSTER, true);
-                    tcount++;
-                }
-
-            if (tcount > 0)
-                mpr("The kraken's tentacles slip beneath the water.", MSGCH_WARN);
-        }
+            learned_something_new(HINT_FLEEING_MONSTER);
 
         break;
 
@@ -939,15 +1092,19 @@ void behaviour_event(monsters *mon, mon_event_type event, int src,
             break;
 
         // Just set behaviour... foe doesn't change.
-        if (!mons_is_cornered(mon))
+        if (!mons_is_cornered(mon) && !mon->has_ench(ENCH_WITHDRAWN))
         {
-            if (mon->friendly() && !crawl_state.arena)
+            if (mon->friendly() && !crawl_state.game_is_arena())
             {
                 mon->foe = MHITYOU;
-                simple_monster_message(mon, " returns to your side!");
+                msg = "PLAIN:@The_monster@ returns to your side!";
             }
             else if (mon->type != MONS_KRAKEN_TENTACLE)
-                simple_monster_message(mon, " turns to fight!");
+            {
+                msg = getSpeakString(mon->name(DESC_PLAIN) + " cornered");
+                if (msg.empty())
+                    msg = "PLAIN:@The_monster@ turns to fight!";
+            }
         }
 
         mon->behaviour = BEH_CORNERED;
@@ -963,6 +1120,7 @@ void behaviour_event(monsters *mon, mon_event_type event, int src,
         {
             mon->target = you.pos();
             mon->attitude = ATT_HOSTILE;
+            mons_att_changed(mon);
         }
         else if (src != MHITNOT)
             mon->target = src_pos;
@@ -970,7 +1128,10 @@ void behaviour_event(monsters *mon, mon_event_type event, int src,
 
     // Now, break charms if appropriate.
     if (breakCharm)
+    {
         mon->del_ench(ENCH_CHARM);
+        mons_att_changed(mon);
+    }
 
     // Do any resultant foe or state changes.
     handle_behaviour(mon);
@@ -1002,12 +1163,25 @@ void behaviour_event(monsters *mon, mon_event_type event, int src,
         mon->behaviour = BEH_LURK;
     }
 
-    ASSERT(!crawl_state.arena
+    if (!msg.empty() && mon->visible_to(&you))
+        mons_speaks_msg(mon, msg, MSGCH_TALK, silenced(mon->pos()));
+
+    ASSERT(!crawl_state.game_is_arena()
            || mon->foe != MHITYOU && mon->target != you.pos());
 }
 
-void make_mons_stop_fleeing(monsters *mon)
+void make_mons_stop_fleeing(monster* mon)
 {
     if (mons_is_fleeing(mon))
         behaviour_event(mon, ME_CORNERED);
 }
+
+// Returns the position of the Orb, or you.pos() if
+// the Orb's not found.
+coord_def zotdef_target()
+{
+    coord_def tgt = orb_position();
+    if (tgt.origin())
+        tgt = you.pos();
+    return tgt;
+}
