@@ -15,11 +15,17 @@ public:
     bool process_list(const char *list_file);
     bool write_data();
 protected:
-    bool load_image(tile &img, const char *filename);
+    bool load_image(tile &img, const char *filename, bool background = false);
     bool process_line(char *read_line, const char *list_file, int line);
     void add_image(tile &img, const char *enumname);
     void recolour(tile &img);
 
+    void add_abstracts(
+        FILE *fp,
+        const char *format,
+        const std::vector<std::string> &lc_enum,
+        const std::vector<std::string> &uc_max_enum);
+
     std::string m_name;
 
     tile_page m_page;
@@ -33,9 +39,10 @@ protected:
     std::vector<tile*> m_back;
     std::string m_parts_ctg;
     std::string m_sdir;
+    std::string m_back_sdir;
     std::string m_prefix;
     std::string m_start_value;
-    std::string m_include;
+    std::vector<std::string> m_include;
     std::vector<std::string> m_categories;
     std::vector<int> m_ctg_counts;
     tile m_compose;
@@ -58,6 +65,10 @@ protected:
     lum_list m_lum;
 
     std::vector<std::string> m_depends;
+
+    typedef std::pair<std::string, std::string> string_pair;
+    typedef std::vector<string_pair> string_pair_list;
+    string_pair_list m_abstract;
 };
 
 #endif
