@@ -443,7 +443,7 @@ int spell_fail(spell_type spell)
 int calc_spell_power(spell_type spell, bool apply_intel, bool fail_rate_check,
                      bool cap_power, bool rod)
 {
-    int power;
+    int power = 0;
     if (rod) {
         // This is only the average of the power. It's used for display and
         // calculating range. The real power is randomized in staff_spell()
@@ -451,10 +451,6 @@ int calc_spell_power(spell_type spell, bool apply_intel, bool fail_rate_check,
     }
     else
     {
-        // When checking failure rates, wizardry is handled after the various
-        // stepping calulations.
-        power = (you.skill(SK_SPELLCASTING) / 2)
-                     + (fail_rate_check? 0 : player_mag_abil(false));
         int enhanced = 0;
 
         unsigned int disciplines = get_spell_disciplines(spell);
@@ -469,10 +465,16 @@ int calc_spell_power(spell_type spell, bool apply_intel, bool fail_rate_check,
             {
                 unsigned int bit = (1 << ndx);
                 if (disciplines & bit)
-                    power += (you.skill(spell_type2skill(bit)) * 2) / skillcount;
+                    power += you.skill(spell_type2skill(bit)) * 2;
             }
+            power /= skillcount;
         }
 
+        // When checking failure rates, wizardry is handled after the various
+        // stepping calulations.
+        power += (you.skill(SK_SPELLCASTING) / 2)
+                  + (fail_rate_check? 0 : player_mag_abil(false));
+
         if (apply_intel)
             power = (power * you.intel()) / 10;
 
