@@ -1,8 +1,7 @@
-/*
- *  File:       transform.cc
- *  Summary:    Misc function related to player transformations.
- *  Written by: Linley Henzell
- */
+/**
+ * @file
+ * @brief Misc function related to player transformations.
+**/
 
 #include "AppHdr.h"
 
@@ -16,15 +15,17 @@
 #include "artefact.h"
 #include "delay.h"
 #include "env.h"
+#include "godabil.h"
 #include "invent.h"
-#include "it_use2.h"
 #include "item_use.h"
 #include "itemprop.h"
 #include "items.h"
 #include "output.h"
 #include "player.h"
 #include "player-equip.h"
+#include "player-stats.h"
 #include "random.h"
+#include "religion.h"
 #include "skills2.h"
 #include "state.h"
 #include "stuff.h"
@@ -34,23 +35,57 @@
 
 static void _extra_hp(int amount_extra);
 
-bool transformation_can_wield(transformation_type trans)
+bool form_can_wield(transformation_type form)
 {
-    return (trans == TRAN_NONE
-            || trans == TRAN_STATUE
-            || trans == TRAN_LICH);
+    return (form == TRAN_NONE || form == TRAN_STATUE || form == TRAN_LICH);
 }
 
-bool transform_allows_wearing_item(const item_def& item)
+bool form_can_fly(transformation_type form)
 {
-    return (
-        transform_allows_wearing_item(item,
-                                      static_cast<transformation_type>(
-                                          you.attribute[ATTR_TRANSFORMATION])));
+    if (form == TRAN_LICH
+        && you.species == SP_KENKU
+        && (you.experience_level >= 15 || you.airborne()))
+    {
+        return (true);
+    }
+    return (form == TRAN_DRAGON || form == TRAN_BAT);
 }
 
-bool transform_allows_wearing_item(const item_def& item,
-                                   transformation_type transform)
+bool form_can_swim(transformation_type form)
+{
+    // Ice floats.
+    if (form == TRAN_ICE_BEAST)
+        return (true);
+
+    if (you.species == SP_MERFOLK && !form_changed_physiology(form))
+        return (true);
+
+    size_type size = you.transform_size(form, PSIZE_BODY);
+    if (size == SIZE_CHARACTER)
+        size = you.body_size(PSIZE_BODY, true);
+
+    return (size >= SIZE_GIANT);
+}
+
+bool form_likes_water(transformation_type form)
+{
+    return (form_can_swim(form) || you.species == SP_GREY_DRACONIAN
+                                   && !form_changed_physiology(form));
+}
+
+bool form_can_butcher_barehanded(transformation_type form)
+{
+    return (form == TRAN_BLADE_HANDS || form == TRAN_DRAGON
+            || form == TRAN_ICE_BEAST);
+}
+
+// Used to mark transformations which override species/mutation intrinsics.
+bool form_changed_physiology(transformation_type form)
+{
+    return (form != TRAN_NONE && form != TRAN_BLADE_HANDS);
+}
+
+bool form_can_wear_item(const item_def& item, transformation_type form)
 {
     bool rc = true;
 
@@ -58,7 +93,7 @@ bool transform_allows_wearing_item(const item_def& item,
     {
         // Everything but bats can wear all jewellery; bats and pigs can
         // only wear amulets.
-        if ((transform == TRAN_BAT || transform == TRAN_PIG)
+        if ((form == TRAN_BAT || form == TRAN_PIG)
              && !jewellery_is_amulet(item))
         {
             rc = false;
@@ -68,9 +103,8 @@ bool transform_allows_wearing_item(const item_def& item,
     {
         // It's not jewellery, and it's worn, so it must be armour.
         const equipment_type eqslot = get_armour_slot(item);
-        const bool is_soft_helmet   = is_helmet(item) && !is_hard_helmet(item);
 
-        switch (transform)
+        switch (form)
         {
         // Some forms can wear everything.
         case TRAN_NONE:
@@ -82,30 +116,28 @@ bool transform_allows_wearing_item(const item_def& item,
         case TRAN_DRAGON:
         case TRAN_BAT:
         case TRAN_PIG:
+        case TRAN_SPIDER:
             rc = false;
             break;
 
         // And some need more complicated logic.
-        case TRAN_SPIDER:
-            rc = is_soft_helmet;
-            break;
-
         case TRAN_BLADE_HANDS:
             rc = (eqslot != EQ_SHIELD && eqslot != EQ_GLOVES);
             break;
 
         case TRAN_STATUE:
-            rc = (eqslot == EQ_CLOAK || eqslot == EQ_HELMET);
+            rc = (eqslot == EQ_CLOAK || eqslot == EQ_HELMET
+                  || eqslot == EQ_SHIELD);
             break;
 
         case TRAN_ICE_BEAST:
-            rc = (eqslot == EQ_CLOAK || is_soft_helmet);
+            rc = (eqslot == EQ_CLOAK);
             break;
 
         default:                // Bug-catcher.
             mprf(MSGCH_ERROR, "Unknown transformation type %d in "
-                 "transform_allows_wearing_item",
-                 you.attribute[ATTR_TRANSFORMATION]);
+                 "form_can_wear_item",
+                 you.form);
             break;
         }
     }
@@ -114,15 +146,14 @@ bool transform_allows_wearing_item(const item_def& item,
 }
 
 static std::set<equipment_type>
-_init_equipment_removal(transformation_type trans)
+_init_equipment_removal(transformation_type form)
 {
     std::set<equipment_type> result;
-    if (!transformation_can_wield(trans) && you.weapon())
+    if (!form_can_wield(form) && you.weapon() || you.melded[EQ_WEAPON])
         result.insert(EQ_WEAPON);
 
     // Liches can't wield holy weapons.
-    if (trans == TRAN_LICH
-        && you.weapon()
+    if (form == TRAN_LICH && you.weapon()
         && get_weapon_brand(*you.weapon()) == SPWPN_HOLY_WRATH)
     {
         result.insert(EQ_WEAPON);
@@ -132,7 +163,7 @@ _init_equipment_removal(transformation_type trans)
     {
         const equipment_type eq = static_cast<equipment_type>(i);
         const item_def *pitem = you.slot_item(eq, true);
-        if (pitem && !transform_allows_wearing_item(*pitem, trans))
+        if (pitem && !form_can_wear_item(*pitem, form))
             result.insert(eq);
     }
     return (result);
@@ -150,7 +181,14 @@ static void _remove_equipment(const std::set<equipment_type>& removed,
         if (equip == NULL)
             continue;
 
-        bool unequip = (e == EQ_WEAPON || !meld);
+        bool unequip = !meld;
+        if (!unequip && e == EQ_WEAPON)
+        {
+            if (you.form == TRAN_NONE || form_can_wield(you.form))
+                unequip = true;
+            if (equip->base_type != OBJ_WEAPONS && equip->base_type != OBJ_STAVES)
+                unequip = true;
+        }
 
         mprf("%s %s%s %s", equip->name(DESC_CAP_YOUR).c_str(),
              unequip ? "fall" : "meld",
@@ -182,12 +220,13 @@ static void _remove_equipment(const std::set<equipment_type>& removed,
 }
 
 // FIXME: merge this with you_can_wear(), can_wear_armour(), etc.
-bool _mutations_prevent_wearing(const item_def& item)
+static bool _mutations_prevent_wearing(const item_def& item)
 {
     const equipment_type eqslot = get_armour_slot(item);
 
     if (is_hard_helmet(item)
         && (player_mutation_level(MUT_HORNS)
+            || player_mutation_level(MUT_ANTENNAE)
             || player_mutation_level(MUT_BEAK)))
     {
         return (true);
@@ -195,8 +234,8 @@ bool _mutations_prevent_wearing(const item_def& item)
 
     // Barding is excepted here.
     if (item.sub_type == ARM_BOOTS
-        && (player_mutation_level(MUT_HOOVES)
-            || player_mutation_level(MUT_TALONS)))
+        && (player_mutation_level(MUT_HOOVES) >= 3
+            || player_mutation_level(MUT_TALONS) >= 3))
     {
         return (true);
     }
@@ -204,6 +243,10 @@ bool _mutations_prevent_wearing(const item_def& item)
     if (eqslot == EQ_GLOVES && player_mutation_level(MUT_CLAWS) >= 3)
         return (true);
 
+    if (eqslot == EQ_HELMET && (player_mutation_level(MUT_HORNS) == 3
+        || player_mutation_level(MUT_ANTENNAE) == 3))
+        return (true);
+
     return (false);
 }
 
@@ -213,6 +256,17 @@ static void _unmeld_equipment_slot(equipment_type e)
 
     if (item.base_type == OBJ_JEWELLERY)
         unmeld_slot(e);
+    else if (e == EQ_WEAPON)
+    {
+        if (you.slot_item(EQ_SHIELD)
+            && is_shield_incompatible(item, you.slot_item(EQ_SHIELD)))
+        {
+            mpr(item.name(DESC_CAP_YOUR) + " is pushed off your body!");
+            unequip_item(e);
+        }
+        else
+            unmeld_slot(e);
+    }
     else
     {
         // In case the player was mutated during the transformation,
@@ -246,7 +300,7 @@ static void _unmeld_equipment(const std::set<equipment_type>& melded)
     for (iter = melded.begin(); iter != melded.end(); ++iter)
     {
         const equipment_type e = *iter;
-        if (e == EQ_WEAPON || you.equip[e] == -1)
+        if (you.equip[e] == -1)
             continue;
 
         _unmeld_equipment_slot(e);
@@ -267,205 +321,15 @@ void remove_one_equip(equipment_type eq, bool meld, bool mutation)
     _remove_equipment(r, meld, mutation);
 }
 
-static bool _tran_may_meld_cursed(int transformation)
-{
-    switch (transformation)
-    {
-    case TRAN_BAT:
-        // Vampires of sufficient level may transform into bats even
-        // with cursed gear.
-        if (you.species == SP_VAMPIRE && you.experience_level >= 10)
-            return (true);
-        // intentional fall-through
-    case TRAN_SPIDER:
-        return (false);
-    default:
-        return (true);
-    }
-}
-
-// Returns true if any piece of equipment that has to be removed is cursed.
-// Useful for keeping low level transformations from being too useful.
-static bool _check_for_cursed_equipment(const std::set<equipment_type> &remove,
-                                        const int trans, bool quiet = false)
-{
-    std::set<equipment_type>::const_iterator iter;
-    for (iter = remove.begin(); iter != remove.end(); ++iter)
-    {
-        const equipment_type e = *iter;
-        if (you.equip[e] == -1)
-            continue;
-
-        const item_def& item = you.inv[ you.equip[e] ];
-        if (item.cursed())
-        {
-            if (e != EQ_WEAPON && _tran_may_meld_cursed(trans))
-                continue;
-
-            // Wielding a cursed non-weapon/non-staff won't hinder
-            // transformations.
-            if (e == EQ_WEAPON && item.base_type != OBJ_WEAPONS
-                && item.base_type != OBJ_STAVES)
-            {
-                continue;
-            }
-
-            if (!quiet)
-            {
-                mpr( "Your cursed equipment won't allow you to complete the "
-                     "transformation." );
-            }
-
-            return (true);
-        }
-    }
-    return (false);
-}
-
-// Count the stat boosts yielded by all items to be removed, and count
-// future losses (caused by the transformation) like a current stat boost,
-// as well. If the sum of all boosts of a stat is equal to or greater than
-// the current stat, give a message and return true.
-bool check_transformation_stat_loss(const std::set<equipment_type> &remove,
-                                    bool quiet, int str_loss, int dex_loss,
-                                    int int_loss)
-{
-    // Initialise with additional losses, if any.
-    int prop_str = str_loss;
-    int prop_dex = dex_loss;
-    int prop_int = int_loss;
-
-    // Might is very much temporary and might run out at any point during
-    // your transformation, possibly resulting in stat loss caused by a
-    // combination of an unequipping (and/or stat lowering) transformation
-    // and Might running out at an inopportune moment.
-    if (you.duration[DUR_MIGHT])
-        prop_str += 5;
-    if (you.duration[DUR_BRILLIANCE])
-        prop_int += 5;
-    if (you.duration[DUR_AGILITY])
-        prop_dex += 5;
-
-    if (prop_str >= you.strength
-        || prop_int >= you.intel
-        || prop_dex >= you.dex)
-    {
-        if (!quiet)
-        {
-            mpr("This transformation would result in fatal stat loss!",
-                MSGCH_WARN);
-        }
-        return (true);
-    }
-
-    // Check over all items to be removed or melded.
-    std::set<equipment_type>::const_iterator iter;
-    for (iter = remove.begin(); iter != remove.end(); ++iter)
-    {
-        equipment_type e = *iter;
-        if (you.equip[e] == -1)
-            continue;
-
-        const item_def& item = you.inv[you.equip[e]];
-
-        // There are no stat-boosting non-weapons/non-staves.
-        if (e == EQ_WEAPON
-            && item.base_type != OBJ_WEAPONS && item.base_type != OBJ_STAVES)
-        {
-            continue;
-        }
-
-        // Currently, the only non-artefacts which have stat-changing
-        // effects are rings.
-        if (item.base_type == OBJ_JEWELLERY)
-        {
-            if (!item_ident(item, ISFLAG_KNOW_PLUSES))
-                continue;
-
-            switch (item.sub_type)
-            {
-            case RING_STRENGTH:     prop_str += item.plus; break;
-            case RING_DEXTERITY:    prop_dex += item.plus; break;
-            case RING_INTELLIGENCE: prop_int += item.plus; break;
-            default:                                       break;
-            }
-        }
-        else if (item.base_type == OBJ_ARMOUR)
-        {
-            switch (get_armour_ego_type( item ))
-            {
-            case SPARM_STRENGTH:     prop_str += 3; break;
-            case SPARM_DEXTERITY:    prop_dex += 3; break;
-            case SPARM_INTELLIGENCE: prop_int += 3; break;
-            default:                                break;
-            }
-        }
-
-        if (is_artefact(item))
-        {
-            prop_str += artefact_known_wpn_property(item, ARTP_STRENGTH);
-            prop_int += artefact_known_wpn_property(item, ARTP_INTELLIGENCE);
-            prop_dex += artefact_known_wpn_property(item, ARTP_DEXTERITY);
-        }
-
-        // Since there might be multiple items whose effects cancel each other
-        // out while worn, if at any point in the order of checking this list
-        // (which is the same order as when removing items) one of your stats
-        // would reach 0, return true.
-        if (prop_str >= you.strength
-            || prop_int >= you.intel
-            || prop_dex >= you.dex)
-        {
-            if (!quiet)
-            {
-                mpr("This transformation would result in fatal stat loss!",
-                    MSGCH_WARN);
-            }
-            return (true);
-        }
-    }
-
-    return (false);
-}
-
-// Returns true if the player got prompted by an inscription warning and
-// chose to opt out.
-bool _check_transformation_inscription_warning(
-            const std::set<equipment_type> &remove)
-{
-    // Check over all items to be removed or melded.
-    std::set<equipment_type>::const_iterator iter;
-    for (iter = remove.begin(); iter != remove.end(); ++iter)
-    {
-        equipment_type e = *iter;
-        if (you.equip[e] == -1)
-            continue;
-
-        const item_def& item = you.inv[you.equip[e]];
-
-        operation_types op = OPER_WEAR;
-        if (e == EQ_WEAPON)
-            op = OPER_WIELD;
-        else if (item.base_type == OBJ_JEWELLERY)
-            op = OPER_PUTON;
-
-        if (!check_old_item_warning(item, op))
-            return (true);
-    }
-
-    return (false);
-}
-
 // FIXME: Switch to 4.1 transforms handling.
 size_type transform_size(int psize)
 {
-    return you.transform_size(psize);
+    return you.transform_size(you.form, psize);
 }
 
-size_type player::transform_size(int psize) const
+size_type player::transform_size(transformation_type tform, int psize) const
 {
-    const int transform = attribute[ATTR_TRANSFORMATION];
-    switch (transform)
+    switch (tform)
     {
     case TRAN_SPIDER:
     case TRAN_BAT:
@@ -495,11 +359,105 @@ static bool _abort_or_fizzle(bool just_check)
     if (!just_check && you.turn_is_over)
     {
         canned_msg(MSG_SPELL_FIZZLES);
+        move_player_to_grid(you.pos(), false, true);
         return (true); // pay the necessary costs
     }
     return (false); // SPRET_ABORT
 }
 
+monster_type transform_mons()
+{
+    switch(you.form)
+    {
+    case TRAN_SPIDER:
+        return MONS_SPIDER;
+    case TRAN_STATUE:
+        return MONS_STATUE;
+    case TRAN_ICE_BEAST:
+        return MONS_ICE_BEAST;
+    case TRAN_DRAGON:
+        return dragon_form_dragon_type();
+    case TRAN_LICH:
+        return MONS_LICH;
+    case TRAN_BAT:
+        return you.species == SP_VAMPIRE ? MONS_VAMPIRE_BAT : MONS_BAT;
+    case TRAN_PIG:
+        return MONS_HOG;
+    case TRAN_BLADE_HANDS:
+    case TRAN_NONE:
+        return MONS_PLAYER;
+    }
+    die("unknown transformation");
+    return MONS_PLAYER;
+}
+
+std::string blade_parts(bool terse)
+{
+    if (you.species == SP_CAT)
+        return terse ? "paws" : "front paws";
+    return "hands";
+}
+
+monster_type dragon_form_dragon_type()
+{
+    switch(you.species)
+    {
+        case SP_WHITE_DRACONIAN:
+             return MONS_ICE_DRAGON;
+        case SP_GREEN_DRACONIAN:
+             return MONS_SWAMP_DRAGON;
+        case SP_YELLOW_DRACONIAN:
+             return MONS_GOLDEN_DRAGON;
+        case SP_GREY_DRACONIAN:
+             return MONS_IRON_DRAGON;
+        case SP_BLACK_DRACONIAN:
+             return MONS_STORM_DRAGON;
+        case SP_PURPLE_DRACONIAN:
+             return MONS_QUICKSILVER_DRAGON;
+        case SP_MOTTLED_DRACONIAN:
+             return MONS_MOTTLED_DRAGON;
+        case SP_PALE_DRACONIAN:
+             return MONS_STEAM_DRAGON;
+        case SP_RED_DRACONIAN:
+        default:
+             return MONS_DRAGON;
+    }
+}
+
+bool feat_dangerous_for_form(transformation_type which_trans,
+                             dungeon_feature_type feat)
+{
+    // Everything is okay if we can fly.
+    if (form_can_fly(which_trans) || you.is_levitating())
+        return (false);
+
+    // We can only cling for safety if we're already doing so.
+    if (which_trans == TRAN_SPIDER && you.is_wall_clinging())
+        return (false);
+
+    if (feat == DNGN_LAVA)
+        return (true);
+
+    if (feat == DNGN_DEEP_WATER)
+        return (!form_likes_water(which_trans) && !beogh_water_walk());
+
+    return (false);
+}
+
+static bool _transformation_is_safe(transformation_type which_trans,
+                                    dungeon_feature_type feat, bool quiet)
+{
+    if (!feat_dangerous_for_form(which_trans, feat))
+        return (true);
+
+    if (!quiet)
+    {
+        mprf("You would %s in your new form.",
+             feat == DNGN_DEEP_WATER ? "drown" : "burn");
+    }
+    return (false);
+}
+
 // Transforms you into the specified form. If force is true, checks for
 // inscription warnings are skipped, and the transformation fails silently
 // (if it fails). If just_check is true the transformation doesn't actually
@@ -507,6 +465,18 @@ static bool _abort_or_fizzle(bool just_check)
 bool transform(int pow, transformation_type which_trans, bool force,
                bool just_check)
 {
+    transformation_type previous_trans = you.form;
+    bool was_in_water = you.in_water();
+    const flight_type was_flying = you.flight_mode();
+
+    // Zin's protection.
+    if (!just_check && you.religion == GOD_ZIN
+        && x_chance_in_y(you.piety, MAX_PIETY) && which_trans != TRAN_NONE)
+    {
+        simple_god_message(" protects your body from unnatural transformation!");
+        return (false);
+    }
+
     if (!force && crawl_state.is_god_acting())
         force = true;
 
@@ -518,30 +488,18 @@ bool transform(int pow, transformation_type which_trans, bool force,
         return (false);
     }
 
-    if (you.species == SP_MERFOLK && you.swimming()
-        && which_trans != TRAN_DRAGON && which_trans != TRAN_BAT)
-    {
-        // This might be overkill, but it's okay because obviously
-        // whatever magical ability that lets them walk on land is
-        // removed when they're in water (in this case, their natural
-        // form is completely over-riding any other... goes well with
-        // the forced transform when entering water)... but merfolk can
-        // transform into flying forms.
-        if (!force)
-            mpr("You cannot transform out of your normal form while in water.");
+    if (!_transformation_is_safe(which_trans, env.grid(you.pos()), force))
         return (false);
-    }
 
     // This must occur before the untransform() and the is_undead check.
-    if (you.attribute[ATTR_TRANSFORMATION]
-        == static_cast<unsigned>(which_trans))
+    if (previous_trans == which_trans)
     {
         if (you.duration[DUR_TRANSFORMATION] < 100 * BASELINE_DELAY)
         {
             if (just_check)
                 return (true);
 
-            if (which_trans==TRAN_PIG)
+            if (which_trans == TRAN_PIG)
                 mpr("You feel you'll be a pig longer.");
             else
                 mpr("You extend your transformation's duration.");
@@ -551,7 +509,7 @@ bool transform(int pow, transformation_type which_trans, bool force,
         }
         else
         {
-            if (!force && which_trans!=TRAN_PIG)
+            if (!force && which_trans != TRAN_PIG)
                 mpr("You cannot extend your transformation any further!");
             return (false);
         }
@@ -561,7 +519,7 @@ bool transform(int pow, transformation_type which_trans, bool force,
     // equipment). Ideally, untransforming should cost a turn but nothing
     // else (as does the "End Transformation" ability). As it is, you
     // pay with mana and hunger if you already untransformed.
-    if (!just_check && you.attribute[ATTR_TRANSFORMATION] != TRAN_NONE)
+    if (!just_check && previous_trans != TRAN_NONE)
     {
         bool skip_wielding = false;
         switch (which_trans)
@@ -574,7 +532,7 @@ bool transform(int pow, transformation_type which_trans, bool force,
             break;
         }
         // Skip wielding weapon if it gets unwielded again right away.
-        untransform(skip_wielding);
+        untransform(skip_wielding, true);
     }
 
     // Catch some conditions which prevent transformation.
@@ -599,30 +557,35 @@ bool transform(int pow, transformation_type which_trans, bool force,
 
     std::set<equipment_type> rem_stuff = _init_equipment_removal(which_trans);
 
-    if (which_trans != TRAN_PIG
-        && _check_for_cursed_equipment(rem_stuff, which_trans, force))
-    {
-        return (_abort_or_fizzle(just_check));
-    }
-
-    int str = 0, dex = 0, symbol = '@', colour = LIGHTGREY, xhp = 0, dur = 0;
+    int str = 0, dex = 0, xhp = 0, dur = 0;
     const char* tran_name = "buggy";
-    const char* msg = "You transform into something buggy!";
+    std::string msg;
+
+    if (was_in_water && form_can_fly(which_trans))
+        msg = "You fly out of the water as you turn into ";
+    else if (form_can_fly(previous_trans)
+             && form_can_swim(which_trans)
+             && feat_is_water(grd(you.pos())))
+        msg = "As you dive into the water, you turn into ";
+    else
+        msg = "You turn into ";
+
     switch (which_trans)
     {
     case TRAN_SPIDER:
         tran_name = "spider";
         dex       = 5;
-        symbol    = 's';
-        colour    = BROWN;
         dur       = std::min(10 + random2(pow) + random2(pow), 60);
-        msg       = "You turn into a venomous arachnid creature.";
+        msg      += "a venomous arachnid creature.";
         break;
 
     case TRAN_BLADE_HANDS:
         tran_name = "Blade Hands";
+        if (you.species == SP_CAT)
+            tran_name = "Blade Paws";
         dur       = std::min(10 + random2(pow), 100);
-        msg       = "Your hands turn into razor-sharp scythe blades.";
+        msg       = "Your " + blade_parts()
+                    + " turn into razor-sharp scythe blades.";
         break;
 
     case TRAN_STATUE:
@@ -630,45 +593,31 @@ bool transform(int pow, transformation_type which_trans, bool force,
         str       = 2;
         dex       = -2;
         xhp       = 15;
-        symbol    = '8';
-        colour    = LIGHTGREY;
         dur       = std::min(20 + random2(pow) + random2(pow), 100);
         if (player_genus(GENPC_DWARVEN) && one_chance_in(10))
             msg = "You inwardly fear your resemblance to a lawn ornament.";
         else
-            msg = "You turn into a living statue of rough stone.";
+            msg += "a living statue of rough stone.";
         break;
 
     case TRAN_ICE_BEAST:
         tran_name = "ice beast";
         xhp       = 12;
-        symbol    = 'I';
-        colour    = WHITE;
         dur       = std::min(30 + random2(pow) + random2(pow), 100);
-        msg       = "You turn into a creature of crystalline ice.";
+        msg      += "a creature of crystalline ice.";
         break;
 
     case TRAN_DRAGON:
         tran_name = "dragon";
         str       = 10;
         xhp       = 16;
-        symbol    = 'D';
-        colour    = GREEN;
         dur       = std::min(20 + random2(pow) + random2(pow), 100);
-        if (you.species == SP_MERFOLK && you.swimming())
-        {
-            msg = "You fly out of the water as you turn into "
-                  "a fearsome dragon!";
-        }
-        else
-            msg = "You turn into a fearsome dragon!";
+        msg      += "a fearsome dragon!";
         break;
 
     case TRAN_LICH:
         tran_name = "lich";
         str       = 3;
-        symbol    = 'L';
-        colour    = LIGHTGREY;
         dur       = std::min(20 + random2(pow) + random2(pow), 100);
         msg       = "Your body is suffused with negative energy!";
         break;
@@ -677,52 +626,36 @@ bool transform(int pow, transformation_type which_trans, bool force,
         tran_name = "bat";
         str       = -5;
         dex       = 5;
-        symbol    = 'b';
-        colour    = (you.species == SP_VAMPIRE ? DARKGREY : LIGHTGREY);
         dur       = std::min(20 + random2(pow) + random2(pow), 100);
         if (you.species == SP_VAMPIRE)
-            msg = "You turn into a vampire bat.";
+            msg += "a vampire bat.";
         else
-            msg = "You turn into a bat.";
+            msg += "a bat.";
         break;
 
     case TRAN_PIG:
         tran_name = "pig";
-        symbol    = 'h';
-        colour    = RED;
         dur       = pow;
         msg       = "You have been turned into a pig!";
         you.transform_uncancellable = true;
         break;
 
     case TRAN_NONE:
-    case NUM_TRANSFORMATIONS:
         break;
-    }
-
-    if (check_transformation_stat_loss(rem_stuff, force || which_trans == TRAN_PIG,
-                                       std::max(-str, 0), std::max(-dex,0)))
-    {   // would have died to stat loss
-        if (which_trans == TRAN_PIG)
-        {   // no easy way around this!
-            mpr("A dreadful feeling locks you in place!");
-            if (you.duration[DUR_PARALYSIS]<10 * BASELINE_DELAY)
-                you.duration[DUR_PARALYSIS]=10 * BASELINE_DELAY;
-        }
-        return (_abort_or_fizzle(just_check));
+    default:
+        msg += "something buggy!";
     }
 
     // If we're just pretending return now.
     if (just_check)
         return (true);
 
-    if (!force && _check_transformation_inscription_warning(rem_stuff))
-        return (_abort_or_fizzle(just_check));
-
     // All checks done, transformation will take place now.
     you.redraw_evasion      = true;
     you.redraw_armour_class = true;
     you.wield_change        = true;
+    if (form_changed_physiology(which_trans))
+        you.fishtail = false;
 
     // Most transformations conflict with stone skin.
     if (which_trans != TRAN_NONE
@@ -735,26 +668,24 @@ bool transform(int pow, transformation_type which_trans, bool force,
     // Give the transformation message.
     mpr(msg);
 
-    _remove_equipment(rem_stuff);
-
     // Update your status.
-    you.attribute[ATTR_TRANSFORMATION] = which_trans;
+    you.form = which_trans;
     you.set_duration(DUR_TRANSFORMATION, dur);
-    you.symbol = symbol;
-    you.colour = colour;
+    update_player_symbol();
 
+    _remove_equipment(rem_stuff);
     burden_change();
 
     if (str)
     {
-        modify_stat(STAT_STRENGTH, str, true,
+        notify_stat_change(STAT_STR, str, true,
                     make_stringf("gaining the %s transformation",
                                  tran_name).c_str());
     }
 
     if (dex)
     {
-        modify_stat(STAT_DEXTERITY, dex, true,
+        notify_stat_change(STAT_DEX, dex, true,
                     make_stringf("gaining the %s transformation",
                                  tran_name).c_str());
     }
@@ -766,7 +697,7 @@ bool transform(int pow, transformation_type which_trans, bool force,
     switch (which_trans)
     {
     case TRAN_STATUE:
-        if (you.duration[DUR_STONEMAIL] || you.duration[DUR_STONESKIN])
+        if (you.duration[DUR_STONESKIN])
             mpr("Your new body merges with your stone armour.");
         break;
 
@@ -780,6 +711,7 @@ bool transform(int pow, transformation_type which_trans, bool force,
         {
             mpr("The net rips apart!");
             you.attribute[ATTR_HELD] = 0;
+            you.redraw_quiver = true;
             int net = get_trapping_net(you.pos());
             if (net != NON_ITEM)
                 destroy_item(net);
@@ -806,6 +738,8 @@ bool transform(int pow, transformation_type which_trans, bool force,
         break;
     }
 
+    you.check_clinging(false);
+
     // This only has an effect if the transformation happens passively,
     // for example if Xom decides to transform you while you're busy
     // running around or butchering corpses.
@@ -814,12 +748,16 @@ bool transform(int pow, transformation_type which_trans, bool force,
     if (you.species != SP_VAMPIRE || which_trans != TRAN_BAT)
         transformation_expiration_warning();
 
-    return (true);
-}
+    // Re-check terrain now that be may no longer be swimming or flying.
+    if (was_flying && you.flight_mode() == FL_NONE
+                   || feat_is_water(grd(you.pos()))
+                      && which_trans == TRAN_BLADE_HANDS
+                      && you.species == SP_MERFOLK)
+    {
+        move_player_to_grid(you.pos(), false, true);
+    }
 
-bool transform_can_butcher_barehanded(transformation_type tt)
-{
-    return (tt == TRAN_BLADE_HANDS || tt == TRAN_DRAGON || tt == TRAN_ICE_BEAST);
+    return (true);
 }
 
 void untransform(bool skip_wielding, bool skip_move)
@@ -830,18 +768,15 @@ void untransform(bool skip_wielding, bool skip_move)
     you.redraw_armour_class = true;
     you.wield_change        = true;
 
-    you.symbol = '@';
-    you.colour = LIGHTGREY;
-
     // Must be unset first or else infinite loops might result. -- bwr
-    const transformation_type old_form =
-        static_cast<transformation_type>(you.attribute[ ATTR_TRANSFORMATION ]);
+    const transformation_type old_form = you.form;
 
     // We may have to unmeld a couple of equipment types.
     std::set<equipment_type> melded = _init_equipment_removal(old_form);
 
-    you.attribute[ATTR_TRANSFORMATION] = TRAN_NONE;
+    you.form = TRAN_NONE;
     you.duration[DUR_TRANSFORMATION]   = 0;
+    update_player_symbol();
 
     burden_change();
 
@@ -851,35 +786,35 @@ void untransform(bool skip_wielding, bool skip_move)
     {
     case TRAN_SPIDER:
         mpr("Your transformation has ended.", MSGCH_DURATION);
-        modify_stat( STAT_DEXTERITY, -5, true,
-                     "losing the spider transformation" );
+        notify_stat_change(STAT_DEX, -5, true,
+                     "losing the spider transformation");
+        if (!skip_move)
+            you.check_clinging(false);
         break;
 
     case TRAN_BAT:
         mpr("Your transformation has ended.", MSGCH_DURATION);
-        modify_stat( STAT_DEXTERITY, -5, true,
-                     "losing the bat transformation" );
-        modify_stat( STAT_STRENGTH, 5, true,
-                     "losing the bat transformation" );
+        notify_stat_change(STAT_DEX, -5, true,
+                     "losing the bat transformation");
+        notify_stat_change(STAT_STR, 5, true,
+                     "losing the bat transformation");
         break;
 
     case TRAN_BLADE_HANDS:
-        mpr( "Your hands revert to their normal proportions.", MSGCH_DURATION );
+        mprf(MSGCH_DURATION, "Your %s revert to their normal proportions.",
+             blade_parts().c_str());
         you.wield_change = true;
         break;
 
     case TRAN_STATUE:
-        mpr( "You revert to your normal fleshy form.", MSGCH_DURATION );
-        modify_stat( STAT_DEXTERITY, 2, true,
-                     "losing the statue transformation" );
-        modify_stat( STAT_STRENGTH, -2, true,
+        mpr("You revert to your normal fleshy form.", MSGCH_DURATION);
+        notify_stat_change(STAT_DEX, 2, true,
+                     "losing the statue transformation");
+        notify_stat_change(STAT_STR, -2, true,
                      "losing the statue transformation");
 
         // Note: if the core goes down, the combined effect soon disappears,
         // but the reverse isn't true. -- bwr
-        if (you.duration[DUR_STONEMAIL])
-            you.duration[DUR_STONEMAIL] = 1;
-
         if (you.duration[DUR_STONESKIN])
             you.duration[DUR_STONESKIN] = 1;
 
@@ -887,7 +822,7 @@ void untransform(bool skip_wielding, bool skip_move)
         break;
 
     case TRAN_ICE_BEAST:
-        mpr( "You warm up again.", MSGCH_DURATION );
+        mpr("You warm up again.", MSGCH_DURATION);
 
         // Note: if the core goes down, the combined effect soon disappears,
         // but the reverse isn't true. -- bwr
@@ -895,29 +830,24 @@ void untransform(bool skip_wielding, bool skip_move)
             you.duration[DUR_ICY_ARMOUR] = 1;
 
         hp_downscale = 12;
-
-        // Re-enter the terrain, it might kill us.
-        if (!skip_move && feat_is_water(grd(you.pos())))
-            move_player_to_grid(you.pos(), false, true, true);
-
         break;
 
     case TRAN_DRAGON:
-        mpr( "Your transformation has ended.", MSGCH_DURATION );
-        modify_stat(STAT_STRENGTH, -10, true,
-                    "losing the dragon transformation" );
+        mpr("Your transformation has ended.", MSGCH_DURATION);
+        notify_stat_change(STAT_STR, -10, true,
+                    "losing the dragon transformation");
         hp_downscale = 16;
         break;
 
     case TRAN_LICH:
-        mpr( "You feel yourself come back to life.", MSGCH_DURATION );
-        modify_stat(STAT_STRENGTH, -3, true,
-                    "losing the lich transformation" );
+        mpr("You feel yourself come back to life.", MSGCH_DURATION);
+        notify_stat_change(STAT_STR, -3, true,
+                    "losing the lich transformation");
         you.is_undead = US_ALIVE;
         break;
 
     case TRAN_PIG:
-        mpr( "Your transformation has ended.", MSGCH_DURATION );
+        mpr("Your transformation has ended.", MSGCH_DURATION);
         break;
 
     default:
@@ -926,11 +856,16 @@ void untransform(bool skip_wielding, bool skip_move)
 
     _unmeld_equipment(melded);
 
-    // Re-check terrain now that be may no longer be flying.
-    if (!skip_move && old_flight && you.flight_mode() == FL_NONE)
-        move_player_to_grid(you.pos(), false, true, true);
+    // Re-check terrain now that be may no longer be swimming or flying.
+    if (!skip_move && (old_flight && you.flight_mode() == FL_NONE
+                       || (feat_is_water(grd(you.pos()))
+                           && (old_form == TRAN_ICE_BEAST
+                               || you.species == SP_MERFOLK))))
+    {
+        move_player_to_grid(you.pos(), false, true);
+    }
 
-    if (transform_can_butcher_barehanded(old_form))
+    if (form_can_butcher_barehanded(old_form))
         stop_butcher_delay();
 
     // If nagas wear boots while transformed, they fall off again afterwards:
@@ -948,6 +883,17 @@ void untransform(bool skip_wielding, bool skip_move)
             remove_one_equip(EQ_BOOTS);
     }
 
+    // End Ozocubu's Icy Armour if you unmelded wearing heavy armour
+    if (you.duration[DUR_ICY_ARMOUR]
+        && !player_effectively_in_light_armour())
+    {
+        you.duration[DUR_ICY_ARMOUR] = 1;
+
+        const item_def *armour = you.slot_item(EQ_BODY_ARMOUR, false);
+        mprf(MSGCH_DURATION, "%s cracks your icy armour.",
+             armour->name(DESC_CAP_YOUR).c_str());
+    }
+
     if (hp_downscale != 10 && you.hp != you.hp_max)
     {
         you.hp = you.hp * 10 / hp_downscale;
@@ -968,7 +914,7 @@ void untransform(bool skip_wielding, bool skip_move)
 
 // XXX: This whole system is a mess as it still relies on special
 // cases to handle a large number of things (see wear_armour()) -- bwr
-bool can_equip( equipment_type use_which, bool ignore_temporary )
+bool can_equip(equipment_type use_which, bool ignore_temporary)
 {
     if (use_which == EQ_HELMET
         && (player_mutation_level(MUT_HORNS)
@@ -985,7 +931,7 @@ bool can_equip( equipment_type use_which, bool ignore_temporary )
 
     if (!ignore_temporary)
     {
-        switch (you.attribute[ATTR_TRANSFORMATION])
+        switch (you.form)
         {
         case TRAN_NONE:
         case TRAN_LICH:
@@ -998,6 +944,7 @@ bool can_equip( equipment_type use_which, bool ignore_temporary )
 
         case TRAN_STATUE:
             return (use_which == EQ_WEAPON
+                    || use_which == EQ_SHIELD
                     || use_which == EQ_CLOAK
                     || use_which == EQ_HELMET);
 
@@ -1012,7 +959,7 @@ bool can_equip( equipment_type use_which, bool ignore_temporary )
     return (true);
 }
 
-void _extra_hp(int amount_extra) // must also set in calc_hp
+static void _extra_hp(int amount_extra) // must also set in calc_hp
 {
     calc_hp();
 
@@ -1021,16 +968,3 @@ void _extra_hp(int amount_extra) // must also set in calc_hp
 
     deflate_hp(you.hp_max, false);
 }
-
-// Used to mark transformations which override species/mutation intrinsics.
-// If phys_scales is true then we're checking to see if the form keeps
-// the physical (AC/EV) properties from scales... the special intrinsic
-// features (resistances, etc.) are lost in those forms however.
-bool transform_changed_physiology( bool phys_scales )
-{
-    return (you.attribute[ATTR_TRANSFORMATION] != TRAN_NONE
-            && you.attribute[ATTR_TRANSFORMATION] != TRAN_BLADE_HANDS
-            && (!phys_scales
-                || (you.attribute[ATTR_TRANSFORMATION] != TRAN_LICH
-                    && you.attribute[ATTR_TRANSFORMATION] != TRAN_STATUE)));
-}
