@@ -1,5 +1,5 @@
-#!/usr/bin/perl -w
-
+#!/usr/bin/env perl
+use warnings;
 use strict;
 
 my $ART_ENUM = 'art-enum.h';
@@ -43,6 +43,7 @@ my %field_type = (
     MP       => "num",
     MUTATE   => "num",
     NAME     => "str",
+    NOGEN    => "bool",
     NOISES   => "bool",
     NOSPELL  => "bool",
     NOTELEP  => "bool",
@@ -77,12 +78,12 @@ my %field_type = (
     plus      => "num",
     plus2     => "num",
     base_type => "enum",
-    sub_type  => "enum"
+    sub_type  => "enum",
 );
 
 my %union_name = (
     melee_effect => "fight_func",
-    launch       => "fight_func"
+    launch       => "fight_func",
 );
 
 my @field_list = keys(%field_type);
@@ -150,7 +151,7 @@ sub finish_art
         {
             $artefact->{BRAND} = "SPWPN_NORMAL";
         }
-        elsif($type eq "OBJ_ARMOUR")
+        elsif ($type eq "OBJ_ARMOUR")
         {
             $artefact->{BRAND} = "SPARM_NORMAL";
         }
@@ -200,15 +201,15 @@ sub finish_art
             {
                 $artefact->{$field} = "";
             }
-            elsif($type eq "num" || $type eq "bool")
+            elsif ($type eq "num" || $type eq "bool")
             {
                 $artefact->{$field} = "0";
             }
-            elsif($type eq "enum")
+            elsif ($type eq "enum")
             {
                 error($artefact, "No enumeration for field '$field'");
             }
-            elsif($type eq "func")
+            elsif ($type eq "func")
             {
                 $artefact->{$field} = "NULL";
             }
@@ -230,7 +231,7 @@ sub finish_art
     my $flags = "";
     my $flag;
     foreach $flag ("SPECIAL", "HOLY", "EVIL", "CHAOTIC",
-                   "CORPSE_VIOLATING")
+                   "CORPSE_VIOLATING", "NOGEN")
     {
         if ($artefact->{$flag})
         {
@@ -356,7 +357,7 @@ sub process_line
         $artefact->{base_type} = $parts[0];
         $artefact->{sub_type}  = $parts[1];
     }
-    elsif($field eq "PLUS")
+    elsif ($field eq "PLUS")
     {
         my @parts = split(m!/!, $value);
 
@@ -511,7 +512,7 @@ sub art_to_str
                 $indent++;
                 $str .= "\n" . (" " x ($indent * 4));
             }
-            elsif($part eq "}")
+            elsif ($part eq "}")
             {
                 $indent--;
                 $str .= "\n" . (" " x ($indent * 4)) . "},";
@@ -568,10 +569,7 @@ sub write_data
 
     (my $guard = $ART_DATA) =~ tr/a-zA-Z/_/c;
     print HEADER <<"ENDofTEXT";
-/*
- * File:       $ART_DATA
- * Summary:    Definitions for unrandom artefacts.
- */
+/* Definitions for unrandom artefacts. */
 
 /**********************************************************************
  * WARNING!
@@ -688,7 +686,7 @@ ARTENUM
 sub write_tiles
 {
     my $tilefile = "dc-unrand.txt";
-    print "Updating $tilefile...\n";
+    print "    GEN $tilefile\n";
 
     die "Can't write to $tilefile\n"  if (-e $tilefile && !-w $tilefile);
     unless (open(TILES, ">$tilefile"))
@@ -707,26 +705,26 @@ sub write_tiles
             next;
         }
 
-        # The path always has the form /item/$folder/artefact.
+        # The path always has the form /item/$dir/artefact.
         my $type   = $artefact->{base_type} || "";
-        my $folder = "";
+        my $dir = "";
         if ($type eq "OBJ_WEAPONS")
         {
-            $folder = "weapon";
+            $dir = "weapon";
         }
         elsif ($type eq "OBJ_ARMOUR")
         {
-            $folder = "armour";
+            $dir = "armour";
         }
         elsif ($type eq "OBJ_JEWELLERY")
         {
             if ($artefact->{sub_type} =~ /RING_/)
             {
-                $folder = "ring";
+                $dir = "ring";
             }
             else
             {
-                $folder = "amulet";
+                $dir = "amulet";
             }
         }
         else
@@ -736,20 +734,20 @@ sub write_tiles
 
         my $definition = "$artefact->{TILE} UNRAND_$artefact->{_ENUM}";
         my $needrim    = ($artefact->{TILERIM} ? "1" : "0");
-        if (defined $art_by_type{$folder})
+        if (defined $art_by_type{$dir})
         {
-            if (defined $art_by_type{$folder}{$needrim})
+            if (defined $art_by_type{$dir}{$needrim})
             {
-                push @{$art_by_type{$folder}{$needrim}}, $definition;
+                push @{$art_by_type{$dir}{$needrim}}, $definition;
             }
             else
             {
-                $art_by_type{$folder}{$needrim} = [$definition];
+                $art_by_type{$dir}{$needrim} = [$definition];
             }
         }
         else
         {
-            $art_by_type{$folder} = {$needrim => [$definition]};
+            $art_by_type{$dir} = {$needrim => [$definition]};
         }
     }
 
@@ -878,7 +876,7 @@ HEADER_END
     # Create tiledef-unrand.cc for the function unrandart_to_tile().
     # Should we also create tiledef-unrand.h this way?
     $tilefile = "tiledef-unrand.cc";
-    print "Updating $tilefile...\n";
+    print "    GEN $tilefile\n";
 
     die "Can't write to $tilefile\n"  if (-e $tilefile && !-w $tilefile);
     unless (open(TILES, ">$tilefile"))
@@ -929,7 +927,7 @@ HEADER_END
         print TILES (" " x 4) . "case $enum:"
             . " " x ($longest_enum - length($enum) + 2) . "return TILE_$enum;\n";
     }
-    print TILES (" " x 4) . "default: return -1;\n";
+    print TILES (" " x 4) . "default: return 0;\n";
     print TILES (" " x 4) . "}\n";
     print TILES "}\n\n";
 
@@ -953,7 +951,7 @@ HEADER_END
                 . " " x ($longest_enum - length($enum) + 2) . "return $t_enum;\n";
         }
     }
-    print TILES (" " x 4) . "default: return -1;\n";
+    print TILES (" " x 4) . "default: return 0;\n";
     print TILES (" " x 4) . "}\n";
     print TILES "}\n\n";
     close(TILES);
