@@ -1,8 +1,7 @@
-/*
- *  File:       wiz-dgn.h
- *  Summary:    Dungeon related wizard functions.
- *  Written by: Linley Henzell and Jesse Jones
- */
+/**
+ * @file
+ * @brief Dungeon related wizard functions.
+**/
 
 #include "AppHdr.h"
 
@@ -13,6 +12,7 @@
 #include "coord.h"
 #include "coordit.h"
 #include "delay.h"
+#include "dgn-actions.h"
 #include "dungeon.h"
 #include "effects.h"
 #include "env.h"
@@ -28,8 +28,12 @@
 #include "place.h"
 #include "player.h"
 #include "religion.h"
+#include "stairs.h"
 #include "stuff.h"
 #include "terrain.h"
+#ifdef USE_TILE
+ #include "tileview.h"
+#endif
 #include "travel.h"
 #include "traps.h"
 #include "view.h"
@@ -118,7 +122,7 @@ static dungeon_feature_type _find_appropriate_stairs(bool down)
     return DNGN_UNSEEN;
 }
 
-void wizard_place_stairs( bool down )
+void wizard_place_stairs(bool down)
 {
     dungeon_feature_type stairs = _find_appropriate_stairs(down);
 
@@ -130,7 +134,7 @@ void wizard_place_stairs( bool down )
 
 // Try to find and use stairs already in the portal vault level,
 // since this might be a multi-level portal vault like a ziggurat.
-bool _take_portal_vault_stairs( const bool down )
+static bool _take_portal_vault_stairs(const bool down)
 {
     ASSERT(you.level_type == LEVEL_PORTAL_VAULT);
 
@@ -154,14 +158,14 @@ bool _take_portal_vault_stairs( const bool down )
     you.set_position(stair_pos);
 
     if (down)
-        down_stairs(you.absdepth0);
+        down_stairs();
     else
         up_stairs();
 
     return (true);
 }
 
-void wizard_level_travel( bool down )
+void wizard_level_travel(bool down)
 {
     if (you.level_type == LEVEL_PORTAL_VAULT)
         if (_take_portal_vault_stairs(down))
@@ -182,7 +186,7 @@ void wizard_level_travel( bool down )
     }
 
     if (down)
-        down_stairs(you.absdepth0, stairs);
+        down_stairs(stairs);
     else
         up_stairs(stairs);
 }
@@ -194,32 +198,36 @@ static void _wizard_go_to_level(const level_pos &pos)
         abs_depth > you.absdepth0? DNGN_STONE_STAIRS_DOWN_I
                                   : DNGN_STONE_STAIRS_UP_I;
 
-    if (abs_depth > you.absdepth0 && pos.id.depth == 1
+    if (pos.id.depth == branches[pos.id.branch].depth)
+        stair_taken = DNGN_STONE_STAIRS_DOWN_I;
+
+    if (pos.id.branch != you.where_are_you && pos.id.depth == 1
         && pos.id.branch != BRANCH_MAIN_DUNGEON)
     {
         stair_taken = branches[pos.id.branch].entry_stairs;
     }
 
-    const int old_level = you.absdepth0;
-    const branch_type old_where = you.where_are_you;
-    const level_area_type old_level_type = you.level_type;
+    const level_id old_level = level_id::current();
+    const bool keep_travel_data = can_travel_interlevel();
 
     you.level_type    = LEVEL_DUNGEON;
     you.where_are_you = static_cast<branch_type>(pos.id.branch);
     you.absdepth0    = abs_depth;
 
-    const bool newlevel = load(stair_taken, LOAD_ENTER_LEVEL, old_level_type,
-        old_level, old_where);
+    const bool newlevel = load(stair_taken, LOAD_ENTER_LEVEL, old_level);
 #ifdef USE_TILE
-    TileNewLevel(newlevel);
+    tile_new_level(newlevel);
 #else
     UNUSED(newlevel);
 #endif
     if (!crawl_state.test)
         save_game_state();
     new_level();
-    viewwindow(true);
+    seen_monsters_react();
+    viewwindow();
 
+    if (!keep_travel_data)
+        travel_cache.erase_level_info(old_level);
     // Tell stash-tracker and travel that we've changed levels.
     trackers_init_new_level(true);
 }
@@ -245,7 +253,7 @@ void wizard_create_portal()
     char specs[256];
     if (cancelable_get_line(specs, sizeof(specs)))
     {
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         return;
     }
 
@@ -267,6 +275,7 @@ void wizard_create_portal()
         marker->set_property("feature_description",
                              "wizard portal, dest = " + dst);
         env.markers.add(marker);
+        env.markers.clear_need_activate();
         dungeon_terrain_changed(you.pos(), DNGN_ENTER_PORTAL_VAULT, false);
     }
 }
@@ -338,9 +347,22 @@ void wizard_create_feature()
             return;
         }
 
-        dungeon_terrain_changed(you.pos(), feat, false);
 #ifdef USE_TILE
         env.tile_flv(you.pos()).special = 0;
+        const dungeon_feature_type old_feat = grd(you.pos());
+#endif
+        dungeon_terrain_changed(you.pos(), feat, false);
+#ifdef USE_TILE
+        // Update gate tiles, if existing.
+        if (feat_is_door(old_feat) || feat_is_door(feat))
+        {
+            const coord_def left  = you.pos() - coord_def(1, 0);
+            const coord_def right = you.pos() + coord_def(1, 0);
+            if (map_bounds(left) && feat_is_door(grd(left)))
+                tile_init_flavour(left);
+            if (map_bounds(right) && feat_is_door(grd(right)))
+                tile_init_flavour(right);
+        }
 #endif
     }
     else
@@ -351,7 +373,7 @@ void wizard_list_branches()
 {
     for (int i = 0; i < NUM_BRANCHES; ++i)
     {
-        if (branches[i].startdepth != - 1)
+        if (branches[i].startdepth != -1)
         {
             mprf(MSGCH_DIAGNOSTICS, "Branch %d (%s) is on level %d of %s",
                  i, branches[i].longname, branches[i].startdepth,
@@ -400,11 +422,11 @@ void wizard_list_branches()
             }
         }
 
-        mprf(MSGCH_DIAGNOSTICS, "%lu on D:%lu (%s)", temples.size(),
+        mprf(MSGCH_DIAGNOSTICS, "%u on D:%u (%s)", temples.size(),
              i + 1,
-             comma_separated_line( god_names.begin(),
-                                   god_names.end() ).c_str()
-            );
+             comma_separated_line(god_names.begin(),
+                                   god_names.end()).c_str()
+          );
     }
 }
 
@@ -421,7 +443,7 @@ void wizard_map_level()
     {
         if (!yesno("Force level to be mappable?", true, 'n'))
         {
-            canned_msg( MSG_OK );
+            canned_msg(MSG_OK);
             return;
         }
 
@@ -460,8 +482,8 @@ void debug_make_trap()
         return;
     }
 
-    mprf(MSGCH_PROMPT, "What kind of trap? ");
-    get_input_line( requested_trap, sizeof( requested_trap ) );
+    msgwin_get_line("What kind of trap? ",
+                    requested_trap, sizeof(requested_trap));
     if (!*requested_trap)
         return;
 
@@ -505,9 +527,10 @@ void debug_make_trap()
         }
     }
 
-    place_specific_trap(you.pos(), trap);
-
-    mprf("Created a %s trap, marked it undiscovered", trap_name(trap));
+    if (place_specific_trap(you.pos(), trap))
+        mprf("Created a %s trap, marked it undiscovered", trap_name(trap));
+    else
+        mpr("Could not create trap - too many traps on level.");
 
     if (trap == TRAP_SHAFT && !is_valid_shaft_level())
         mpr("NOTE: Shaft traps aren't valid on this level.");
@@ -542,8 +565,8 @@ void debug_make_shop()
         return;
     }
 
-    mprf(MSGCH_PROMPT, "What kind of shop? ");
-    get_input_line( requested_shop, sizeof( requested_shop ) );
+    msgwin_get_line("What kind of shop? ",
+                    requested_shop, sizeof(requested_shop));
     if (!*requested_shop)
         return;
 
@@ -630,7 +653,14 @@ static void debug_load_map_by_name(std::string name)
     }
 
     if (dgn_place_map(toplace, true, false, where))
+    {
         mprf("Successfully placed %s.", toplace->name.c_str());
+#ifdef USE_TILE
+        // Fix up doors from vaults and any changes to the default walls
+        // and floors from the vault.
+        tile_init_flavour();
+#endif
+    }
     else
         mprf("Failed to place %s.", toplace->name.c_str());
 }
@@ -666,12 +696,12 @@ static void _debug_kill_traps()
 
 static int _debug_time_explore()
 {
-    viewwindow(false);
+    viewwindow();
     start_explore(false);
 
     unwind_var<int> es(Options.explore_stop, 0);
 
-    const long start = you.num_turns;
+    const int start = you.num_turns;
     while (you_are_delayed())
     {
         you.turn_is_over = false;
@@ -740,4 +770,75 @@ void debug_shift_labyrinth()
     }
     change_labyrinth(true);
 }
+
+void wizard_list_levels()
+{
+    travel_cache.update_da_counters();
+
+    std::vector<level_id> levs = travel_cache.known_levels();
+
+    mpr("Known levels:");
+    for (unsigned int i = 0; i < levs.size(); i++)
+    {
+        const LevelInfo* lv = travel_cache.find_level_info(levs[i]);
+        ASSERT(lv);
+
+        std::string cnts = "";
+        for (int j = 0; j < NUM_DA_COUNTERS; j++)
+        {
+            char num[20];
+            sprintf(num, "%d/", lv->da_counters[j]);
+            cnts += num;
+        }
+        mprf(MSGCH_DIAGNOSTICS, i+1, // inhibit merging
+             "%-10s : %s", levs[i].describe().c_str(), cnts.c_str());
+    }
+
+    std::string cnts = "";
+    for (int j = 0; j < NUM_DA_COUNTERS; j++)
+    {
+        char num[20];
+        sprintf(num, "%d/", query_da_counter((daction_type)j));
+        cnts += num;
+    }
+    mprf("%-10s : %s", "`- total", cnts.c_str());
+}
+
+void wizard_recreate_level()
+{
+    // Need to allow reuse of vaults, otherwise we'd run out of them fast.
+    for (rectangle_iterator ri(MAPGEN_BORDER); ri; ++ri)
+        env.level_map_ids(*ri) = INVALID_MAP_INDEX;
+    for (vault_placement_refv::const_iterator vp = env.level_vaults.begin();
+         vp != env.level_vaults.end(); ++vp)
+    {
+        (*vp)->seen = false;
+    }
+    dgn_erase_unused_vault_placements();
+
+    level_id lev = level_id::current();
+    dungeon_feature_type stair_taken = DNGN_STONE_STAIRS_DOWN_I;
+
+    if (lev.depth == 1 && lev != BRANCH_MAIN_DUNGEON)
+        stair_taken = branches[lev.branch].entry_stairs;
+
+    if (lev.level_type == LEVEL_DUNGEON)
+        you.get_place_info().levels_seen--;
+    Generated_Levels.erase(lev);
+    const bool newlevel = load(stair_taken, LOAD_START_GAME, lev);
+#ifdef USE_TILE
+    tile_new_level(newlevel);
+#else
+    UNUSED(newlevel);
+#endif
+    if (!crawl_state.test)
+        save_game_state();
+    new_level();
+    seen_monsters_react();
+    viewwindow();
+
+    travel_cache.erase_level_info(lev);
+    trackers_init_new_level(true);
+}
+
 #endif
