@@ -1,8 +1,7 @@
-/*
- *  File:       wiz-item.cc
- *  Summary:    Item related wizard functions.
- *  Written by: Linley Henzell and Jesse Jones
- */
+/**
+ * @file
+ * @brief Item related wizard functions.
+**/
 
 #include "AppHdr.h"
 
@@ -10,6 +9,7 @@
 
 #include <errno.h>
 
+#include "acquire.h"
 #include "artefact.h"
 #include "coordit.h"
 #include "message.h"
@@ -18,17 +18,19 @@
 #include "decks.h"
 #include "effects.h"
 #include "env.h"
+#include "godpassive.h"
 #include "itemprop.h"
 #include "items.h"
 #include "item_use.h"
-#include "it_use2.h"
 #include "invent.h"
 #include "makeitem.h"
+#include "mapdef.h"
 #include "mon-iter.h"
 #include "mon-stuff.h"
 #include "mon-util.h"
 #include "options.h"
 #include "output.h"
+#include "player-equip.h"
 #include "religion.h"
 #include "skills2.h"
 #include "spl-book.h"
@@ -62,6 +64,25 @@ static void _make_all_books()
     }
 }
 
+void wizard_create_spec_object_by_name()
+{
+    char buf[500];
+    mprf(MSGCH_PROMPT, "Enter name of item: ");
+    if (cancelable_get_line(buf, sizeof buf) || !*buf)
+    {
+        canned_msg(MSG_OK);
+        return;
+    }
+
+    std::string error;
+    create_item_named(buf, you.pos(), &error);
+    if (!error.empty())
+    {
+        mprf(MSGCH_ERROR, "Error: %s", error.c_str());
+        return;
+    }
+}
+
 //---------------------------------------------------------------
 //
 // create_spec_object
@@ -118,7 +139,7 @@ void wizard_create_spec_object()
             class_wanted = OBJ_FOOD;
         else if (keyin == '$')
             class_wanted = OBJ_GOLD;
-        else if (keyin == ESCAPE || keyin == ' '
+        else if (key_is_escape(keyin) || keyin == ' '
                 || keyin == '\r' || keyin == '\n')
         {
             msgwin_reply("");
@@ -145,10 +166,10 @@ void wizard_create_spec_object()
     }
     else if (class_wanted == OBJ_GOLD)
     {
-        int amount = debug_prompt_for_int( "How much gold? ", true );
+        int amount = prompt_for_int("How much gold? ", true);
         if (amount <= 0)
         {
-            canned_msg( MSG_OK );
+            canned_msg(MSG_OK);
             return;
         }
 
@@ -166,7 +187,7 @@ void wizard_create_spec_object()
             return;
         }
 
-        if (mons_weight(mon) <= 0)
+        if (mons_weight(mon) == 0)
         {
             if (!yesno("That monster doesn't leave corpses; make one "
                        "anyway?", true, 'y'))
@@ -181,13 +202,14 @@ void wizard_create_spec_object()
             mon = MONS_DRACONIAN;
         }
 
-        monsters dummy;
+        monster dummy;
         dummy.type = mon;
 
         if (mons_genus(mon) == MONS_HYDRA)
-            dummy.number = debug_prompt_for_int("How many heads? ", false);
+            dummy.number = prompt_for_int("How many heads? ", false);
 
-        if (fill_out_corpse(&dummy, mitm[thing_created], true) == -1)
+        if (fill_out_corpse(&dummy, dummy.type,
+                            mitm[thing_created], true) == -1)
         {
             mpr("Failed to create corpse.");
             mitm[thing_created].clear();
@@ -204,7 +226,7 @@ void wizard_create_spec_object()
         std::string temp = specs;
         trim_string(temp);
         lowercase(temp);
-        strcpy(specs, temp.c_str());
+        strlcpy(specs, temp.c_str(), sizeof(specs));
 
         if (class_wanted == OBJ_BOOKS && temp == "all")
         {
@@ -214,7 +236,7 @@ void wizard_create_spec_object()
 
         if (specs[0] == '\0')
         {
-            canned_msg( MSG_OK );
+            canned_msg(MSG_OK);
             return;
         }
 
@@ -235,9 +257,9 @@ void wizard_create_spec_object()
 
     // Deck colour (which control rarity) already set.
     if (!is_deck(mitm[thing_created]))
-        item_colour( mitm[thing_created] );
+        item_colour(mitm[thing_created]);
 
-    move_item_to_grid( &thing_created, you.pos() );
+    move_item_to_grid(&thing_created, you.pos());
 
     if (thing_created != NON_ITEM)
     {
@@ -289,7 +311,7 @@ const char* _prop_name[ARTP_NUM_PROPERTIES] = {
 #define ARTP_VAL_POS  1
 #define ARTP_VAL_ANY  2
 
-char _prop_type[ARTP_NUM_PROPERTIES] = {
+int8_t _prop_type[ARTP_NUM_PROPERTIES] = {
     ARTP_VAL_POS,  //BRAND
     ARTP_VAL_ANY,  //AC
     ARTP_VAL_ANY,  //EVASION
@@ -356,9 +378,9 @@ static void _tweak_randart(item_def &item)
             choice = '1' + choice_num - 26;
 
         if (props[i])
-            sprintf(buf, "%c) <w>%-5s</w> ", choice, _prop_name[i]);
+            snprintf(buf, sizeof(buf), "%c) <w>%-5s</w> ", choice, _prop_name[i]);
         else
-            sprintf(buf, "%c) %-5s ", choice, _prop_name[i]);
+            snprintf(buf, sizeof(buf), "%c) %-5s ", choice, _prop_name[i]);
 
         prompt += buf;
 
@@ -368,12 +390,12 @@ static void _tweak_randart(item_def &item)
 
     mpr("Change which field? ", MSGCH_PROMPT);
 
-    char     keyin = tolower( get_ch() );
+    char     keyin = tolower(get_ch());
     unsigned int  choice;
 
-    if (isalpha(keyin))
+    if (isaalpha(keyin))
         choice = keyin - 'a';
-    else if (isdigit(keyin) && keyin != '0')
+    else if (isadigit(keyin) && keyin != '0')
         choice = keyin - '1' + 26;
     else
         return;
@@ -397,7 +419,7 @@ static void _tweak_randart(item_def &item)
 
     case ARTP_VAL_POS:
         mprf(MSGCH_PROMPT, "%s was %d.", _prop_name[prop], props[prop]);
-        val = debug_prompt_for_int("New value? ", true);
+        val = prompt_for_int("New value? ", true);
 
         if (val < 0)
         {
@@ -410,7 +432,7 @@ static void _tweak_randart(item_def &item)
         break;
     case ARTP_VAL_ANY:
         mprf(MSGCH_PROMPT, "%s was %d.", _prop_name[prop], props[prop]);
-        val = debug_prompt_for_int("New value? ", false);
+        val = prompt_for_int("New value? ", false);
         artefact_set_property(item, static_cast<artefact_prop_type>(prop),
                              val);
         break;
@@ -428,7 +450,7 @@ void wizard_tweak_object(void)
     int item = prompt_invent_item("Tweak which item? ", MT_INVLIST, -1);
     if (item == PROMPT_ABORT)
     {
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         return;
     }
 
@@ -439,7 +461,7 @@ void wizard_tweak_object(void)
 
     while (true)
     {
-        void *field_ptr = NULL;
+        int64_t old_val = 0; // flags are uint64_t, but they don't care
 
         while (true)
         {
@@ -458,22 +480,22 @@ void wizard_tweak_object(void)
 
             mpr("Which field? ", MSGCH_PROMPT);
 
-            keyin = tolower( get_ch() );
+            keyin = tolower(get_ch());
 
             if (keyin == 'a')
-                field_ptr = &(you.inv[item].plus);
+                old_val = you.inv[item].plus;
             else if (keyin == 'b')
-                field_ptr = &(you.inv[item].plus2);
+                old_val = you.inv[item].plus2;
             else if (keyin == 'c')
-                field_ptr = &(you.inv[item].special);
+                old_val = you.inv[item].special;
             else if (keyin == 'd')
-                field_ptr = &(you.inv[item].quantity);
+                old_val = you.inv[item].quantity;
             else if (keyin == 'e')
-                field_ptr = &(you.inv[item].flags);
-            else if (keyin == ESCAPE || keyin == ' '
+                old_val = you.inv[item].flags;
+            else if (key_is_escape(keyin) || keyin == ' '
                     || keyin == '\r' || keyin == '\n')
             {
-                canned_msg( MSG_OK );
+                canned_msg(MSG_OK);
                 return;
             }
 
@@ -487,45 +509,51 @@ void wizard_tweak_object(void)
             continue;
         }
 
-        if (keyin != 'c' && keyin != 'e')
-        {
-            const short *const ptr = static_cast< short * >( field_ptr );
-            mprf("Old value: %d (0x%04x)", *ptr, *ptr );
-        }
+        if (keyin != 'e')
+            mprf("Old value: %"PRId64" (0x%04"PRIx64")", old_val, old_val);
         else
-        {
-            const long *const ptr = static_cast< long * >( field_ptr );
-            mprf("Old value: %ld (0x%08lx)", *ptr, *ptr );
-        }
-
-        mpr("New value? ", MSGCH_PROMPT);
-        get_input_line( specs, sizeof( specs ) );
+            mprf("Old value: 0x%08"PRIx64, old_val);
 
+        msgwin_get_line("New value? ", specs, sizeof(specs));
         if (specs[0] == '\0')
             return;
 
         char *end;
         const bool hex = (keyin == 'e');
-        int   new_value = strtol(specs, &end, hex ? 16 : 0);
+        int64_t new_val = strtoll(specs, &end, hex ? 16 : 0);
 
-        if (new_value == 0 && end == specs)
+        if (keyin == 'e' && new_val & ISFLAG_ARTEFACT_MASK
+            && (!you.inv[item].props.exists(KNOWN_PROPS_KEY)
+             || !you.inv[item].props.exists(ARTEFACT_PROPS_KEY)))
+        {
+            mpr("You can't set this flag on a non-artefact.");
+            continue;
+        }
+
+        if (end == specs)
             return;
 
-        if (keyin != 'c' && keyin != 'e')
-        {
-            short *ptr = static_cast< short * >( field_ptr );
-            *ptr = new_value;
-        }
+        if (keyin == 'a')
+            you.inv[item].plus = new_val;
+        else if (keyin == 'b')
+            you.inv[item].plus2 = new_val;
+        else if (keyin == 'c')
+            you.inv[item].special = new_val;
+        else if (keyin == 'd')
+            you.inv[item].quantity = new_val;
+        else if (keyin == 'e')
+            you.inv[item].flags = new_val;
         else
-        {
-            long *ptr = static_cast< long * >( field_ptr );
-            *ptr = new_value;
-        }
+            die("unhandled keyin");
+
+        // cursedness might have changed
+        ash_check_bondage();
+        ash_id_inventory();
     }
 }
 
 // Returns whether an item of this type can be an artefact.
-static bool _item_type_can_be_artefact( int type)
+static bool _item_type_can_be_artefact(int type)
 {
     return (type == OBJ_WEAPONS || type == OBJ_ARMOUR || type == OBJ_JEWELLERY
             || type == OBJ_BOOKS);
@@ -538,7 +566,7 @@ static bool _make_book_randart(item_def &book)
     do
     {
         mpr("Make book fixed [t]heme or fixed [l]evel? ", MSGCH_PROMPT);
-        type = tolower(getch());
+        type = tolower(getchk());
     }
     while (type != 't' && type != 'l');
 
@@ -550,7 +578,7 @@ static bool _make_book_randart(item_def &book)
 
 void wizard_value_artefact()
 {
-    int i = prompt_invent_item( "Value of which artefact?", MT_INVLIST, -1 );
+    int i = prompt_invent_item("Value of which artefact?", MT_INVLIST, -1);
 
     if (!prompt_failed(i))
     {
@@ -610,8 +638,8 @@ void wizard_create_all_artefacts()
 
 void wizard_make_object_randart()
 {
-    int i = prompt_invent_item( "Make an artefact out of which item?",
-                                MT_INVLIST, -1 );
+    int i = prompt_invent_item("Make an artefact out of which item?",
+                                MT_INVLIST, -1);
 
     if (prompt_failed(i))
         return;
@@ -630,8 +658,13 @@ void wizard_make_object_randart()
         return;
     }
 
-    if (you.weapon() == &item)
-        you.wield_change = true;
+    const equipment_type eq = item_equip_slot(item);
+    int invslot = 0;
+    if (eq != EQ_NONE)
+    {
+        invslot = you.equip[eq];
+        unequip_item(eq);
+    }
 
     if (is_random_artefact(item))
     {
@@ -641,8 +674,10 @@ void wizard_make_object_randart()
             return;
         }
 
-        if (item_is_equipped(item))
-            unuse_artefact(item);
+        // need to trim before the object changes, or else the old properties
+        // won't be removed
+        if (Options.autoinscribe_artefacts)
+            trim_randart_inscrip(item);
 
         item.special = 0;
         item.flags  &= ~ISFLAG_RANDART;
@@ -652,9 +687,9 @@ void wizard_make_object_randart()
     mpr("Fake item as gift from which god (ENTER to leave alone): ",
         MSGCH_PROMPT);
     char name[80];
-    if (!cancelable_get_line(name, sizeof( name )) && name[0])
+    if (!cancelable_get_line(name, sizeof(name)) && name[0])
     {
-        god_type god = string_to_god(name, false);
+        god_type god = str_to_god(name, false);
         if (god == GOD_NO_GOD)
            mpr("No such god, leaving item origin alone.");
         else
@@ -681,9 +716,9 @@ void wizard_make_object_randart()
     if (Options.autoinscribe_artefacts)
         add_autoinscription(item, artefact_auto_inscription(you.inv[i]));
 
-    // If equipped, apply new randart benefits.
-    if (item_is_equipped(item))
-        use_artefact(item);
+    // If it was equipped, requip the item.
+    if (eq != EQ_NONE)
+        equip_item(eq, invslot);
 
     mpr(item.name(DESC_INVENTORY_EQUIP).c_str());
 }
@@ -691,7 +726,8 @@ void wizard_make_object_randart()
 // Returns whether an item of this type can be cursed.
 static bool _item_type_can_be_cursed(int type)
 {
-    return (type == OBJ_WEAPONS || type == OBJ_ARMOUR || type == OBJ_JEWELLERY);
+    return (type == OBJ_WEAPONS || type == OBJ_ARMOUR || type == OBJ_JEWELLERY
+            || type == OBJ_STAVES);
 }
 
 void wizard_uncurse_item()
@@ -717,7 +753,7 @@ void wizard_identify_pack()
     for (int i = 0; i < ENDOFPACK; ++i)
     {
         item_def& item = you.inv[i];
-        if (item.is_valid())
+        if (item.defined())
         {
             set_ident_type(item, ID_KNOWN_TYPE);
             set_ident_flags(item, ISFLAG_IDENT_MASK);
@@ -735,7 +771,7 @@ void wizard_unidentify_pack()
     for (int i = 0; i < ENDOFPACK; ++i)
     {
         item_def& item = you.inv[i];
-        if (item.is_valid())
+        if (item.defined())
         {
             set_ident_type(item, ID_UNKNOWN_TYPE);
             unset_ident_flags(item, ISFLAG_IDENT_MASK);
@@ -747,7 +783,7 @@ void wizard_unidentify_pack()
     // Forget things that nearby monsters are carrying, as well.
     // (For use with the "give monster an item" wizard targeting
     // command.)
-    for (monster_iterator mon(&you.get_los()); mon; ++mon)
+    for (monster_iterator mon(you.get_los()); mon; ++mon)
     {
         for (int j = 0; j < NUM_MONSTER_SLOTS; ++j)
         {
@@ -756,7 +792,7 @@ void wizard_unidentify_pack()
 
             item_def &item = mitm[mon->inv[j]];
 
-            if (!item.is_valid())
+            if (!item.defined())
                 continue;
 
             set_ident_type(item, ID_UNKNOWN_TYPE);
@@ -787,24 +823,24 @@ void wizard_list_items()
                     mpr(si->name(DESC_PLAIN, false, false, false).c_str());
             }
 
-        mpr(EOL);
+        mpr("");
     }
 
     mpr("Item stacks (by location and top item):");
     for (int i = 0; i < MAX_ITEMS; ++i)
     {
         item_def &item(mitm[i]);
-        if (!item.is_valid() || item.held_by_monster())
+        if (!item.defined() || item.held_by_monster())
             continue;
 
         if (item.link != NON_ITEM)
         {
             mprf("(%2d,%2d): %s", item.pos.x, item.pos.y,
-                 item.name(DESC_PLAIN, false, false, false).c_str() );
+                 item.name(DESC_PLAIN, false, false, false).c_str());
         }
     }
 
-    mpr(EOL);
+    mpr("");
     mpr("Floor items (stacks only show top item):");
 
     const coord_def start(1,1), end(GXM-1, GYM-1);
@@ -841,8 +877,8 @@ static void _debug_acquirement_stats(FILE *ostat)
         MSGCH_PROMPT);
 
     object_class_type type;
-    const int keyin = tolower( get_ch() );
-    switch ( keyin )
+    const int keyin = tolower(get_ch());
+    switch (keyin)
     {
     case 'a': type = OBJ_WEAPONS;    break;
     case 'b': type = OBJ_ARMOUR;     break;
@@ -853,15 +889,15 @@ static void _debug_acquirement_stats(FILE *ostat)
     case 'g': type = OBJ_MISCELLANY; break;
     case 'h': type = OBJ_FOOD;       break;
     default:
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         return;
     }
 
-    const int num_itrs = debug_prompt_for_int("How many iterations? ", true);
+    const int num_itrs = prompt_for_int("How many iterations? ", true);
 
     if (num_itrs == 0)
     {
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         return;
     }
 
@@ -882,7 +918,7 @@ static void _debug_acquirement_stats(FILE *ostat)
     {
         if (kbhit())
         {
-            getch();
+            getchk();
             mpr("Stopping early due to keyboard input.");
             break;
         }
@@ -891,7 +927,7 @@ static void _debug_acquirement_stats(FILE *ostat)
 
         if (!acquirement(type, AQ_WIZMODE, true, &item_index, true)
             || item_index == NON_ITEM
-            || !mitm[item_index].is_valid())
+            || !mitm[item_index].defined())
         {
             mpr("Acquirement failed, stopping early.");
             break;
@@ -974,8 +1010,8 @@ static void _debug_acquirement_stats(FILE *ostat)
     fprintf(ostat, "%s the %s, Level %d %s %s%s\n\n",
             you.your_name.c_str(), player_title().c_str(),
             you.experience_level,
-            species_name(you.species, you.experience_level).c_str(),
-            you.class_name, godname.c_str());
+            species_name(you.species).c_str(),
+            you.class_name.c_str(), godname.c_str());
 
     // Print player equipment.
     const int e_order[] =
@@ -1198,12 +1234,12 @@ static void _debug_acquirement_stats(FILE *ostat)
         {
             const int mannum = subtype_quants[BOOK_MANUAL];
             fprintf(ostat, "\nManuals:\n");
-            for (int i = SK_FIGHTING; i <= SK_EVOCATIONS; ++i)
+            for (int i = SK_FIRST_SKILL; i <= SK_LAST_SKILL; ++i)
             {
                 const int k = 200 + i;
                 if (subtype_quants[k] > 0)
                 {
-                    fprintf(ostat, "%17s: %5.2f\n", skill_name(i),
+                    fprintf(ostat, "%17s: %5.2f\n", skill_name((skill_type)i),
                             100.0 * (float) subtype_quants[k] / (float) mannum);
                 }
             }
@@ -1232,7 +1268,7 @@ static void _debug_acquirement_stats(FILE *ostat)
         item.sub_type = i;
         std::string name = item.name(desc, terse, true);
 
-        max_width = std::max(max_width, (int) name.length());
+        max_width = std::max(max_width, strwidth(name));
     }
 
     // Now output the sub types.
@@ -1260,12 +1296,12 @@ static void _debug_acquirement_stats(FILE *ostat)
 
 static void _debug_rap_stats(FILE *ostat)
 {
-    int i = prompt_invent_item( "Generate randart stats on which item?",
-                                MT_INVLIST, -1 );
+    int i = prompt_invent_item("Generate randart stats on which item?",
+                                MT_INVLIST, -1);
 
     if (i == PROMPT_ABORT)
     {
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         return;
     }
 
@@ -1344,7 +1380,7 @@ static void _debug_rap_stats(FILE *ostat)
     {
         if (kbhit())
         {
-            getch();
+            getchk();
             mpr("Stopping early due to keyboard input.");
             break;
         }
@@ -1353,7 +1389,7 @@ static void _debug_rap_stats(FILE *ostat)
 
         // Generate proprt once and hand it off to randart_is_bad(),
         // so that randart_is_bad() doesn't generate it a second time.
-        artefact_wpn_properties( item, proprt );
+        artefact_wpn_properties(item, proprt);
         if (randart_is_bad(item, proprt))
         {
             bad_randarts++;
@@ -1480,7 +1516,7 @@ static void _debug_rap_stats(FILE *ostat)
     mpr("Results written into 'items.stat'.");
 }
 
-void debug_item_statistics( void )
+void debug_item_statistics(void)
 {
     FILE *ostat = fopen("items.stat", "a");
 
@@ -1492,13 +1528,13 @@ void debug_item_statistics( void )
 
     mpr("Generate stats for: [a] acquirement [b] randart properties");
 
-    const int keyin = tolower( get_ch() );
-    switch ( keyin )
+    const int keyin = tolower(get_ch());
+    switch (keyin)
     {
     case 'a': _debug_acquirement_stats(ostat); break;
     case 'b': _debug_rap_stats(ostat);
     default:
-        canned_msg( MSG_OK );
+        canned_msg(MSG_OK);
         break;
     }
 
@@ -1519,12 +1555,12 @@ void wizard_draw_card()
     lowercase(wanted);
 
     bool found_card = false;
-    for ( int i = 0; i < NUM_CARDS; ++i )
+    for (int i = 0; i < NUM_CARDS; ++i)
     {
         const card_type c = static_cast<card_type>(i);
         std::string card = card_name(c);
         lowercase(card);
-        if ( card.find(wanted) != std::string::npos )
+        if (card.find(wanted) != std::string::npos)
         {
             card_effect(c, DECK_RARITY_LEGENDARY);
             found_card = true;
